/**
 * Loads:
 *   diigo.prefs.js
 *   diigo.urls.js
 *   diigo.handlers.js
 */

var diigo = {
	utils: window.diigoUtil,
	/**
	 * Make or return a namespace object and extend it with properties
	 * e.g. namespace('level1.level2', {cool: true})
	 */
	namespace: function(ns, properties) {
		var root = this;
		
		var parts = ns.split('.');
		var o = root; 
		for (var i = 0, v; v = parts[i], i < parts.length; i++) {
			o[v] = o[v] || {};
			o = o[v];
		}
		if (properties)
			for (var k in properties) {
				o[k] = properties[k];
			}
	    return o;
	},
	
	/*_scopes: {},
	
	peekScope: function(scopeName, what) {
		return this._scopes[scopeName](what);
	},
	
	//eval the return value to register c

	registerScope: function(scopeName) {
		var code = this.utils.string.showTpl(
			'diigo._scopes["#{0}"] = function(s) {return eval(s);};',
			[scopeName]
		);
		
		//alert(code);
		return code;
	},*/
	
	/**
	 * eval the return value of this method to import expression in a scope as local variables of the evaling scope
	 * @param {Array} what [name, {name: 'newName'}, ...]
	 */
	importFromScope: function(scopeName, what, later/* = true*/) {
		later = later === undefined ? true : later;
		var U = this.utils, S = U.string;

		var declaration = '', assignment = '';
		var tpl = S.showTpl('\\#{0} = window.diigo.peekScope("#{0}", "\\#{1}");\n', [scopeName]);
		
		what.forEach(function(name) {
			if (typeof name == 'string') {
				declaration += 'var #N;\n'.replace('#N', name);
				assignment += S.showTpl(tpl, [name, name]); 
			} else {
				U.each(name, function(newName, name) {
					declaration += 'var #N;\n'.replace('#N', newName);
					assignment += S.showTpl(tpl, [newName, name]);
				});
			}
		});
		
		if (later)
			code = declaration + 
			'window.diigo.registerRunLater(function () {#BODY});'
			.replace('#BODY', 
				assignment
			);
		else
			code = declaration + assignment;
		
		return code;
	},
	
	/**
	 * A list of functions to be called "later"
	 */
	_runLater: [],
	registerRunLater: function(f) {this._runLater.push(f);},
	
	/**
	 * Invokes all functions in the _runLater list.
	 * Call this method when "later" comes.
	 */
	runRunLater: function() {
		this._runLater.forEach(function(f) {f();});
	},
	
	/**
	 * eval the return value of this method to import properties of D as local variables of the evaling scope
	 * 
	 * @param {String} objStr
	 * @param {Array} what [name, {name: 'newName'}, ...]
	 * @return {String} code to eval
	 */
	importFromObject: function(objStr, what, later/* = true*/) {
		later = later === undefined ? true : later;
		var U = this.utils, S = U.string;

		var declaration = '', assignment = '';
		var tpl = S.showTpl('\\#{0} = #{0}["\\#{1}"];\n', [objStr]);
		
		what.forEach(function(name) {
			if (typeof name == 'string') {
				declaration += 'var #N;\n'.replace('#N', name);
				assignment += S.showTpl(tpl, [name, name]); 
			} else {
				U.each(name, function(newName, name) {
					declaration += 'var #N;\n'.replace('#N', newName);
					assignment += S.showTpl(tpl, [newName, name]);
				});
			}
		});
		
		if (later)
			code = declaration + 
			'window.diigo.registerRunLater(function () {#BODY});'
			.replace('#BODY', 
				assignment
			);
		else
			code = declaration + assignment;
		
		return code;
	},
	importCommons: function() {
		var code=this.importFromObject('window', [{diigo: 'D'}], false) +
		this.importFromObject('window.diigo', [{utils: 'Utils'}], false) +
		this.importFromObject('window.diigo.utils', ['Class', {'string': 'S'}], false) +
		this.importFromObject('window.diigo', 'Bookmark Annotation TextHighlight ImageHighlight FloatNote VideoClip FlashClip Clip Comment PageComment InlineComment Toolbar Sidebar Csm DlgAnnotation WebAPI PageContext DlgIC'.split(' ')
			.concat([])
		);
		return code;
	}
	
};

diigo.version = DIIGO_VERSION;
diigo.extensionId = DIIGO_EXTENSION_ID;
diigo.isMoa = DIIGO_IS_AMO;
diigo.id = function(id) {return document.getElementById(id);};

/*
 * require utils
 */
(function() {
	var D = diigo;
	var Utils = D.utils;
	

	var Toolbar;
		var Sidebar;
		var WebAPI;
		var PageContext;
		window.diigo.registerRunLater(function () {
		Toolbar = window.diigo["Toolbar"];
		Sidebar = window.diigo["Sidebar"];
		WebAPI = window.diigo["WebAPI"];
		PageContext = window.diigo["PageContext"];
		});
	
	D.j = window.diigoJQuery3;
	var $ = D.$ = window.$.noConflict(true);//diigoJQuery3(window);
	
	D.EventDispatcher = {
		mixin: function(o) {
			o._events = {};
			o._supressEvents = false;
			Utils.extend(o, this.methods);
		},
		
		methods: {
			addEventListener: function(eventName, listener){
				if (!this._events[eventName]) 
					this._events[eventName] = [];
				
				var listenerList = this._events[eventName];
				if (listenerList.indexOf(listener) == -1) 
					listenerList.push(listener);
			},
			
			removeEventListener: function(eventName, listener){
				var listenerList = this._events[eventName];
				if (listenerList) {
					if (arguments.length == 2) {
						//listener provided
						var i = listenerList.indexOf(listener);
						if (i > -1) {
							listenerList.splice(i);
						}
					}
					else {
						//listener provided, remove all
						delete this._events[eventName];
					}
				}
			},
			
			fireEvent: function(eventName, paramArray){
				if (this._supressEvents) 
					return;
				
				Utils.dump('[event] firing', eventName);
				var listenerList = this._events[eventName];
				if (listenerList) {
					for (var i = 0, listener, len = listenerList.length; listener = listenerList[i], i < len; i++) {
						(typeof listener == 'function' ? listener : listener['on' + eventName]).apply(listener, paramArray);
					}
				}
			}
		}
	};
	
	D.EventDispatcher.mixin(D);

	
	Utils.extend(D, {
		PRIVACY_MODE_PUBLIC: 0,
		PRIVACY_MODE_PRIVATE: 2,
		PRIVACY_MODE_GROUP: 3,
		
		BOOKMARK_TYPE_WEBPAGE: 1,
	    BOOKMARK_TYPE_IMAGE: 2,
	    BOOKMARK_TYPE_FLASH: 3,
	    BOOKMARK_TYPE_VIDEO: 4,
	    BOOKMARK_TYPE_LIST: 10,
		
		ANNOTATION_TYPE_TEXT: 0,
	    ANNOTATION_TYPE_IMAGE: 1,
	    ANNOTATION_TYPE_FLOATNOTE: 2,
	    ANNOTATION_TYPE_FLASH: 3,
	    ANNOTATION_TYPE_VIDEO: 4,
	    ANNOTATION_TYPE_CLIP: 5,
		ANNOTATION_TYPE_ICON: 9,
			
		TOOLBAR_HIDDEN_ITEMS: 'diigotb-tb-searchBox,diigotb-tb-search',
		
		/**
		 * runtime information
		 */
		env: {
			signedIn: false,
			user: '',
			userLevel: 0,
			myProfile: {},
			
			myTags: [],
			myList: [], /*[{LIST_RESP}, ...]*/
			outliners: [], /*[{LIST_RESP}, ...]*/
			myArchivedBookmarkLists: [], /*[{LIST_RESP}, ...]*/
			myContactLists: [], /*[{LIST_RESP}, ...]*/
			recentlySavedBookmarks: [],
			unreadBookmarks: [],
			recentlyReadBookmarks: [],
			myCustomizedSearch:[{i:"web-0",name:"Web",engines:[{i:0,name:"Diigo & Google",url:"https://www.diigo.com/search/g?cx=!partner-pub-7625644023173335%3Agqffh9-8lvc&cof=FORID%3A11&ie=UTF-8&q=diigosearch&sa=Search"},{i:0,name:"Diigo & Bing",url:"https://www.diigo.com/search/bing?q=diigosearch"}]}],
			
			//temporarily for share window
			myContacts: [],
			myBlackList: [],
			
			myFriends: [],
			
			myGroups: [], /*[{name: 'g_name', displayName: 'display_name'}, ...]*/
			myArchivedGroups: [], /*[{name: 'g_name', displayName: 'display_name'}, ...]*/
			
			/**
			 * Group tags dictionary (what bad diction!)
			 * {'groupName': ['tag', ...], ...}
			 */
			groupTagsDict: {},
			
			otherServices: [], //not used
			lastAdKeyword: "",
			
			twitterToken:{},

			capturedImageNo: 0,
			
			silent: false,
			bShrinkShare: true,
			
			permissions:{premium:false,image:true,capture:true,snapshot:true}, 
			
			/**
			 * Clear all user related data
			 */
			reset: function() {
				this.myTags.length =
				this.myList.length = 
				this.outliners.length = 
				this.myContactLists.length =
				this.myGroups.length =  
				this.recentlySavedBookmarks.length =
				this.unreadBookmarks.length =
				this.recentlyReadBookmarks.length = 
				//this.myCustomizedSearch.length = 
				this.myContacts.length = 
				this.myFriends.length = 0;
				
				this.permissions={premium:false,image:true,capture:true,snapshot:true};
				this.myProfile = {};
				this.groupTagsDict = {};
				this.myBlackList = [];
				this.myArchivedGroups = [];
				this.myArchivedBookmarkLists = [];
				this.myCustomizedSearch = [{i:"web-0",name:"Web",engines:[{i:0,name:"Diigo & Google",url:"https://www.diigo.com/search/g?cx=!partner-pub-7625644023173335%3Agqffh9-8lvc&cof=FORID%3A11&ie=UTF-8&q=diigosearch&sa=Search"},{i:0,name:"Diigo & Bing",url:"https://www.diigo.com/search/bing?q=diigosearch"}]}];
			},
			
			/**
			 * Add (unique) tags to myTags
			 */
			addToMyTags: function(tags) {
				var myTags = this.myTags;
				tags.forEach(function(tag) {
					if (myTags.indexOf(tag) == -1)
						myTags.push(tag);
				});
			},
			
			addRecentBookmark: function(bm) {
				var i = Utils.array.findIndex(this.recentlySavedBookmarks, function(v) {return v.url == bm.url;});
				
				var data = {
					datetime: bm.datatime,
					mode: bm.mode,
					title: (bm.getTitle ? bm.getTitle() : bm.title) || bm.url,
					url: bm.url,
					user: bm.user,
					userOnline: bm.userOnline
				};
					
				if (i == -1) {
					this.recentlySavedBookmarks.unshift(data);
				} else {
					this.recentlySavedBookmarks[i] = data;
				}
				
				Sidebar.refresh('myBookmarks');
			},
			
			removeRecentBookmark: function(bm) {
				var i = Utils.array.findIndex(this.recentlySavedBookmarks, function(v) {return v.url == bm.url;});
				if (i > -1) {
					this.recentlySavedBookmarks.splice(i, 1);
					Sidebar.refresh('myBookmarks');
				}
			},
			
			addUnreadBookmark: function(bm) {
				D.utils.dump('[env] add unread bookmark', bm);
				var i = Utils.array.findIndex(this.unreadBookmarks, function(v) {return v.url == bm.url;});
				
				var data = {
					datetime: bm.datatime,
					mode: bm.mode,
					title: (bm.getTitle ? bm.getTitle() : bm.title) || bm.url,
					url: bm.url,
					user: bm.user,
					userOnline: bm.userOnline
				};
					
				if (i == -1) {
					this.unreadBookmarks.unshift(data);
				} else {
					this.unreadBookmarks[i] = data;
				}
				
				Sidebar.refresh('myBookmarks');
			},
			
			/**
			 * 
			 * @param {Object} bm
			 * @param {Boolean} whether to move the bm object to D.env.recentlyReadBookmarks array
			 */
			removeUnreadBookmark: function(bm, addToRecentlyRead) {
				D.utils.dump('[env] remove unread bookmark', bm);				
				var i = Utils.array.findIndex(this.unreadBookmarks, function(v) {return v.url == bm.url;});
				if (i > -1) {
					var bm = this.unreadBookmarks.splice(i, 1)[0];
					if (addToRecentlyRead) {
						this.recentlyReadBookmarks.unshift(bm);
					}
					//Sidebar.refresh('myBookmarks');
				}
			}
		}
	});
	
	//run time information
	D.runtime = {
		//currently focused context (for highight)
		//this is needed because getCurrentContext doesn't work for frames
		//should be set after each highlightable selection
		_focusedContext: null,
		
		_focusedNode: null,
		
		// A special url triggers toolbar uninstallation. 
		// To prevent unintentioned visit to that url from accidentally uninstall toobar,
		// we use a flag to identify whether the user has clicked uninstall
		uninstalling: false
		
		//inClipMode: false,
		
		//clip_currentNode: null
	};
	
	//parsing tag input
	Utils.extend(D, {
		/**
		 * Quotes a single tag
		 */
		quoteTag: function (tag) {
			tag = tag
			.replace(/"/g, "'")
			.replace(/\s+/g, ' ')
			.replace(/^\s+|\s+$/g, '');
			
			if (tag.match(/\s+|,/)) {
				tag = '"' + tag + '"';
			}
			return tag;
		},
		
		//input text => tag array
		//tags input: comma or space separated. double quotes allowed
		//merge quoted if necessary
		//TEST: 'a b c' => ['a', 'b', 'c']
		//TEST: 'a,b,c' => ['a', 'b', 'c']
		//TEST: 'a b,c' => ['a', 'b', 'c']
		//TEST: 'a "b,c"' => ['a', 'b,c']
		//TEST: '"a b" c' => ['a b', 'c']
		//TEST: '"a b c' => ['a b c']
		parseTags: function (strTags, validate) {
			var stack = [], tags = [];
			var begin_delimiter = false;
			
			for (var i = 0, len = strTags.length, c; c = strTags.charAt(i), i < len; i++) {
				if (c == '"') {
					if (!begin_delimiter) 
						begin_delimiter = true;
					else {
						begin_delimiter = false;
						clearStack();
					}
				} else {
					if (begin_delimiter) {
						stack.push(c);
					} else {
						if (/\s/.test(c) || c == ',') 
							clearStack();
						else {
							stack.push(c);
						}
					}
				}
			}
			
			clearStack(); //if there is unclosed delimiter
			
			if (validate) 
				tags = Utils.array.unique(tags.filter(function(t) {return !/^\s*$/.test(t);})).map(function(t) {return Utils.string.trim2(t);});
			return tags;
			
			function clearStack() {
				if (stack.length > 0) {
			  	tags.push(stack.join(''));
			  	stack.length = 0;
			  }
			}
			
		},
		
		/**
		 * tag array => input text
		 */
		unparseTags: function (tagArray, joinBy) {
			joinBy = joinBy || ' '
			return tagArray.map(function(t) {return D.quoteTag(t);}).join(joinBy);
		}
	});
	
	Utils.extend(D, {
		/**
		 * Copies html to clipboard
		 * http://developer.mozilla.org/en/docs/Using_the_Clipboard
		 * 
		 * @param {String} html the html presentation
		 * @param {String} text the plain text presentation
		 */
		copy: function (html, text, sourceWindow) {
			//Utils.dump('[Copy]', html, text);
			try {
				var clipid = Components.interfaces.nsIClipboard;
				var clip   = Components.classes["@mozilla.org/widget/clipboard;1"].getService(clipid);

				function Transferable(source) {
					const nsTransferable = Components.Constructor("@mozilla.org/widget/transferable;1", "nsITransferable");
				    var res = nsTransferable();
				    if ('init' in res) {
				        // When passed a Window object, find a suitable privacy context for it.
				        if (source instanceof Ci.nsIDOMWindow)
				            // Note: in Gecko versions >16, you can import the PrivateBrowsingUtils.jsm module
				            // and use PrivateBrowsingUtils.privacyContextFromWindow(sourceWindow) instead
				            source = source.QueryInterface(Ci.nsIInterfaceRequestor)
				                           .getInterface(Ci.nsIWebNavigation);

				        res.init(source);
				    }
				    return res;
				}
				// get some useful components
				var trans = Transferable();
				  
				clip.emptyClipboard(clipid.kGlobalClipboard);
		
				this._setTransferData(trans, "text/unicode", text);
				this._setTransferData(trans, "text/html", html);
				
				clip.setData(trans, null, clipid.kGlobalClipboard);
			} catch (e) {
				Utils.dump('[Copy]', e);
			}
			return true;
		},
		
		_setTransferData: function(trans, flavor, text) {
			var textData = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
			textData.data = text;
		
			trans.addDataFlavor(flavor);
			trans.setTransferData(flavor, textData, text.length * 2);
		}
	});
	
	Utils.extend(D, {
		getCurrentContext: function() {
			return content.document && content.document.__diigo;
		},
		
		getFocusedFrame: function() {
			//TODO can we just use document.commandDispatcher.focusedWindow?
			// see browser.js
			var fr = D.runtime._focusedContext;
			fr = fr && fr.win;
			if (!fr) {
				D.utils.dump('getFocusedFrame not found');
			}
			return fr || content;
		},
		
		isCurrentLocationBookmarked: function() {
			var context = this.getCurrentContext();
			return context && context.bookmark.saved;
		},
		
		isCurrentLocationBookmarkable: function() {
			return content.document && this.isDocumentDiigoable(content.document)
				&&!this.isDocumentAnnotated(content.document.URL);
		},
		
		isCurrentLocationCommented: function() {
			var context = this.getCurrentContext();
			return context && (context.bookmark.annotated || context.pageComments.length > 0 || context.annotations.length > 0);
		},
		
		/**
		 * Update whether current location is bookmarkable
		 */
		updateBookmarkableState: function() {
			var cmdBookmark = $('#cmd_diigotb-bookmark');
			var cmdUnbookmark = $('#cmd_diigotb-unbookmark');
			
			var bookmarked = this.isCurrentLocationBookmarked();
			var bookmarkable = this.isCurrentLocationBookmarkable();
			
			cmdUnbookmark.attr({
				disabled: !bookmarked || !bookmarkable,
				hidden: bookmarked || !bookmarkable
			});	
			
			cmdBookmark.attr({
				disabled: !bookmarkable
			});
			
			Toolbar.refresh('bookmark');
			if(bookmarkable&&D.env.permissions.capture==false){
				Toolbar.refresh('premium');
			}
		},
		
		/**
		 * Tells whether a document can be bookmarked by diigo
		 * @param {Document} doc
		 * @return {Boolean}
		 */
		isDocumentDiigoable: function (doc) {
			return doc && 
			this.isURIDiigoable(doc.URL) &&
			//top doc must be diigoable too
			this.isURIDiigoable(doc.defaultView.top.document.URL);
		},

		isDocumentAnnotated: function (url) {
			if (url.indexOf(D.urls.getDiigoURL() + '/annotated') >= 0||
				url.indexOf(D.urls.getDiigoURL() + '/cached') >= 0||
				url.indexOf(D.urls.getDiigoURL() + '/item/pdf/') >= 0||
					url.indexOf(D.urls.getDiigoURL() + '/bookmark') >= 0) {
				return true;
			}
			return false;
		},
		
		isSlidesDiigoable: function (doc) {
			return doc && (
			this.isSlidesUrl(doc.URL)||
			//top doc must be diigoable too
			this.isSlidesUrl(doc.defaultView.top.document.URL));
		},
		/**
		 * 
		 */
		isDocumentHighlightable: function(doc) {


			if (!this.isDocumentDiigoable(doc)||this.isSlidesDiigoable(doc)){ 
				return false;
			}
			
			//content type must be html/xhtml/plain text 
			//(text/html application/xhtml+xml text/plain)
			var ct = doc.contentType.toLowerCase();
			if (ct.indexOf('html') == -1 && ct != 'text/plain'){
				return false;
			}
			
			if (doc.designMode.toLowerCase() == 'on') {
				return false;
			}
			
			//if it's a frame
			if (doc.defaultView.top != doc.defaultView) {
				try{
				var topDomain = doc.defaultView.top.location.hostname.toLowerCase();
				var domain = doc.defaultView.location.hostname.toLowerCase();
				//compare domain, frame and to doc must be of the same base domain
				
				if (domain != topDomain) {
					var p = Utils.array.reverse(topDomain.split('.')), 
							pp = Utils.array.reverse(domain.split('.'));
							
					if (Number(p[0]).toString() == p[0]) {
						//if the last part is a number, the host name must be an IP address
						//we requre stric equality
						if (domain != topDomain) return false;	
					} else {
						var same = [];
						for (var i = 0; i < p.length; i++) {
							if (!pp[i] || p[i] != pp[i]) break;
							same.push(p[i]);
						}
						
						if (same.length < 2) return false;
						else if (same.length == 2) {
							if ('aero asia biz cat com coop edu gov info int jobs mil mobi museum name net org pro tel travel'.split(' ').indexOf(same[1]) != -1) return false;
						}
						
					}
				}
				}catch(e){
				}
				
			}
			
			if (doc.URL.match(/^https?:\/\/mail\.google\.com\/mail\//)) return false;		
			return true;
		},
		
		/**
		 * This actually means isURIBookmarkable
		 */
		isURIDiigoable: function (uri) {
			return uri && !!uri.match(/^(http|https|ftp|ftps)/);// && !uri.match(/^http:\/\/slides\.diigo\.com\/(diigo|list)\//i);
		},
		isLocalPDFReader: function (uri) {
			return uri && (!!uri.match(/^chrome(.*)pdf\/viewer\.html\?file=(.*)/i));
		},
		getOriginalPDFURL: function (url) {
			if(D.isLocalPDFReader(url)){
				var pos=url.indexOf("file=http");
				if(pos>0){
					return url.substr(pos+5);
				}
			}
			return url;
		},
		isURIUploadable: function (uri) {
			return uri && !uri.match(/^https/);// && !uri.match(/^http:\/\/slides\.diigo\.com\/(diigo|list)\//i);
		},

		isSlidesUrl: function (uri) {
			return typeof uri=='string' && !!uri.match(/^http:\/\/slides\.diigo\.com\/(diigo|list)\//i);
		},
		
		//tells whether a document is (x)html
		isHTMLDocument: function(doc) {
			return doc.contentType.toLowerCase().indexOf('html') > -1;
		},

		isSelectedURL: function(url) {
			var mapped=[];
			try{
			var useit=D.prefs.val(D.env.user+'.useBlacklist');
			if(!useit||useit=="")return false;
			url=url.toLowerCase();
			var nUri=D.utils.firefox.makeUri(url);
			mapped=D.env.myBlackList.filter(function(g){
				try{
				gUri=D.utils.firefox.makeUri(g);			
				return url==g||nUri.equals(gUri);
				}catch(e){
					//alert(e);
					return false;
				}
			});
			}catch(e){
				//alert(e);
			}
			
			return mapped.length>0;
		},
				
		isHomePage: function(url) {
			return false;
			//path name
			/*var p = new Poly9.URLParser(url.toLowerCase()).getPathname();
			if (!p || p == '/') return true;
			var m = p.match(/^\/([^\.]+)\.(.*)$/);
			return m && m.length == 3 
				? 
					['default', 'index', 'home'].indexOf(m[1]) > -1 &&
					['htm', 'html', 'shtml', 'php', 'jsp', 'asp', 'aspx', 'cfm'].indexOf(m[2]) > -1 
				: 
					false;
			return true;*/
		},
		isPDFPage: function(bm) {
			return bm.contentType=='application/pdf';
		},
		isWORDPage: function(bm) {
			return bm.contentType=='application/msword';
		},		
		isEXCELPage: function(bm) {
			return bm.contentType=='application/vnd.ms-excel';
		},
		isPPTPage: function(bm) {
			return bm.contentType=='application/vnd.ms-powerpoint';
		},
		isTXTPage: function(bm) {
			return bm.contentType=='text/plain';
		},		
		isNormalPage: function(bm) {
			return bm.contentType=='text/html'||bm.contentType=='text/xml';
		},
		isImagePage: function(bm) {
			return bm.contentType.indexOf('image')>=0;
		},	
		isBookmarkInUnreadList: function(bm) {
			var a = D.env.unreadBookmarks;
			for ( var i = a.length, b; i; ){
				b = a[ --i ];
				if (bm.url == b.url || (bm.b_id && b.b_id == bm.b_id)) return true;
			}
			
			return false;
		},
		
		/**
		 * Tells whether 2 urls are "the same"
		 * @return {Boolean}
		 */
		urlEqual: function(url1, url2) {
			//remove hash and trailing slash
			url1 = D.utils.string.trim(url1.toLowerCase()).replace(/#.*$/, '').replace(/\/+$/, '');
			url2 = D.utils.string.trim(url2.toLowerCase()).replace(/#.*$/, '').replace(/\/+$/, '');
			return url1 == url2;
		}
		
//		,
//		
//		parseAnnotatedLinkParams: function(url) {
//			var params = Utils.string.parseQuery(url);
//			if (params.om == undefined) params.om = 0;
//			return params;
//		}
		
	});
	
	Utils.extend(D, {
		/**
		 * Checks cookies to determine
		 * Call me when onload, after sign in/out
		 */
		checkSignIn: function() {
			try{
				//Utils.dump('checking sign in... old user: ', D.env.user);
				var oldUser = D.env.user;
				
				var domain = D.urls.getCookieDomain();
				var cookie = Utils.cookie.getCookie(domain, 'diigoandlogincookie'); //good name, lucky life
				if (cookie) {
					var values = cookie.value.split('-.-');
					var user = values[1], level = values[3];
					D.env.user = user || '';
					D.env.userLevel = Number(level || 0);
					D.env.signedIn = !!user;
				} else {
					D.env.user = '';
					D.env.userLevel = 0;
					D.env.signedIn = false;
				}
				//Utils.dump('signed in: ', D.env.signedIn, 'user: ', D.env.user);
				if (user) {
					if(user != oldUser){
						D.onSignIn();
					}
				} else {
					if(oldUser){
						D.onSignOut();
					}
				}
			}catch(e){
				throw(e);
			}
			return D.env.signedIn;
		},
		
		/**
		 * Tells whether the user has signed in
		 */
		hasSignedIn: function () {
			return D.env.user && D.env.signedIn;
		},
		
		onSignIn: function () {
			Utils.dump('onSignIn');
			D.env.reset();
			
			Sidebar.onSignIn();
			
			D.DlgIC.RefreshUser();
			
			if (/^mac/i.test(navigator.platform) && D.ff3up) ff3ForMacPleaseDontCrash();
			$('#diigotb-availableAfterSignedIn').attr({disabled: false, hidden: false});
			$('#diigotb-availableBeforeSignedIn').attr({disabled: true, hidden: true});
			
			$('#cmd_diigotb-signOut').attr({label: 'Sign out (' + D.env.user + ')'});

			Toolbar.onSignIn();
			
			WebAPI.loadMyStuff();
			WebAPI.loadMyFriends();
			D.MessageAlert.refresh();
			D.QAF.checkEnabled();
			var szlist=D.prefs.val(D.env.user+'.Blacklist');
			if(szlist.length<=0){
				szlist="[]";
				D.prefs.val(D.env.user+'.Blacklist',szlist);
			}
			var useBlank=D.prefs.val(D.env.user+'.useBlacklist');
			if(useBlank==""){
				D.prefs.val(D.env.user+'.useBlacklist',true);
			}
			D.env.myBlackList=Utils.json.parseJSON(szlist);

		},
		
		onSignOut: function () {
			Utils.dump('onSignOut');
			
			D.env.reset();
			
			//clear sign in data
			D.env.user = '';
			D.env.signedIn = false;
			D.env.userLevel = 0;
			
			$('#diigotb-availableAfterSignedIn').attr({disabled: true, hidden: true});
			$('#diigotb-availableBeforeSignedIn').attr({disabled: false, hidden: false});
			
			//do not leak last user name
			$('#cmd_diigotb-signOut').attr({label: ''});
			
			Toolbar.onSignOut();
			Sidebar.onSignOut();
			D.QAF.checkEnabled();
			//TODO recursively unpaint all annotations and clear PageContext for each window (frame)
		},
		
		publicAnnotationsAllowed: function(url) {
			return /*!D.isHomePage(url) &&*/ D.env.userLevel > 0;
		}
	});
	
	Utils.extend(D, {
		expired: false,
		
		/**
		 * set by webapi
		 * do not use this unless necessory because this circumvents firefox secure update
		 */
		_updateUrl: null,
		
		checkForUpdate: function() {
			D.WebAPI.checkForUpdate();
		},
		
		isFlock: function() {
			return Utils.firefox.isApp('flock');
		},
		
		expire: function(msg) {
			this.expired = true;
			Toolbar.refresh('expire');
			alert(D.strings.getString('expired'));
		}
	});
	
	/**
	 * Displays notification messages
	 */
	Utils.extend(D, {
		importing:false,
		_hideTimer: null,
		_intervalTime:null,
		_importTime:null,
		
		_doNotify: function(msg, msgType, timeout) {
			var z = this,j;
			
			timeout = timeout || 3000;
			var className = ({ok: 'success', tip: 'tip', error: 'fail'})[msgType];
			
			//var referenceBox = gBrowser.mCurrentBrowser.boxObject;
			
			if(D.ff3up)
				j = $('#diigotb-msg');
			else
				j = $('div#diigotb-msg');
			j.attr({'class': className})
					.empty()
					.css({top: 0, left: 0});
			
			//var r = document.createRange();
			//r.selectNode(j[0]);
			//var f = r.createContextualFragment(msg);//!!!!!!!conflict with tab mix plus
			j.text(msg);
			
			j.show();
			
			clearTimeout(z._hideTimer);
			z._hideTimer = null;
			
			z._hideTimer = setTimeout(function() {
				z._hideTimer = null;
				j.hide();
			}, timeout);
			
		},
		
		notify: function(msg,timeout) {
			this._doNotify(msg, 'tip',timeout);
		},
		
		notifyOK: function(msg,timeout) {
			this._doNotify(msg, 'ok',timeout);
		},
		
		notifyError: function(msg,timeout) {
			this._doNotify(msg, 'error',timeout);
		},

		notifyImport: function(msg) {
			var z = this,j;			
			
			if(D.ff3up)
				j = $('#diigotb-msg');
			else
				j = $('div#diigotb-msg');
			j.attr({'class': 'tip'})
					.empty()
					.css({top: 0, left: 0});
			
			//var r = document.createRange();
			//r.selectNode(j[0]);
			//var f = r.createContextualFragment(msg);
			j.text(msg);
			
			j.show();
			
			clearInterval(z._importTime);
			z._importTime = null;
			
			z._importTime=setInterval(function(){
				try{
				if(!D.importing){
					j.hide();
					clearInterval(z._importTime); 
				}
				}catch(e){}
			},3000)
				
		}
	});
	
	
	var NOTIFY_STATE_DOCUMENT =
	  Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT;
	var NOTIFY_STATE_ALL =
	  Components.interfaces.nsIWebProgress.NOTIFY_STATE_ALL;
	var STATE_IS_DOCUMENT =
	  Components.interfaces.nsIWebProgressListener.STATE_IS_DOCUMENT;
	var STATE_START =
	  Components.interfaces.nsIWebProgressListener.STATE_START;
	var STATE_STOP =
	  Components.interfaces.nsIWebProgressListener.STATE_STOP;
	var STATE_TRANSFERRING =
	  Components.interfaces.nsIWebProgressListener.STATE_TRANSFERRING;
	var STATE_REDIRECTING = 
	  Components.interfaces.nsIWebProgressListener.STATE_REDIRECTING; 
	  
	var progressListener = {
	  onStateChange: function(aProgress, aRequest, aFlag, aStatus) {
	  	if (!D.hasSignedIn()) return 0;
	  	
	  	
	  	if (aFlag & STATE_START && 
	  		aFlag & Components.interfaces.nsIWebProgressListener.STATE_IS_NETWORK &&
	  		aRequest && 
	  		aRequest.name)
	  	{
	  		D.EventMachine.onRequesting(aRequest.name);
	  	}

	  },
	  
	  onLocationChange: function(aProgress, aRequest, aURI) {
	  		onLocationChange(aURI);
	  },
	  
	  onProgressChange: function(a,b,c,d,e,f) {
	  },
	  onStatusChange: function(a,b,c,d) {
	  },
	  onSecurityChange: function(a,b,c) {},
	
	  /* 
	    This is not nsIWebProgressListener method,
	    just killing an error in tabbrowser.xml
	    Maybe a bug.
	  */
	  onLinkIconAvailable: function(a) {}
	};

/*	var PluginWatcherObserver = {
	   observe: function(subject, topic, data) {	
	   
		 if (topic == "experimental-notify-plugin-call"){
		   //This gets executed each time a runtime notification arrives
		   // --YOUR CODE GOES HERE--        
		   //alert(subject+" : "+topic+" : "+data);
		 }
	  
	 
	   },

	   register: function() {
		 if (this.registered == false) { 

		   var observerService = Components.classes["@mozilla.org/observer-service;1"]
										   .getService(Components.interfaces.nsIObserverService);
		   observerService.addObserver(this, "experimental-notify-plugin-call", false);
		   this.registered = true;
		 }
	   },

	   unregister: function() {
		 if (this.registered == true) {
		   var observerService = Components.classes["@mozilla.org/observer-service;1"]
										   .getService(Components.interfaces.nsIObserverService);
		   observerService.removeObserver(this, "experimental-notify-plugin-call");
		   this.registered = false;
		 }
	   }
	 };*/

	
	/**
	 * Initializes the extension
	 */
	function init() {
		//delay the execution, wait for Firefox to finish it's fancy initialization
		//this should avoid weird problems such as breaking plurk, downlaod statusbar
		setTimeout(_init, 300);
//		_init();
	}
	
	//setting hidden = false on some elements crashes ff3 for mac. however this is a way to cure (e.hidden=true;e.hidden=false;)
	//this is totally insane. See the dev notes Diigo Toolbar 3.1.5.7 crashes Firefox 3 for Mac when signing in to diigo.com
	function ff3ForMacPleaseDontCrash() {
		var j = $('menu[observes=diigotb-availableAfterSignedIn],menu>observes[element=diigotb-availableAfterSignedIn]');
		j.get().forEach(function(e) {e.hidden=true;e.hidden=false;}); //setting it to true and false again. I'm afraid this is the only way.
	}
	
	function _init() {
		D.ff3up = true;//;
		D.ff4up = D.utils.firefox.ff4OrAbove();
		D.ff31beta = D.utils.firefox.ff31beta();
		
		D.strings = document.getElementById('strings-diigo');
		window.getBrowser().addProgressListener(progressListener);
		
		//so that we can observe the webProgressListener better
		//return;
		
		var appContent = document.getElementById('appcontent');
		appContent.addEventListener('DOMContentLoaded', onPageLoad, true);
		
		//I have seen a page that doesn't trigger DOMContentLoaded
		//http://www.worldofwarcraft.com/index.xml
		//so we need to also listen to load event to catch it
		appContent.addEventListener('load', onPageLoad, true);
		
		appContent.addEventListener('unload', onPageUnload, true);
		
		appContent.addEventListener('mouseup', onPageMouseUp, false);
		appContent.addEventListener('mousedown', onPageMouseDown, false);
		appContent.addEventListener('click', onPageClick, false);
		appContent.addEventListener('mouseover', onPageMouseOver, false);
		appContent.addEventListener('mouseout', onPageMouseOut, false);
		
		function suppressEmptyTooltip(event) {

			var tt = document.tooltipNode;
			if (tt && event.originalTarget.localName == 'tooltip') {
				if (("menuitem" == tt.nodeName || "menu" == tt.nodeName) && tt.getAttribute("tooltiptext") == "") {
					event.preventDefault();
				}
			}
		}

		// A 6-year old mozilla tooltip bug
		// Bug 147670 (tooltips) wrong (last or empty) tooltip text displayed for dropdown list menu items
		// https://bugzilla.mozilla.org/show_bug.cgi?id=147670
		// the workaround is inspired by google toolbar
		window.addEventListener('popupshowing', suppressEmptyTooltip, false);

		//document.getElementById("diigotb-toolbar").addEventListener("DOMAttrModified",onDOMAttrModified,false,true);

		[
			D.prefs,
			D.Csm, 
			D.AnnotationContextMenu, 
			D.RightClickMenu, 
			D.DlgIC, 
			D.Toolbar, 
			D.Chevron, 
			D.ColorfulFind, 
			D.Search, 
			D.Keys, 
			D.Clip,
			D.QuickAccessFilter
		].forEach(function(v) {
			if(v)
				v.init();
		});
		
		//D.checkSignIn();//checked when page is loaded;
		D.checkForUpdate();
		
		//fresh install? show welcome page
		var version = D.prefs.val('version');
		if (version != D.version) {
			//version is different, it is a fresh install
			//go to welcome page
			//setTimeout because firefox overrides tabs later
			//FlashGot also sets timeout
			setTimeout(function() {
				Utils.firefox.openTab(D.urls.getToolbarWelcomeURL());
			}, 700);
			if(!version){
				var hidebutton="diigotb-tb-comment,diigotb-tb-message";
				D.prefs.val("toolbar.showItems.diigotb-tb-highlight",true);
				D.prefs.val("toolbar.showItems.diigotb-tb-comment",false);
				D.prefs.val("toolbar.showItems.diigotb-tb-message",false);
				D.prefs.val("toolbar.showItems.diigotb-tb-share",true);
				D.prefs.val("toolbar.showItems.diigotb-tb-options",true);
				D.prefs.val("toolbar.showItems.diigotb-tb-search",true);
				D.prefs.val("toolbar.showItems.diigotb-tb-searchBox",true);
				D.prefs.val("toolbar.showItems.diigotb-tb-capture",true);
				D.prefs.val("toolbar.showItems.diigotb-tb-upgrade",true);
				if(D.ff3up){
					D.prefs.val("toolbar.showItems.diigotb-tb-quickAccessFilter-item",true);
				}else{
					D.prefs.val("toolbar.showItems.diigotb-tb-quickAccessFilter-item",false);
					hidebutton+=",diigotb-tb-quickAccessFilter-item"
				}
				D.prefs.val("toolbar.hiddenButtons",hidebutton);
				D.prefs.val("csm.enabled", false);
				D.prefs.val("cm.insertAtTop", false);
				D.prefs.val('curSearchEngine',0);
				D.prefs.val('insertSEs',true);
				D.prefs.val('opensearchintab',false);
				D.prefs.val('showimagepanel',true);
				D.prefs.val('bookmark.tags','[]');
				D.prefs.val(D.env.user+'.Blacklist',"[]");
				D.prefs.val(D.env.user+'.useBlacklist',true);
			}
			
			D.prefs.val('version', D.version);
			
			//set server to www if previous version is prior to 3.1.5.3 because there were users having preview for the value
			if (D.prefs.val('server') != 'cn' && (!version || Components.classes['@mozilla.org/xpcom/version-comparator;1'].createInstance(Ci.nsIVersionComparator).compare(version, '4.1.0.14') < 0))
				D.prefs.val('server', 'www');
				
			//if(D.prefs.val('search')!=true){
				Utils.searchBox.SetSearchEngine();
			//}
			var versionComponent=Components.classes['@mozilla.org/xpcom/version-comparator;1'].createInstance(Ci.nsIVersionComparator);
			if(version&&versionComponent.compare(version, '3.1.6.9') <= 0){
				updateToolbar();
			}
			if(version&&versionComponent.compare(version, '3.1.6.40') <= 0){
				//
				D.prefs.val('bookmark.tags','[]');
			}
			if(version&&versionComponent.compare(version, '3.1.6.33') <= 0){
				updateToolbarsearch();
			}
			if(version&&versionComponent.compare(version, '4.1.0.15') <= 0){
				var hiddeBtn=D.prefs.val("toolbar.hiddenButtons");
				if(hiddeBtn.indexOf("diigotb-tb-search")>=0)
					D.prefs.val("toolbar.hiddenButtons",hiddeBtn.replace(/diigotb-tb-search/g,''));
				D.prefs.val("toolbar.showItems.diigotb-tb-search",true);
			}
			if(version&&versionComponent.compare(version, '4.1.0.40') <= 0){
				updateCaptureBtn();
			}
			if(version&&versionComponent.compare(version, '4.1.0.44') <= 0){
				D.prefs.val("toolbar.showItems.diigotb-tb-search",true);
				D.prefs.val("toolbar.showItems.diigotb-tb-searchBox",true);
				updateSearchBox();
				D.prefs.val('curSearchEngine',0);
				D.prefs.val('insertSEs',true);
				D.prefs.val('opensearchintab',false);
				D.prefs.val('showimagepanel',true);
			}

			if(versionComponent.compare(D.version, DIIGO_VERSION) == 0){
				D.prefs.val('server', 'www');
				/*D.prefs.val('keys.capture.enabled',false);
				D.prefs.val('keys.capture.keys','control+alt+p');*/
				var a=D.prefs.val('bookmark.tags');
				if(a==''){
					D.prefs.val('bookmark.tags','[]');
				}
			}
				
			updateTorgglebtn();
			//D.prefs.val('.Blacklist',"[]");
			//D.prefs.val('.useBlacklist',true);
			//D.prefs.val('isSpecialUser',false);
			//D.prefs.val('server', 'preview');
			if(version&&versionComponent.compare(version, '3.1.6.17') <= 0){
				D.prefs.val(D.env.user+'.Blacklist',"[]");
				D.prefs.val(D.env.user+'.useBlacklist',true);
				D.prefs.val('isSpecialUser',false);
			}
		}

		Toolbar.refresh('server buttons trace toggleHighlight highlightPen');

		refreshToolbarStatus();

		var filter=D.prefs.val('filter');
		if(typeof filter!="string"){
			D.prefs.val('filter','_all');
		}
		
		if(D.ff31beta&&content){
			doc=content.document;
			if(doc.URL&&doc.URL.length>0){
				D.checkSignIn();
				onLocationChange(doc.URL);
			}
		}
		
		D.env.myBlackList=Utils.json.parseJSON(D.prefs.val(D.env.user+'.Blacklist'));
		if(D.env.myBlackList==null)
			D.env.myBlackList=[];
		
		D.handle('OnInsertDiigoFindLink');
		
		//hide "Refresh User Data" unless it's me!
		if (D.prefs.val('isSpecialUser')){
			try{
			document.getElementById('diigotb-tb-refreshToolbarData').hidden = false;
			document.getElementById('diigotb-tb-server').hidden = false;
			document.getElementById('diigotb-tb-trace').hidden = false;
			}catch(e){
				//throw(e);
			}
		}
	}


	function refreshToolbarStatus() {
		var item=$('#diigotb-menu-toggle');
		var Dtoolbar=document.getElementById("diigotb-toolbar");
		if(!D.prefs.val('showToolbar')){
			item.attr("label","Show Diigo Toolbar");
			Dtoolbar.setAttribute("collapsed","true");
		}else{
			item.attr("label","Hide Diigo Toolbar");
			Dtoolbar.setAttribute("collapsed","false");
		}
	}

	function updateTorgglebtn() {
		 try {
		      var nav = document.getElementById("nav-bar");
			  if(nav){
				  var curSet = nav.currentSet;
				  if(curSet) {
					var set=curSet.replace(/,diigotb-tb-toggle/, "");

			        if (curSet.indexOf("urlbar-container") != -1) {
			          set = curSet.replace(/urlbar-container/, "diigotb-tb-toggle,urlbar-container");
			        }else if (curSet.indexOf("home-button") != -1){
					  set = curSet.replace(/home-button/, "home-button,diigotb-tb-toggle");
					}else{
			          set = "diigotb-tb-toggle,"+curSet;
			        }
					nav.setAttribute("currentset", set);
					nav.currentSet = set;
					document.persist("nav-bar", "currentset");
					nav.setAttribute("defaultset", set);
					nav.defaultset = set;
					document.persist("nav-bar", "defaultset");

					try {
					  BrowserToolboxCustomizeDone(true);
					}
					catch (e) { throw(e)}
				  }
			  }
		  } catch(e) { throw(e)}
	}

	function updateCaptureBtn(){
		var curSet = Toolbar.getCurrentSet();
		var set;
		curSet=curSet.replace(/,diigotb-tb-capture/, "");
		if(curSet.indexOf("diigotb-tb-highlight")>=0){
			set = curSet.replace(/diigotb-tb-highlight/, "diigotb-tb-highlight,diigotb-tb-capture");
		}else if(set.indexOf("diigotb-tb-comment")>=0){
			set = curSet.replace(/diigotb-tb-comment/, "diigotb-tb-capture,diigotb-tb-comment");
		}else{
			set=curSet+",diigotb-tb-capture";
		}

		Toolbar.setCurrentSet(set);
	}
	
	function updateSearchBox(){
		var curSet = Toolbar.getCurrentSet();
		var set;
		curSet=curSet.replace(/diigotb-tb-searchBox/, "");
		if(curSet.indexOf("diigotb-tb-diigo")>=0){
			set = curSet.replace(/diigotb-tb-diigo/, "diigotb-tb-diigo,diigotb-tb-searchBox");
		}else if(set.indexOf("diigotb-tb-bookmarkStar")>=0){
			set = curSet.replace(/diigotb-tb-bookmarkStar/, "diigotb-tb-searchBox,diigotb-tb-bookmarkStar");
		}else{
			set=curSet+",diigotb-tb-searchBox";
		}

		Toolbar.setCurrentSet(set);
	}
	
	function updateToolbarsearch() {
		try{
		var curSet = Toolbar.getCurrentSet();
		var set;
		if(curSet.indexOf("diigotb-tb-toggleSearchHighlight")>=0){
			set = curSet.replace(/,diigotb-tb-toggleSearchHighlight/, "");
		}
		if(set.indexOf("Diigo-X-SearchWordsContainer")>=0){
			set = set.replace(/,Diigo-X-SearchWordsContainer/, "");
		}
		if(set.indexOf("diigotb-tb-search")>=0){
			set = set.replace(/,diigotb-tb-search/, "");
			set+=",diigotb-tb-search";
		}

		Toolbar.setCurrentSet(set);
		}catch(e){throw(e);}
	}

	function updateToolbar() {
		 try {
		      var curSet = Toolbar.getCurrentSet();
		      if (curSet.indexOf("diigotb-tb-readitLater") == -1) {
		        var set;
		        if (curSet.indexOf("diigotb-tb-message") != -1) {
		          set = curSet.replace(/diigotb-tb-message/, "diigotb-tb-message,diigotb-tb-readitLater");
		        }else if (curSet.indexOf("diigotb-tb-quickAccessFilter-item") != -1){
				  set = curSet.replace(/diigotb-tb-quickAccessFilter-item/, "diigotb-tb-readLater,diigotb-tb-quickAccessFilter-item");
				}else{
		          set = curSet + ",diigotb-tb-readLater";
		        }
		        Toolbar.setCurrentSet(set);
		      }
		  } catch(e) {throw(e) }
	}
	
	function tearDown(event) {
		window.getBrowser().removeProgressListener(progressListener);
		var appContent = document.getElementById('appcontent');
		appContent.removeEventListener('DOMContentLoaded', onPageLoad, true);
		appContent.removeEventListener('load', onPageLoad, true);
		appContent.removeEventListener('unload', onPageUnload, true);
		//appContent.removeEventListener('close', onPageClose, true);
		
		appContent.removeEventListener('mouseup', onPageMouseUp, false);
		appContent.removeEventListener('mousedown', onPageMouseDown, false);
		appContent.removeEventListener('click', onPageClick, false);
		appContent.removeEventListener('mouseover', onPageMouseOver, false);
		appContent.removeEventListener('mouseout', onPageMouseOut, false);

		//PluginWatcherObserver.unregister();
		document.getElementById("diigotb-toolbar").removeEventListener("DOMAttrModified",onDOMAttrModified,false);
		
		D.prefs.tearDown();
	}

	function onDOMAttrModified(event){
		try{
			switch(event.attrName){
				case "collapsed":
					var C=document.getElementById("diigotb-toolbar").getAttribute("collapsed");
					var item=$('#diigotb-menu-toggle');
					if(C=="true"){
						item.attr("label","Show Diigo Toolbar");
						D.prefs.val('showToolbar',false)
					}else{
						item.attr("label","Hide Diigo Toolbar");
						D.prefs.val('showToolbar',true)
					}
					break;
			}
		}catch(e){
			throw(e)
		}
	}
	
	function insertPdfBtn(doc){
		if(doc.getElementById('diigo-pdf')){
			return;
		}
            var enterBtn = doc.createElement('div');
            enterBtn.id = 'diigo-pdf';
            enterBtn.innerHTML = '<div id="diigo-enter-pdf"></div><div id="diigo-pdfbtn-tip"><div id="diigo-i-know"></div></div>';
            var insertNode = doc.body;
            $(insertNode).append(enterBtn);

            
            if(D.prefs.val('isPdfTipClicked')){
            	$('#diigo-pdfbtn-tip').hide();
            }


            $('#outerContainer').bind("click", function(){
                //console.log("aaa");
                if(document.body.scrollTop == 0){
                    $('#diigo-enter-pdf').show();
                }
                else{
                    $('#diigo-pdf').hide();
                }

            });

            $('#diigo-enter-pdf').bind('click', function () {
                Components.utils.import("resource://gre/modules/AddonManager.jsm");
				AddonManager.getAddonByID('{fc2b8f80-d9a5-4f51-8076-7c7ce3c67ee3}', function(addon){
					var addonLocation = addon.getResourceURI("").QueryInterface(Components.interfaces.nsIFileURL).file.path;
					doc.defaultView.location.href="chrome:\/\/diigotb\/content\/pdf\/viewer.html?file="+doc.URL;
				});
            });
            
            Utils.dom.HTML_addCSS(doc, window.SCREENSHOT_CSS);
	}

	/**
	 * This may be called twice for each page. One from DOMContentLoaded and one from load.
	 * Because firefox doesn't trigger DOMContentLoaded event for some pages.
	 * So we use a flag (doc.__diigo_loaded) make this reenterable.
	 *
	 */
	function onPageLoad(event) {
		var doc = event.originalTarget;

		//just for document
		if (doc.nodeName != '#document') return;
		
		
		//don't do anything if not signed in
		D.checkSignIn();
		if (!D.hasSignedIn()) return;
		
		//if (redirectAnnotatedLink(doc.defaultView.location.href, doc.defaultView)) return;

		//safe to reenter
		if (doc.__diigo_loaded) {
			//there is something to do even if __diigo_loaded is already set!
			
			//when we are painting highlights by DOMContentLoaded, frames documents may not be loaded
			//load event of top doc triggers after all frames are loaded
			//good time to paint highlights in frames
			var context = doc.__diigo;
			if (context 
				&& context.annotationsLoaded 
				&& context.win == context.win.top 
				&& context.win.frames.length > 0 
				&& event.type == 'load') 
			{
				Utils.dump('[Paint frame anns]', event.type);
				context.paintAllAnnotations(true);
			}
			
			//when diigofying or sidebar is refresing bookmark info, frames may not be loaded
			//some pages change title after frames are loaded. e.g. http://www.microsoft.com/resources/documentation/windows/xp/all/proddocs/en-us/sec_cook.mspx?mfr=true
			//so we need to refresh bookmark info here.
			if (context && context.win == context.win.top) { //do not overwrite saved title

				context.htmlcontent="<html>"+doc.body.parentNode.innerHTML+"</html>";

				if(!context.bookmark.saved){
					context.bookmark.title = context.doc.title;
					Sidebar.refresh('thisUrl', context, true);
				}
			}
			
			return;
		};
		
		//mark as loaded, when this flag is set, it is safe to paint highlights
		doc.__diigo_loaded = true;
		
		Utils.dump('[Page loaded]', event.type, doc.URL);
		
		//diigofy
		var context = doc.__diigo;
		if (context && context.win == context.win.top ) {
			
			Utils.dump('page loaded diigofy2', context.annotations);
			
			if (context.annotationsLoaded) {
				Utils.dump('[Paint all anns]', event.type);
				context.paintAllAnnotations();
			}

//			D.upload.saveImageLocal(D.upload.capturePng());
			//D.captureMode.enterCaptureMode(doc);

			if (!context.bookmark.saved) { //do not overwrite saved title
				context.bookmark.title = context.doc.title;
				Sidebar.refresh('thisUrl', context, true);
			}
			
		} else {
			Utils.dump('page loaded diigofy');
			context = D.PageContext.diigofy(doc);
			//doc..addEventListener('close', onPageClose, true);
		}
		
		if(doc&&doc.contentType=="application/pdf"&& !(D.isLocalPDFReader(doc.URL))){
	  		insertPdfBtn(doc);
	  		context.isPdfReader = true;
	  		context.$ = D.j(context.win);
	  	}
		
		//if it's the selected tab
		if (doc == content.document)
			onReadyLocationChange();
			
			
	}
	
	function onPageUnload(event) {
		if (!D.hasSignedIn()) return;
		
		var doc = event.originalTarget;
		//just for document
		if (doc.nodeName != '#document') return;
		
		Utils.dump('[Page unloaded]', doc, doc.URL);
		
		PageContext.undiigofy(doc);

		//alert(doc.URL);
		
		doc.__diigo_unloaded = true;
		// clear permalink info 
//		var win = doc.defaultView, browser;
// 		if (win.top == win && (browser = gBrowser.getBrowserForDocument(win.document))) {
// 			if (browser.__diigo_permalinkInfo && win.location.href == browser.__diigo_permalinkInfo.url)
// 				delete browser.__diigo_permalinkInfo;
// 		}
	}
	
	function onPageMouseUp(event) {
		var t = event.target, d = t.ownerDocument.__diigo;
		if (!d) return;
		d.documentMouseUp(event);
	}
	
	function onPageMouseDown(event) {
		var t = event.target, d = t.ownerDocument.__diigo;
		if (!d) return;
		d.documentMouseDown(event);
	}
	
	function onPageClick(event) {
		var t = event.target, d = t.ownerDocument.__diigo;
		if (!d) return;
		d.documentClick(event);
	}
	
	function onPageMouseOver(event) {
		var t = event.target, d = t.ownerDocument.__diigo;
		if (!d) return;
		d.documentMouseOver(event);
	}
	
	function onPageMouseOut(event) {
		var t = event.target, d = t.ownerDocument.__diigo;
		if (!d) return;
		d.documentMouseOut(event);
	}
	
	// I assume when onLocationChange is called, the corresponding window is window.content
	function onLocationChange(nUri) {
		if (!D.hasSignedIn()) return;
		var url;
		if(nUri&&nUri.spec)
			url=nUri.spec;
		else
			url='about:blank';
		
		Utils.dump('[location change]', url);
		
		//uninstall toolbar special trigger url http://www.diigo.cn/cgi-bin/1.py?cmd=3010
		if (url.indexOf('/cgi-bin/1.py?cmd=3010') > -1 && D.runtime.uninstalling) {
			Utils.dump('uninstall triggered');
			D.utils.firefox.uninstallExtension(D.extensionId);
			alert('Diigo Toolbar has been uninstalled. You\'ll need to restart Firefox for the changes to take effect.');
			return;
		}
		//if (redirectAnnotatedLink(url, content)) return;
		
		if (url.indexOf('/cgi-bin/1.py?cmd=3060') > -1 ) {
			D.WebAPI.loadMyStuff('permissions');
			return;
		}
		
		//diigofy and load bookmark early
		//so that the page download and webapi request can go in parallel
		PageContext.diigofy(content.document);
		
		D.runtime._focusedContext = D.getCurrentContext();
		D.updateBookmarkableState();
		
		Sidebar.changeLocation();
		D.DlgIC.changeLocation();
		D.Clip.changeLocation();
		Toolbar.refresh('comment readLater HighlightPen');

		try{
			_focusedcontext=D.runtime._focusedContext;
			
			if(_focusedcontext&&_focusedcontext.filterMode!=D.prefs.val('filter')){
				_focusedcontext.filterMode=D.prefs.val('filter');
				_focusedcontext.unpaintAllAnnotations();
				_focusedcontext.paintAllAnnotations();
			}
			var cap=false;
			if(_focusedcontext){
				var captureItem=_focusedcontext.captureItem;
				if(captureItem){
					if(captureItem.isCaptureMode){
						//D.captureMode.enterCaptureMode(_focusedcontext.doc);
						document.getElementById('diigotb-tb-capture').checked=true;
						D.captureMode.currentItem=captureItem;
						cap=true;
						D.captureMode.updateEventHandler(_focusedcontext.doc);
					}
				}
			}
			if(!cap){
				document.getElementById('diigotb-tb-capture').checked=false;
				D.captureMode.currentItem=null;
			}
			
		}catch(e){
			throw(e);
		}

		
		var doc = content.document;
		if (doc.__diigo_loaded) onReadyLocationChange();
		
		D.EventMachine.changeLocation();
	}
	
	/**
	 * When location changes to a ready document
	 */
	function onReadyLocationChange() {
		Utils.dump('[ready location change]', content.document.URL);
		
		var doc = content.document;
		D.ColorfulFind.onReadyLocationChange();
	}
	
	/**
	 * Redirects
	 * compact annotated link -> full annotated link
	 * full permanlink -> original page
	 * 
	 * Stores/clears annotated link info
	 * @return true if a redirect is performed, otherwise false
	 * 
	 * Calling code can use the return value to determine whether further processing is needed.
	 */
	function redirectAnnotatedLink(url, win) {
		//if requesting a annotated link
	    if (url.indexOf(D.urls.getDiigoURL() + '/annotated') == 0) {
	    	diigo.utils.dump('[annotated link]===========', url);
	    	var params = Utils.string.parseQuery(url);
	     	
	     	if (params.full) {
	     		// if the annotated link url is expanded (thus containing all necessary info)
	     		// store the annotated link info to tab	
	     		diigo.utils.dump('[annotated link]=========== full', url);
	     		
	     		var browser;
	     		if (win.top == win && (browser = gBrowser.getBrowserForDocument(win.document))) {
						
					params.legacy = params.legacy == 1; //...&legacy=1 to bool
	     			browser.__diigo_permalinkInfo = params;
//						D.pm = params;
//						browser.kkk = params;
	     		}
	     		
	     		win.location.href = params.url;
		     	return true;
	     	} else if (!params.more) {
	     		//the annotated link url doesn't contain all necessary info, request the full url
		    	diigo.utils.dump('[annotated link]=========== requesting full url', url);
		     	win.location.href = url + (url.indexOf('?') == -1 ? '?' : '&') + 'more=1';
		     	return true;
		    }
	    }
	    
	    return false;
	}
	
	window.addEventListener('load', init, false);
	window.addEventListener('unload', tearDown, false);

	function closeWindow(event){
		if(D.env.capturedImageNo>0){
			var check = {value: false};                  

			var prompts=Utils.firefox.promptService();
			var flags = prompts.BUTTON_POS_0 * prompts.BUTTON_TITLE_YES +
						prompts.BUTTON_POS_1 * prompts.BUTTON_TITLE_NO  ;

			var button = prompts.confirmEx(window, "Confirmation", "Uploading is not finished yet. Are you sure you want to close the window?",
                               flags, "", "", "", null, check);
			if(button==1){
				event.preventDefault();
			}
		}
	}
	window.addEventListener('close', closeWindow, false);
	
	D.AnnotatedLink = {
		/**
		 * @param {PageContext} context optional. If provided, will check in cache first and updated cache after web api responds
		 */
		'get': function(url, mode, callback, context) {
			var annotatedUrl;
			if (context && (annotatedUrl = context.annotatedLinkCache[mode])) {
				callback(annotatedUrl);
			} else {
				D.WebAPI.makeAnnotatedLink(url, mode, context, function(data) {
					if (context)
						context.annotatedLinkCache[mode] = data.url;
					callback(data.url);
				});
			}
		}
	};
	
	D.AnnotationFormatter = {
		format: function(context, options) {
			var options = diigoUtil.extend({
				mode: 3,
				includePageComments: false
			}, options || {});
			
			var includeOthers = (options.mode & 2) == 2;
			var includePrivate = (options.mode & 1) == 1;
			
			//D.utils.dump('[AnnotationFormatter] format mode:', options.mode, 'includeOthers', includeOthers, 'includePrivate', includePrivate);
			
			var bm = context.bookmark;
		
			var body = '';
			
			//page comments
			if (options.includePageComments) {
				body += ('<ul>' + context.pageComments
				.filter(function(pc) {
					return (includeOthers ? true : pc.user == D.env.user) && (includePrivate ? true : pc.mode != D.PRIVACY_MODE_PRIVATE);
				})
				.map(function(c) {
					return '<li style="line-height:150%;margin-bottom:.6em;">' + diigoUtil.string.stripTags(c.content) + '</li>'
				})
				.join('\n') + '</ul>')
			}
			
			//annotations
			body += ('<ul>' + context.annotations
			.filter(function(ann) {
				return (includeOthers ? true : ann.user == D.env.user) && (includePrivate ? true : ann.mode != D.PRIVACY_MODE_PRIVATE);
			})
			.map(function(ann) {
				var content= '';
				
				if (ann.type == D.ANNOTATION_TYPE_TEXT) { 
					content = diigoUtil.string.stripTags(ann.content);
				} else if (ann.type == D.ANNOTATION_TYPE_IMAGE) { 
					content = diigoUtil.string.showTpl('<img src="#{0}" title="#{1}" alt="#{1}">(image)', [ann.extra.imageUrl, '']);
				} else if (ann.type == D.ANNOTATION_TYPE_FLOATNOTE) { 
					content = '<img src="https://www.diigo.com/images/v2/float_note.gif" />(floating sticky note)';
				} else if (ann.type == D.ANNOTATION_TYPE_FLASH) { 
					content = '(flash movie)' + ann.content;
				} else if (ann.type == D.ANNOTATION_TYPE_VIDEO) { 
					content = '(video)' + ann.content;
				}
				 
				var html = diigoUtil.string.showTpl(
					'<li style="line-height:150%;margin-bottom:.6em;">#{0} #{1} #{2}</li>', 
					[
						content, 
						ann.groups ? '' : '', 
						('<ul>' + ann.comments
							.filter(function(ann) {
								return (includeOthers ? true : ann.user == D.env.user) && (includePrivate ? true : ann.mode != D.PRIVACY_MODE_PRIVATE);
							})
							.map(function(c) {
								return '<li style="line-height:150%;margin-bottom:.6em;">' + 
									   diigoUtil.string.stripTags(c.content) + 
									   diigoUtil.string.showTpl(' <span style="font-size:11px;color:#aaa">comment by <a href="#{0}">#{1}</a></span>', [D.urls.getUserHomepageURL(c.user), c.realName]) + 
									   '</li>'
							})
							.join('\n') + '</ul>')
					]
				);
				
				return html;
			}).join('\n') + '</ul>');
			
			return body;
		}
	};
	
	//load subscripts
	var subscriptLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
						  .getService(Components.interfaces.mozIJSSubScriptLoader);
	subscriptLoader.loadSubScript('chrome://diigotb/content/pageCss.js');
						  
	subscriptLoader.loadSubScript('chrome://diigotb/content/diigo.prefs.js');
	subscriptLoader.loadSubScript('chrome://diigotb/content/diigo.urls.js');
	subscriptLoader.loadSubScript('chrome://diigotb/content/diigo.handlers.js');
	subscriptLoader.loadSubScript('chrome://diigotb/content/keyboardshortcut.js');
	subscriptLoader.loadSubScript('chrome://diigotb/content/diigo.eventMachine.js');
	subscriptLoader.loadSubScript('chrome://diigotb/content/diigo.messageAlert.js');
	subscriptLoader.loadSubScript('chrome://diigotb/content/twitter.js');
	subscriptLoader.loadSubScript('chrome://diigotb/content/diigo.clip.js');
	subscriptLoader.loadSubScript('chrome://diigotb/content/quickAccessFilter/diigo.quickAccessFilter.js');
	subscriptLoader.loadSubScript('chrome://diigotb/content/diigo.localBookmarks.js');
							  
})();
