(function() {
	var D = window.diigo;
	var Utils = D.utils;
	
/////////////////////////////////////////////////////////////////////////////////

D.prefs = {
	_prefDomain: 'extensions.diigotb.',
	_cache: {},
	
	/**
	 * Whether the pref is cacheable. Do not cache large data
	 * @param {String} k
	 */
	isCacheable: function(k) {
		return true;
	},
	
	/**
	 * Update cache
	 */
	updateCache: function(k, v) {
		return this.isCacheable(k) ? this._cache[k] = v : null;
	},
	
	invalidateCache: function(k) {
		delete this._cache[k];
	},
	
	/**
	 * To be deprecated in favour of get and set
	 * val('key') //getter
	 * val('key', 'val') //setter
	 */
	val: function(k, v) {
		return v === undefined ? this.get(k) : this.set(k, v);
	},
	
	/**
	 * 
	 * @param {String} k prefName
	 * @param {Boolean} whether we are dealing with a unicode string
	 */
	get: function(k, unicode) {
		var v = this._cache[k], p = Utils.prefs;
		if (v === undefined) {
			v = p[unicode ? 'getUnicodePref' : 'getPref'](this._prefDomain + k);
			if (this.isCacheable(k)) {
				this.updateCache(k, v);
			}
			if (v === undefined) {
				v="";
			}
		}
		return v;
	},
	
	set: function(k, v, unicode) {
		var p = Utils.prefs;
		this.invalidateCache(k);
		return p[unicode ? 'setUnicodePref' : 'setPref'](this._prefDomain + k, v);
		//need not to update cache here. the prefObserver takes care of prefs changes
	},
	
	del: function(k) {
		Utils.prefs.deletePref(this._prefDomain + k);
	},
	
	hasUserValue: function(k) {
		return Utils.prefs.hasUserValue(this._prefDomain + k);
	},
	
	init: function() {
		this._prefObserver.register();
	},
	
	tearDown: function() {
		this._prefObserver.unregister();
	},
	
	//nsIPrefObserver
	_prefObserver: {
		register: function() {
			this._branch = Utils.xpcom.CCSV("@mozilla.org/preferences-service;1", "nsIPrefBranch");
			this._branch.addObserver(D.prefs._prefDomain, this, false);
		},
		
		unregister: function() {
			if(!this._branch) return;
			this._branch.removeObserver(D.prefs._prefDomain, this);
		},
		
		observe: function(aSubject, aTopic, aData) {
			if(aTopic != "nsPref:changed") return;

			var name = aData.substr(D.prefs._prefDomain.length);
			D.prefs.invalidateCache(name);
			
			//update prefs cache
			
			Utils.dump('prefs change: ' + aTopic, name, D.prefs.get(name));
			
	        //this.updatePref(name, value);
			if (name == 'server') {
				D.Toolbar.refresh('server');
				D.onSignOut();
				D.checkSignIn();
			} else if (name == 'trace') {
				D.Toolbar.refresh('trace');
//			} else if (name.match('^toolbar\.showItems\.')) {
//				D.Toolbar.refresh('buttons');
			} else if (name.match(/^toolbar\.showItems\.diigotb-tb-quickAccessFilter-item/)) {

				D.QuickAccessFilter.checkEnabled();
			} else if(name == D.env.user+'.useBlacklist'){
				D.runtime._focusedContext = D.getCurrentContext();
				D.handle('toggleHighlight',false);				
				D.runtime._focusedContext.unpaintAllAnnotations();
				D.runtime._focusedContext.paintAllAnnotations();
				Sidebar.refresh('annotations', D.runtime._focusedContext, true);
			} else if (name == 'hl.style') {
				//refresh all painted highlight in all tabs
				gBrowser.browsers.forEach(function(b) {
					var doc = b.contentDocument;
					var context = doc.__diigo; 
					if (context) {
						//TODO use Highlighter consts
						context.$('em.diigoHighlight.type_0')
						.removeClass('a_style b_style c_style').addClass(D.prefs.val('hl.style'));
					}
				});
			} else if (name.indexOf('keys.') == 0) {
				D.Keys.onPrefsChange();
			}
		}
	}
};

/////////////////////////////////////////////////////////////////////////////////
})();