/**
 * Supporting code for Highlight
 * @author Arrix
 */
(function() {

var D = window.diigo;
var Utils;
Utils = D["utils"];
	var Annotation;
	D.registerRunLater(function () {
		Annotation = D["Annotation"];
	});

var H = D.Highlighter = {
	
	//added to each annotation
	HIGHLIGHT_CLASS: 'diigoHighlight',
	HIGHLIGHT_ICON_CLASS: 'diigoIcon',
	//highlight id class prefix
	HIGHLIGHT_ID_CLASS: 'id_',
	HIGHLIGHT_TYPE_CLASS: 'type_',
	
	colorPen:['yellow','blue','green','pink'],
	/**
	 * Tells whether the target element is a highlight element.
	 * Extracts highlight information from an element.
	 * Returns a highlight info hash or false
	 * 
	 * Used by event handlers.
	 * @return {Object}
	 * {
	 *   type: type,
	 *   ids: ['annId', ...]
	 * }
	 */
	isHighlightElement: function(target) {
	try{
		if (target.nodeType == 1 
				&& target.className 
				&& (target.className.indexOf(this.HIGHLIGHT_CLASS) > -1
						||target.className.indexOf(this.HIGHLIGHT_ICON_CLASS) > -1)) 
		{
			
			var r = {type: -1, ids: []};
			r.type = Number(target.className.match(new RegExp(this.HIGHLIGHT_TYPE_CLASS + '(\\d)'))[1]);
			
			Utils.string.scan(
				target.className, 
				new RegExp(this.HIGHLIGHT_ID_CLASS + '([^\\s]+)(?:\\s|$)', 'img'),
				function(m) {
					r.ids.push(m[1]);
				}
			);
			
			//only take the first one
			r.ids = r.ids.slice(0, 1);
			return r;
		}
	}catch(e){
		//alert(e);
	}
		return false;
	}
};

var TH = D.TextHighlighter = Utils.Class.create({
	initialize: function(context) {
		Utils.extend(this, {
			context: context,
			//element black list for highlight
			//to see the full list of html elements, visit http://www.w3.org/TR/html401/index/elements.html
			tagBlackList: ',applet,area,base,basefont,bdo,button,frame,frameset,iframe,head,hr,img,input,link,map,meta,noframes,noscript,optgroup,option,param,script,select,style,textarea,title,',
			docHtml: '',
			docTxt: '',
			docTxtOffsetList: [],
			_previousSelection: null
		});
	},

	/*====================== Implementation =======================*/
	
	/**
	 * Construct a look up table for finding corresponding DOM nodes of some text
	 * The look up table is an Array of {offset: offsetInDOMString, node: the node starting the offset}
	 */
	domSnapshot: function() {
		var z = this;
		this.docString = this.context.doc.body.innerHTML;
		this.docTxt = '';
		this.docTxtOffsetList = [];
		new DOMWalker(this.context.doc.body, function(n) {
			if (n.nodeType == 3) {
				var nodeTxt = n.nodeValue.replace(/\s+/g, '');
				//console.log(nodeTxt);
				if (nodeTxt) {
					z.docTxtOffsetList.push({offset: z.docTxt.length, node: n});
					z.docTxt += nodeTxt;
				}
			}
		}, {
			/**
			 * Performance!
			 */
			filter: function(n) {
				return z.domSnapshotNodeFilter(n);
			}
		}).walk();
	},
	
	domSnapshotNodeFilter: function(n) {
		if (n.nodeType == 3) return true;
		if (n.nodeType != 1) return false;
		//D.utils.dump(n.tagName);
		if (n.tagName && this.tagBlackList.indexOf(',' + n.tagName.toLowerCase() + ',') > -1) {
			return false;
		}
		if (!n.className) return true;
		if (/(^|\s)diigoHighlight(\s|$)/.test(n.className)) return true;
		if (/(^|\s)diigo(\s|$)/.test(n.className)) return false;
		return true;
	},
	
	/**
	 * DOMWalker node filter for highlight
	 */
	isHighlightableNode: function(n) {
		return n.nodeType == 3 || (n.nodeType == 1 && this.tagBlackList.indexOf(',' + n.tagName.toLowerCase() + ',') == -1);
	},
	
	/**
	 * Returns the start/end offset of a string within a text node
	 * @param node the text node
	 * @param txt highlight text without whitespaces
	 * @param startOrEnd 1 - start, 0 - end
	 * @param {Integer} offset the distance between the star/end of the string and the start of the node in the serialized doc string
	 */
	findOffset: function(node, txt, startOrEnd, offset) {
//		D.utils.dump('[text highliter] findOffset', startOrEnd ? 'start' : 'end', offset, node, txt);
		var nodeTxt = node.nodeValue, nodeTxt2 = nodeTxt.replace(/\s+/g, '');
		
		if (startOrEnd) {
			var substr = txt.slice(0, nodeTxt2.length - offset);
			D.utils.dump('[text highliter] findOffset substr', substr);
			for (var j = nodeTxt.length - substr.length; j >= offset ; j--) {
				//console.log(nodeTxt.slice(j).replace(/\s+/g, ''), substr)
				if (nodeTxt.slice(j).replace(/\s+/g, '').indexOf(substr) > -1) {
					return j;									
				}
			}
		} else {
				var substr = txt.slice(Math.max(0, txt.length - offset - 1));
				D.utils.dump('[text highliter] findOffset substr', substr);
				for (var j = offset; j <= nodeTxt.length; j++) {
//					D.utils.dump(j, nodeTxt.slice(0, j).replace(/\s+/g, ''), substr)
					if (nodeTxt.slice(0, j).replace(/\s+/g, '').indexOf(substr) > -1) {
						return j;									
					}
				}
		}
		return startOrEnd ? nodeTxt.length : 0;
	},
	
	/**
	 *  
	 */
	seek: function(html, nth) {
//		if (this.context.doc.contentType == 'text/plain') return this.seek_plainText(html, nth);
		
		nth = nth || 0;
		var txt = this.html2txt(html);
		var docTxt = this.docTxt;
		
		var startIndex = 0, endIndex = 0, n = 0;
		while ((startIndex = docTxt.indexOf(txt, endIndex)) > -1) {
			endIndex = startIndex + txt.length - 1;
			if (n == nth) break;
			n++;
		}
		
		D.utils.dump('[text highliter] seek : ', startIndex, endIndex);
		
		//when startIndex not found
		if (startIndex == -1) return null;
				
		//when endIndex is out of bound
		//2007-06-26 06:39:57 UTC Arrix: I don't think it's necessary.
		//if (endIndex > docTxt.length - 1) return null;
		
		var list = this.docTxtOffsetList;
		
		D.utils.dump('[text highliter] seek : ', list.length, list);
		
		var startNode, endNode, startOffset, endOffset, commonAncestor;
		
		for (var i = 0, a, l = list.length; a = list[i], i < l; i++) {
			if (startIndex == a.offset || (startIndex > a.offset && ((i+1) == list.length || startIndex < list[i+1].offset))) {
				startNode = a.node;
				startOffset = this.findOffset(startNode, txt, 1, startIndex - a.offset);
			}
			
			if (endIndex == a.offset || (endIndex > a.offset && ((i+1) == list.length || endIndex < list[i+1].offset))) {
				endNode = a.node;
				endOffset = this.findOffset(endNode, txt, 0, endIndex - a.offset);
				break;
			}
		}
		
		//find common ancestor
		if (startNode && endNode) {
			function descendantOf(element, ancestor) {
				while (element = element.parentNode)
			      if (element == ancestor) return true;
			    return false;
			}
			var ancestor = startNode;	
			while (ancestor = ancestor.parentNode) {
				if (descendantOf(endNode, ancestor)) {
					commonAncestor = ancestor;
					break;
				}
			}
			
			if (!commonAncestor) return null;
		}
		
		return startNode && endNode ? {
			startNode: startNode, 
			endNode: endNode, 
			startOffset: startOffset, 
			endOffset: endOffset,
			endIndex: endIndex,
			commonAncestor: commonAncestor,
			txt: txt
		} : null;
	},
	
	//a txt file text/plain is render as ...<body><pre>texttexttext...</pre></body>...
	//the text node is usually large, we use a more optimized "seek" here
//	seek_plainText: function(txt, nth) {
//		
//	},
	
	/**
	 * paint a highlight on the page
	 */
	paint: function(id) {
		var z = this;
		var context = z.context;
		var $ = context.$;
		var doc = context.doc;
		
		var ann = Annotation.find(context.getTopLevelContext(), id);
		var classId=H.HIGHLIGHT_ID_CLASS + ann.id;
		var comments=ann.getComments(D.prefs.val('filter'));
		
		//if already painted, update style
		if (ann.paintedSuccessfully) {
			addHighlightClass('em.' + classId, ann);
			addIconClass('div.' + classId, ann);
			$('em.' + classId).filter(":first").toggleClass('commented', comments.length > 0);
			return;
		}

		ann.painted = true;
		
		function addHighlightClass(em, ann) {
			$(em)
				.addClass(H.HIGHLIGHT_CLASS)
				.removeClass('a_style b_style c_style')
				.addClass(D.prefs.val('hl.style'))
				.addClass(classId)
				.addClass(H.HIGHLIGHT_TYPE_CLASS + ann.type)
				.removeClass('yellow blue green pink')
				.addClass((ann.extra.color&&ann.extra.color.length>0)?ann.extra.color:'yellow');
			
			//$(em+":first")
			//	.toggleClass('commented', comments.length > 0);
				//.toggleClass('public', ann.isPublic())
				//.toggleClass('group', ann.inAnyGroups());
			return em;
		}
		
		function wrap(node) {
			var j = $(node.parentNode);
			//if wrapper already created
			if (j.hasClass(H.HIGHLIGHT_CLASS)) {
				j.addClass(classId);
			} else {
				$(node).wrap(addHighlightClass(context.doc.createElement('em'), ann));
			}
		}
		this.domSnapshot();
		D.utils.dump('[text highliter] paint : ', ann);
		
		var info = this.seek(ann.content, ann.nth);
		if (!info) return;
		
		ann.prettyTxt = this.html2txt_pretty(ann.content);
		
		D.utils.dump('[text highliter] paint : ', info.startOffset, info.endOffset);
		D.utils.dump('[text highliter] paint : ', info.startNode);
		
		if (info.startNode === info.endNode) {
			
			//this happens when there is repeated text pattern in a same node
			if (info.endOffset <= info.startOffset) {
				info.startOffset = info.endOffset - info.txt.length;
			}
			var t = info.startNode.splitText(info.startOffset);
			var t1 = t.splitText(info.endOffset - info.startOffset);
			//|| t because of a strange ie bug which happens when highligting full content of a single node
			wrap(t1.previousSibling || t);
		} else {
		
			var startNode = info.startNode.splitText(info.startOffset);
			var endNode = info.endNode.splitText(info.endOffset).previousSibling;
			var on = false;
			var textNodes = [];
			new DOMWalker(info.commonAncestor, function(n) {
				//turn on
				if (!on && n === startNode) {
					on = true;
				}
				if (on) {
					if (z.isTextNode(n) && n.nodeValue.replace(/(^\s+|\s+$)/g, '').length > 0) {
						textNodes.push(n);
					}
				}
				
				//turn off
				if (on && n === endNode) {
					on = false;
					throw DOMWalker.$end;
				}
			}, {
				filter: Utils.func.bind(this.isHighlightableNode, this)
			}).walk();
			
			Utils.each(textNodes, function(v) {wrap(v);});
		}
		
		var j = $('em.' + H.HIGHLIGHT_ID_CLASS + ann.id);
		ann.paintedSuccessfully = j.size() > 0;
		
		//calculate offset
		//the highlight may not be painted
		//calling offset() upon an empty jquery object will cause an error
		var offset = j.offset();
		if (j.size() > 0) {
			ann.extra.top = offset.top;
			ann.extra.left = offset.left;
		}
		function addIconClass(icon, ann) {
			$(icon)
			.addClass(H.HIGHLIGHT_ICON_CLASS)
			.addClass(classId)
			.addClass(H.HIGHLIGHT_TYPE_CLASS + D.ANNOTATION_TYPE_ICON)
			.addClass("TextIcon")
			.toggleClass('public', ann.isPublic())
			.toggleClass('private', ann.isPrivate())
			.toggleClass('group', ann.inAnyGroups())
			.toggleClass('commented', comments.length > 0)
			.toggleClass('diigoEdit', comments.length == 0)
			.css({bottom:0,top:null,left:0})
			;
			if(comments.length > 0){
				$(icon).html("<span class='"+classId+" "+H.HIGHLIGHT_ICON_CLASS+" "+H.HIGHLIGHT_TYPE_CLASS + D.ANNOTATION_TYPE_ICON+"'>"+comments.length+"</span>");
				$(icon).attr("title",D.AnnotationContextMenu.tipMsg(ann,false));
				z.adjustIconBg(ann);
			}else{
				$(icon).html("");
				$(icon).attr("title","");
			}
			return icon;
		}
		if(ann.type==D.ANNOTATION_TYPE_TEXT&&$('div.'+classId).size()<=0&&j.size() > 0){
			var icon=$(context.doc.createElement('div'));
			var firstNode=j.filter(":first");
			firstNode.toggleClass('commented', comments.length > 0);
			icon.prependTo(firstNode);
			addIconClass(icon,ann);
		}

		try {
			this.adjustColor(ann);
		} catch (e) {
			D.utils.dump('[Highlight adjust color]', e);
		}
		
		return ann.paintedSuccessfully;
	},
	
	adjustIconBg: function(ann){
		var curwin=ann.context.win, bgcolor;
		try{
		var high=ann.context.$('em.' + H.HIGHLIGHT_ID_CLASS + ann.id)
		var re=high.parents().filter(function(index){
			var over=curwin.getComputedStyle(this, null).getPropertyValue("background-color");
			if(over!='transparent'){
				bgcolor=over;
				throw "end";
			}
			return false;
		});
		}catch(e){}
		
		var icon=ann.context.$('div.diigoIcon.commented.private.' + H.HIGHLIGHT_ID_CLASS + ann.id);
		//icon.css({"background-color":bgcolor+" !important"});
		icon.attr("style","background-color:"+bgcolor+" !important");
	},

	/**
	 * Adjust text color according to highlight background color to ensure readability
	 * http://www.paciellogroup.com/resources/contrast-analyser.html
	 */
	adjustColor: function(ann) {
		var z = this;
		/**
		 * Returns relative luminance
		 * http://www.paciellogroup.com/resources/contrast-analyser.html
		 */
		function L(color) {
			var RsRGB = color.r/255;
		    var GsRGB = color.g/255;
		    var BsRGB = color.b/255;

		    var R = (RsRGB <= 0.03928) ? 
		    RsRGB / 12.92 : 
		    Math.pow((RsRGB + 0.055) / 1.055, 2.4);
		    
		    var G = (GsRGB <= 0.03928) ? 
		    GsRGB / 12.92 :
		    Math.pow((GsRGB + 0.055) / 1.055, 2.4);
		    
		    var B = (BsRGB <= 0.03928) ? 
		    BsRGB / 12.92 : 
		    Math.pow((BsRGB + 0.055) / 1.055, 2.4);
		    
    		return 0.2126 * R + 0.7152 * G + 0.0722 * B;
		}
		/**
		 * Determining "colour visibility" is based on the Contrast Ratio algorithm
		 */
		function isReadable(txtColor, bgColor) {
			var l1 = L(txtColor);
			var l2 = L(bgColor);
			if (l1 > l2) 
				var a = l1, b = l2;
			else
				var a = l2, b = l1;
			//console.log(txtColor, bgColor, a, b, (a + 0.05) / (b + 0.05));
			return (a + 0.05) / (b + 0.05) > 2.5;
		}
		/**
		 * Returns null if readable else the text color in format of #xxxxxx
		 */
		function readableColor(txtColor, bgColor) {
			if (isReadable(txtColor, bgColor)) return null;
			var l = L(bgColor);
			return l > 0.5 ? '#000000' : '#FFFFFF';
		}
		//h = Ctx.findAnnotation2(h);
		var high=this.context.$('em.' + H.HIGHLIGHT_ID_CLASS + ann.id).each(function(i, v) {
			var jj = z.context.$(v);
			var bgColor = parseColor(jj.css('backgroundColor'));
			var txtColor = parseColor(jj.css('color'));
			var newTxtColor = readableColor(txtColor, bgColor);
			if (newTxtColor) {
				jj.css('color', newTxtColor);
			}
		});
			
		/**
		 * Parses a color string
		 * Returns {r: xx, g: xx, b:xx}
		 * Taken from jqury plugin interface2
		 */
		function parseColor(color) {
			var namedColors = {
				aqua:[0,255,255],
				azure:[240,255,255],
				beige:[245,245,220],
				black:[0,0,0],
				blue:[0,0,255],
				brown:[165,42,42],
				cyan:[0,255,255],
				darkblue:[0,0,139],
				darkcyan:[0,139,139],
				darkgrey:[169,169,169],
				darkgreen:[0,100,0],
				darkkhaki:[189,183,107],
				darkmagenta:[139,0,139],
				darkolivegreen:[85,107,47],
				darkorange:[255,140,0],
				darkorchid:[153,50,204],
				darkred:[139,0,0],
				darksalmon:[233,150,122],
				darkviolet:[148,0,211],
				fuchsia:[255,0,255],
				gold:[255,215,0],
				green:[0,128,0],
				indigo:[75,0,130],
				khaki:[240,230,140],
				lightblue:[173,216,230],
				lightcyan:[224,255,255],
				lightgreen:[144,238,144],
				lightgrey:[211,211,211],
				lightpink:[255,182,193],
				lightyellow:[255,255,224],
				lime:[0,255,0],
				magenta:[255,0,255],
				maroon:[128,0,0],
				navy:[0,0,128],
				olive:[128,128,0],
				orange:[255,165,0],
				pink:[255,192,203],
				purple:[128,0,128],
				red:[255,0,0],
				silver:[192,192,192],
				white:[255,255,255],
				yellow:[255,255,0]
			};
			if (namedColors[color]) 
				return {
					r: namedColors[color][0],
					g: namedColors[color][1],
					b: namedColors[color][2]
				};
			else if (result = /^rgb\(\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*\)$/.exec(color))
				return {
					r: parseInt(result[1]),
					g: parseInt(result[2]),
					b: parseInt(result[3])
				};
			else if (result = /rgb\(\s*([0-9]+(?:\.[0-9]+)?)\%\s*,\s*([0-9]+(?:\.[0-9]+)?)\%\s*,\s*([0-9]+(?:\.[0-9]+)?)\%\s*\)$/.exec(color)) 
				return {
					r: parseFloat(result[1])*2.55,
					g: parseFloat(result[2])*2.55,
					b: parseFloat(result[3])*2.55
				};
			else if (result = /^#([a-fA-F0-9])([a-fA-F0-9])([a-fA-F0-9])$/.exec(color))
				return {
					r: parseInt("0x"+ result[1] + result[1]),
					g: parseInt("0x" + result[2] + result[2]),
					b: parseInt("0x" + result[3] + result[3])
				};
			else if (result = /^#([a-fA-F0-9]{2})([a-fA-F0-9]{2})([a-fA-F0-9]{2})$/.exec(color))
				return {
					r: parseInt("0x" + result[1]),
					g: parseInt("0x" + result[2]),
					b: parseInt("0x" + result[3])
				};
			else return {r: 255, g: 255, b: 255};
		}
	},
	/**
	 * unpaint a highlight by id
	 */
	unpaint: function(id) {
		var $ = this.context.$;
		var ann = Annotation.find(this.context.getTopLevelContext(), id);
		id = ann.id;
		var klass = H.HIGHLIGHT_ID_CLASS + id;
		//remove highlight label
		//diigolet doesn't paint highlight label now
//		this.context.$('.diigolet-highlightLabel.' + klass).remove();
		$('div.' + klass).remove();
		
		$('em.' + klass).each(function(i, v) {
			var j = $(v);
			j.removeClass(klass);
			if (false && /(^|\s)diigolet-highlightId_[a-zA-Z0-9]+(\s|$)/.test(j.attr('className'))) {
				//there are other highlights using the wrapper
				//thiz.updateStyle(j);
				return;
			} else  {
				//remove the highlight wrapper
				//Insert every child node of highlight wrapper before the wrapper and remove the wrapper.
				var e;
				while (e = v.firstChild) {
					v.parentNode.insertBefore(e, v);
				}
				j.remove();
			}
		});
		
	},

	          
	isTextNode: function(node) {
		return node.nodeType == 3;
	},
	
	/**
	 * Tells if the user has selected some text
	 */
	isTextSelected: function() {
		var select=this.context.win.getSelection();
		return Utils.string.strip(select.toString()).length > 0;
	},
	
	/**
	 * 
	 */
	isSelectionHighlightable: function() {
		var z = this;
		
		var sel = this.context.win.getSelection();
		
		// it must be long enough		
		if (Utils.string.strip(sel.toString()).length <= 3)
			return false;
			
		//every node must not be part of a highlight or other diigo stuff
		var range = sel.getRangeAt(0);
		var on = false, ok = true;
		
		function isOK(n) {
			var className = n.className || '';
			return className.indexOf(H.HIGHLIGHT_CLASS) == -1 && className.indexOf('diigotb') == -1; 	
		}
		
		//are we inside a highlight?
		
		//first, make sure that the start/end node is not and is not a child node of a highlighted node
		var p = sel.anchorNode;
		do {
			if (!isOK(p)) return false;
		} while (p = p.parentNode);
		
		p = sel.focusNode;
		do {
			if (!isOK(p)) return false;
		} while (p = p.parentNode);
				
		var endNode,startNode;
		if(range.endContainer.nodeType==3||range.endContainer.nodeType==4||range.endContainer.nodeType==8){
			endNode=range.endContainer;
		}else{
			endNode=range.endContainer.childNodes[range.endOffset];
			if(!endNode||endNode=="undefined")
				endNode=range.endContainer;
		}
		
		if(range.startContainer.nodeType==3||range.startContainer.nodeType==4||range.startContainer.nodeType==8){
			startNode=range.startContainer;
		}else{
			startNode=range.startContainer.childNodes[range.startOffset];
			if(!startNode||startNode=="undefined")
				startNode=range.startContainer;
		}

		//nodes in between
		//if the user selects reversely (right to left or down to up), selection.anchorNode is the ending node of the selected range.
		//so we user range.startContainer/endContainer for DOMWalker
		new DOMWalker(range.commonAncestorContainer, function(n) {

			if (!on && n == startNode) {
				on = true;
			}
			if (on) {
				//Utils.dump('[Highlight] isSelectionHighlightable on', isOK(n));
				if (n.nodeType == 1 && !isOK(n)){
					ok = false;
				} 
				
				if (!ok) throw DOMWalker.$end;
			}
			
			//turn off
			//if startNode == endNode, all child nodes are to be walked through
			if (on && n == endNode && n != startNode) {
				on = false;
				throw DOMWalker.$end;
			}
		}).walk();
		
		return ok;
	},
	
	/**
	 * Checks whether current selection can be highlighted
	 * @return Object {ok: true|false, txt: 'text', html: 'htmlText', msg: 'message'}
	 */
	checkSelection: function() {
		var t;
		
		var sel, result = {ok: false, txt: '', html: '', msg: '', pen: false};
		
		//used when calculating nth
		var endNode = null;
		//return if no text is selected
		if (!this.isTextSelected()) {
			result.msg = D.strings.getString('highlight.cannotHighlight');
			result.pen=true;
			return result;
		}
		
		sel = this.context.win.getSelection();
		t = sel.getRangeAt(0);
		result.html = this.range2html(t);
		endNode = t.endContainer;
		
		//checks text length
		result.txt = this.html2txt(result.html);
		result.txt2 = Utils.string.strip2(sel.toString());
		
		if (!result.txt || result.txt2.length < 5 || result.txt2.length > 5000) {
			result.msg = D.strings.getString('highlight.cannotHighlight');
			if(result.txt2.length < 5){
				result.pen=true;
			}
			return result;
		}
		
//		if (this.context.$('<div></div>').html(result.html).find('.diigolet').size() > 0) {
//			result.msg = 'You can\'t highlight on Diigolet.';
//			return result;
//		}
		
		if (!this.isSelectionHighlightable()) {
			result.msg = D.strings.getString('highlight.cannotHighlight1');
			return result;
		}
		result.ok = true;

		//calculate nth
		if (endNode) {
			var z = this, docTxt = '', nth = 0;
			new DOMWalker(this.context.doc.body, function(n) {
				if (n.nodeType == 3) {
					var nodeTxt = n.nodeValue.replace(/\s+/g, '');
					//console.log(nodeTxt);
					if (nodeTxt) {
						docTxt += nodeTxt;
					}
				}
				if (n == endNode) throw DOMWalker.$end;
			}, {
				filter: function(n) {
					return z.domSnapshotNodeFilter(n);
				}
			}).walk();
			
			var startIndex = 0, endIndex = 0, n = 0;
			while ((startIndex = docTxt.indexOf(result.txt, endIndex)) > -1) {
				nth++;
				endIndex = startIndex + result.txt.length - 1;
			}
			
			result.nth = nth;
		} else {
			result.nth = 1;
		}
		//alert(result.nth);
		//throw ':)';
		return result;
	},
	
	/**
	 * Whether the selection has changed since last detection
	 * In order to use hasSelectionChanged properly, you should call this method whenever the selection may change.
	 * e.g. mouseup/mousedown
	 */
	detectSelectionChange: function() {
		var last = this._lastSelection;
		var sel = this.context.win.getSelection();
		var cur, changed = false;
		
		//empty selection
		if (sel.isCollapsed || sel.rangeCount == 0) {
			cur = null;
			if (last) changed = true;
		} else {
			cur = Utils.extendFiltered({}, sel, function(k) {
				return 'anchorNode anchorOffset focusNode focusOffset'.split(' ')
				.indexOf(k) > -1;
			});
			
			if (last) {
				//if any property is different, the selection has changed
				changed = 'anchorNode anchorOffset focusNode focusOffset'.split(' ')
				.some(function(k) {
					return last[k] != cur[k];
				});
			} else changed = true;
		}
		
//		this.context.console.log(last, cur);
		
		this._lastSelection = cur;
		
		// don't clear dirty flag here
		this._selectionDirty = this._selectionDirty || changed;
		
//		if (changed) {
//			this.context.console.log('selection chaged');
//		}
//		
//		if (this._selectionDirty) {
//			this.context.console.log('selection dirty');
//		}
		return changed;
	},
	
	/**
	 * Tells whether the selection has changed since last call to this method.
	 * The method also updates _selectionDirty.
	 */
	hasSelectionChanged: function() {
		this.detectSelectionChange();
		var dirty = this._selectionDirty;
		
		//clears the dirty flag
		this._selectionDirty = false;
		return dirty;
	},
	
	range2html: function(range) {
		//var clone=range.cloneRange();
		var div = this.context.doc.createElement('div');
		div.appendChild(range.cloneContents());
		return div.innerHTML;
	},
	
	stripScripts: function(s) {
		return s.replace(new RegExp('(?:<script.*?>)((\n|\r|.)*?)(?:<\/script>)', 'img'), '');
	},
	/**
	 * Extracts text from html string 
	 * All formating and white spaces are discarded
	 */
	html2txt: function(html) {
		var txt = '', thiz = this;
		if (this.context.doc.contentType == 'text/plain') {
			txt = html;
		} else {
			var n = this.context.$('<div></div>').html(this.stripScripts(html)).appendTo(this.context.doc.body).hide();
			
			new DOMWalker(n[0], function(n){
				if (n.nodeType == 3) {
					txt += n.nodeValue;
				}
			}, {
				filter: function(n){
					return n.nodeType == 3 ||
					(n.nodeType == 1 &&
					thiz.tagBlackList.indexOf(',' + n.tagName.toLowerCase() + ',') == -1);
				}
			}).walk();
			n.remove();
		}
		return txt.replace(/\s+/g, '');
	},
	
	/**
	 * We want to use browser's own pretty html 2 text conversion
	 * H.html2txt('<p>fdfdf<br/>fdf</p><p style="display:inline">222</p><ol><li>a</li><li>b</li><li>c</li></ol>')
	 */
	html2txt_pretty: function(html) {
		var txt;
		if (this.context.doc.contentType == 'text/plain') {
			txt = html;
		}
		else {
			//we don't use this.context.$().hide() because opera's selection.toString() doesn't include hidden content
			var t = this.context.$('<div></div>').html(this.stripScripts(html)).appendTo(this.context.doc.body);
			var range;
			if (this.context.win.getSelection && this.context.win.getSelection().removeAllRanges) {
				range = this.context.doc.createRange();
				range.setStart(t[0], 0);
				range.setEnd(t[0], t[0].childNodes.length);
				txt = this.range2txt(range);
			}
			else 
				if (this.context.$.browser.msie) {
					//the TextRange can only contain elements attached to the DOM tree or Invalid argument error will occur
					t.hide().appendTo(this.context.doc.body);
					range = this.context.doc.body.createTextRange();
					range.moveToElementText(t[0]);
					txt = this.range2txt(range);
				}
				else {
					txt = this.node2txt(t[0]);
				}
			t.remove();
		}
		return txt;
	},
	
	/**
	 * A '\n' is added at each element boundary
	 */
	node2txt: function(node) {
		var txt = '', prevType = 3;
		new DOMWalker(node, function(n) {
			if (n.nodeType == 3 && this.context.$(n).parents('script').size() == 0) {
				txt += (prevType == 3 ? '' : '\n') + n.nodeValue;
			}
			prevType = n.nodeType;
		}).walk();
		
		return this.normalizeTxt(txt);
	},
	
	range2txt: function(range) {
		var txt;
		if (this.context.win.getSelection) {
			//borrow selection's toString() method
			var selection = this.context.win.getSelection();
	        selection.removeAllRanges ? selection.removeAllRanges() : selection.collapse();
	        selection.addRange(range);
			
			txt = selection.toString();
	        selection.removeAllRanges ? selection.removeAllRanges() : selection.collapse();
		} else if (this.context.$.browser.msie) {
			txt = range.text;
		} else {
			txt = '';
			alert('Browser not supported!');
		}
        return this.normalizeTxt(txt);
	},
	
	normalizeTxt: function(txt) {
		//string normalization
		//multiple spaces -> one space
		//multipe new lines -> one new line
        return Utils.string.strip(txt)
	        .replace(/\s+/g, function(m) {
	        	return m.match(/[\r\n]/) ? '\n' : ' ';
	        })
	        .replace(/[\r|\n]+/g, '\n');
	}
});
 
// Node.ELEMENT_NODE == 1
// Node.ATTRIBUTE_NODE == 2
// Node.TEXT_NODE == 3
// Node.CDATA_SECTION_NODE == 4
// Node.ENTITY_REFERENCE_NODE == 5
// Node.ENTITY_NODE == 6
// Node.PROCESSING_INSTRUCTION_NODE == 7
// Node.COMMENT_NODE == 8
// Node.DOCUMENT_NODE == 9
// Node.DOCUMENT_TYPE_NODE == 10
// Node.DOCUMENT_FRAGMENT_NODE == 11
// Node.NOTATION_NODE == 12 
/**
 * DOM walker
 * Depth-first
 */
D.walker=DOMWalker;

function DOMWalker(node, func, options) {
	this.options = Utils.extend({
		//Whether to walk in reverse order
		reverse: false,
		whatToShow: -1, //not used yet
		filter: null, //function() {return true;}
	}, options || {});
	this.node = node;
	this.func = func;
	//this.curNode = node;
	this.end = false;
}

//throw this to end walking
DOMWalker.$end = new Error('end walking');
DOMWalker.prototype.walk = function() {
	this._walk(this.node, this.func);
};
DOMWalker.prototype._walk = function (node, func) {
	var o = this.options;
	if (this.end) {
		return;
	}

	if (o.filter && !o.filter(node)) return;
	try {
		func(node);
	} catch(e) {
		if (e === DOMWalker.$end) {
			this.end = true;
			return;
		}
		throw e;
	}
	node = o.reverse ? node.lastChild : node.firstChild;
	while(node) {
		this._walk(node, func);
		node = o.reverse ? node.previousSibling : node.nextSibling;
	}
};

})();