//handles keyboard shortcut

(function() {
	var D = window.diigo;
	var Utils = D.utils;
	var $ = D.$;
	
/////////////////////////////////////////////////////////////////////////////////

/**
 * Keyboard shortcuts
 *  toggle open sidebar
 *  add bookmark
 *  add float note
 * 
 * <key> can't get over firefox's ctrl+d
 * so we more aggressively listen to keydown more
 * 
 */
var Keys = D.Keys = {
	_keys: [], /*[{action: 'sidebar/quickBookmark/bookmark/floatNote', key: 'k', ctrlKey: true/false, altKey: true/false}, ...]*/
	
	init: function() {
		var z = this;
		this.onPrefsChange();
		window.addEventListener('keypress', function(event) {
			if (!D.hasSignedIn()) return;
			
			var charCode = event.charCode;
			for (var i = 0, v; v = z._keys[i], i < z._keys.length; i++) {
				if (v.ctrlKey == event.ctrlKey && v.altKey == event.altKey && String.fromCharCode(charCode) == v.key) {
					Utils.dump('[Hot Key]', v.action);
					if (v.action == 'sidebar') {
						D.handle('toggleSidebar');
					} else if (v.action == 'quickBookmark') {
						D.handle('bookmark', true);
					} else if (v.action == 'bookmark') {
						D.handle('bookmark', false);
					} else if (v.action == 'floatNote') {
						D.handle('addStickyNote');
					} else if (v.action == 'capture') {
						D.handle('capture',document.getElementById("diigotb-tb-capture"));
					} 
					
					event.stopPropagation();
					event.preventDefault();
				}
			}
		}, true);
	},
	
	
	onPrefsChange: function() {
		var z = this;
		this._keys.length = 0;
		
		['sidebar', 'quickBookmark', 'bookmark', 'floatNote', 'capture'].forEach(function(what) {
			var j = $('#row-keys-' + what);
			var enabled = D.prefs.val('keys.' + what + '.enabled');
			var keys = D.Keys.parsePref(D.prefs.val('keys.' + what + '.keys'));
			
			if (enabled && keys) {
				z._keys.push({
					action: what,
					key: keys.key,
					ctrlKey: keys.modifiers.indexOf('control') != -1,
					altKey: keys.modifiers.indexOf('alt') != -1
				});
			}
		});
//		var enabled = D.prefs.val('keys.sidebar.enabled');
//		var keys = this.parsePref(D.prefs.val('keys.sidebar.keys'));
//		var j = $('#key_diigotb-sidebar');
//		j.attr('disabled', !(enabled && keys));
//		if (enabled && keys)
//			j.attr({modifiers: keys.modifiers, key: keys.key});
//			
//		var enabled = D.prefs.val('keys.bookmark.enabled');
//		var keys = this.parsePref(D.prefs.val('keys.bookmark.keys'));
//		var j = $('#key_diigotb-bookmark');
//		j.attr('disabled', !(enabled && keys));
//		if (enabled && keys)
//			j.attr({modifiers: keys.modifiers, key: keys.key});
	},
	
	/**
	 * key must be a displayable char 
	 */
	isValidKey: function(key) {
		return /^[a-zA-Z0-9]$/.test(key);
	},
	
	/**
	 * Parses pref value of keys.xxx.keys
	 * 'control + shift + D'
	 * =>
	 * {modifiers: 'control,shift', key: 'D'}
	 * 
	 * returns null if the pref value is invalid
	 */
	parsePref: function (s) {
		var z = this;
		var modifiers = [], key;
		
		Utils.string.split((s || '').replace(/\s/g, ''), '+').forEach(function(a) {
			if (['shift', 'alt', 'meta', 'control', 'accel'].indexOf(a) != -1)
				modifiers.push(a);
			else {
				//if it's not a modifier, it must be a single char
				if (z.isValidKey(a)) key = a;
			}
		});
		
		if (modifiers.length == 0) return null;
		return {modifiers: modifiers.join(','), key: key};
	}
};


/////////////////////////////////////////////////////////////////////////////////
})();