/**
 * Loads:
 *   models.clip.js
 */

(function() {
	var D = window.diigo;

	var Utils;
	Utils = D["utils"];
	var Class;
	Class = Utils["Class"];
	var Toolbar;
	var WebAPI;
	var DlgIC;
	D.registerRunLater(function () {
		Toolbar = D["Toolbar"];
		WebAPI = D["WebAPI"];
		DlgIC = D["DlgIC"];
	});
		
	var Bookmark = D.Bookmark = Class.create({
		initialize: function(data) {
			Utils.extend(this, {
				url: '',
				title: '',
				urlId: '',
				b_id: '',
				description: '',
				user: '',
				image: null,
				unread: false,
				mode: D.PRIVACY_MODE_PUBLIC, //defaults to public
				tags: [],
				cached: 0, //0/1/2
				
				alert: false,
				
				onlyInGroup: false,
				groups: [],
				lists: [],
				datetime: null,
				saved: false,
				uploading: false,
				contentType: "",
				charset: "",
				fileUploaded: false,
				
				privacyModeSetByUser: false, //flag, whether the user has set the privacy mode. this flag is looked at before applying "private by default while bookmarking". also check saved!
				
				toShareToGroups: null, //temp. array of group names for next saveBookmark
				toShareExistingAnnotations: false, //temp
				toAddToLists: null, //temp
				toShareWithContactLists: null, //temp
				toShareWithAllFriends: null, //temp
				 
				context: null
			}, data || {});
			
			//if mode is not specifed, make it private according to prefs
			//do not check privateByDefault here. check it as late as possible (opening bookmark window/saving bookmark)
			//because a user can change the preference after opening tabs
//			if (data.mode === undefined && D.prefs.val('bookmark.privateByDefault')) {
//				this.mode = D.PRIVACY_MODE_PRIVATE;
//			}
			
			if (!this.context)
				throw '[Bookmark] initialize: invalid context!';
		},
		
		getTitle: function() {
			return this.title || (this.context && this.context.doc && (this.context.doc.title || this.url));
		},
		
		/**
		 * Tells whether the bookmark is in a bookmark list
		 */
		inList: function(listOrListId) {
			var id = listOrListId.id || listOrListId;
			return Utils.array.findIndex(this.lists, function(li) {return li.id == id;}) > -1;
		},
		
		/**
		 * append or update a list
		 */
		addList: function(list) {
			var i = Utils.array.findIndex(this.lists, function(li) {
				return li.id == list.id;
			});
			
			if (i == -1) i = this.lists.length;
			
			this.lists[i] = list;
		},
		
		/**
		 * The group structure is slightly different from those of comment/annotation
		 *	
		 * Merge groups to this bookmark.
		 * This method does not call WebAPI.
		 * If the group(identified by name) to be added already exists, user == D.env.user will overwrite user == ''.
		 * 
		 */
		addGroups: function(groupObjects) {
			var z = this;
			groupObjects.forEach(function(groupObj) {
				var i = Utils.array.findIndex(z.groups, function(g) { 
					return g.name == groupObj.name;
				});
				if (i == -1) {
					z.groups.push(groupObj);
					z.context.fireEvent('bm_addGroups', [z, groupObj.name]);
				} else if (groupObj.user) {
					z.groups[i].user = groupObj.user;
				}
			});
		},
		
		/**
		 * Returns an array of group names shared by current user
		 */
		getGroupNamesSharedByMe: function() {
			var user = D.env.user;
			return this.groups.filter(function(g) {
				return g.user == user;
			}).map(function(g) { return g.name;});
		}
	});
	
	Utils.extend(Bookmark, {
		star: function(context) {
			var bm = context.bookmark;
			
			//add default quick save tag
			var quickSaveTags = D.parseTags(D.prefs.val('bookmark.quickSaveTag') || '');
			if(bm.image){
				image=bm.image;
				offset=context.$?context.$(image).offset():{left:0,top:0};
				var bk = new Bookmark({
					image_title: image.alt,
					image_description: "",
					title:bm.getTitle(),
					description: bm.description,
					url: bm.url,
					mode: D.prefs.val('bookmark.privateByDefault') ? D.PRIVACY_MODE_PRIVATE : D.PRIVACY_MODE_PUBLIC,
					tags: quickSaveTags,
					name: image._name,
					unread: D.prefs.val('bookmark.quickSaveUnread'),
					context: true
				});
				var dtype='',dname='';
				var url=image.src.replace(/\?.*/ig,"");
				
				var ft=url.replace(/(.*\/){0,}([^.]+)\.+/ig,"");
				if(ft){
					dtype=ft;
				}

				var fn=url.replace(/(.*\/){0,}([^.]+).*/ig,"$2");//([^.]+).*
				if(fn.length>0){
					dname=fn;
				}
				
				var tipid=image.__id;
				bm.fileUploaded=true;
				
				D.WebAPI.uploadImage(bk,
						image._type,
						false,
						image.src,
						{left:offset.left,top:offset.top},
						dtype,
						dname,
						context,
						function(data,ct,code){
							if(code==1){
								image.__processing=false;
								image.__saved=true;
								image.__urlId=data.urlId;
								if(context.$){
									var tip=context.$("#"+tipid);
									D.ImagePanel.showTip(tip,image);
								}
								if(image==D.ImagePanel.target){
									D.ImagePanel.j.removeClass("processing").addClass("hassaved");
									D.ImagePanel.j.children().attr("title","Delete this image.");
								}
							}else{
								image.__processing=false;
								image.__saved=false;
								D.PageMessage.reset();
								D.PageMessage.scheduleShow(1);
								setTimeout(function(){
									D.PageMessage.hide();
								},10000);
							}
							setTimeout(function(){
								if(!D.ImagePanel.shown||(D.ImagePanel.shown&&image!=D.ImagePanel.target))
									tip.hide();
							},3000);
						});		
				bm.image=null;
				bk=null;
			}else{
				if (quickSaveTags.length > 0) {
					bm.tags = quickSaveTags;
				}
				
				bm.unread = !!D.prefs.val('bookmark.quickSaveUnread');
				
				WebAPI.saveBookmark(context.bookmark, {}, context);
				//lights up the star immediately to improve UE
				if (context == D.getCurrentContext())
					Toolbar.refresh('star');
			}
	
		}
	});
	
	var Annotation = D.Annotation = Class.create({
		initialize: function(data) {
			data = data || {};
			
			Utils.extend(this, {
				id: null,
				//h_id: null,
				user: '',
				mode: D.PRIVACY_MODE_PRIVATE,
				content: '',
				comments: [],
				saved: false,
				datetime: '',
				painted: false,
				paintedSuccessfully: false,
				type: null,
				groups: [],
				onlyInGroup: false,
				extra: {},
				bMouseOver: false,
				oldMouseOverClass: "",
				
				//toplevel context
				context: null
				/*
				 * extra: {
				 *   left: ,
				 *   top: ,
				 *   winHeight: ,
				 *   winWidth: ,
				 *   nth: ,
				 *   url: ,
				 *   urlId: TODO
				 * }
				 * 
				 */
		}, data);
			
			var defaultExtra = {nth: 0};
			this.extra = Utils.extend(defaultExtra, this.extra || {});
			
			if (!this.context || !this.context.isTopLevel())
				throw('[Annotation] add: Invalid context');
		},
		
		/**
		 * Returns the PageContext for painting (may be inside a frame)
		 */
		getPaintContext: function() {
			var url = this.extra.url;
			if (!url || url == this.context.bookmark.url) {
				//rick was posting extra.url for toplevel documents 
				return this.context;
			} else {
				return this.context.getSubContextByUrl(url);
			}
		},
		
		paint: function() {
			if (!this.context.highlightable) return;
			
			//don't paint unless the paint context document has been loaded
			var paintContext = this.getPaintContext();
			if (!paintContext || !paintContext.doc.__diigo_loaded) {
				Utils.dump('[paint] context not found or document is not loaded');
				return;
			}
			this.constructor.paint(this.context, this);
			this.painted = true;
		},
		
		unpaint: function() {
			this.constructor.unpaint(this.context, this);
			this.painted = this.paintedSuccessfully = false;
		},
		
		del: function(groupName) {
			var t = this.constructor.del ? this.constructor : Annotation;
			return t.del(this.context, this, groupName);
		},
		
		canDelete: function(filterMode) {
			return this.user == D.env.user && this.getComments(filterMode).length == 0;
		},
		
		//whether the current user can add comments to the annotations
		canAddComments: function() {
			//annotated link viewers are not allowed to add comments to private anns
			if (this.isPrivate() && this.user != D.env.user) return false;
			
			if (this.isPublic() && !D.publicAnnotationsAllowed(this.context.bookmark.url)) return false;
			return true;
		},
		
		/**
		 * Adds groups to this annotation.
		 * If the group(identified by idInGroup) to be added already exists, no changes will be made.
		 * 
		 */
		addGroups: function(groupObjects) {
			var z = this;
			groupObjects.forEach(function(groupObj) {
				var i = Utils.array.findIndex(z.groups, function(g) { 
					return g.idInGroup == groupObj.idInGroup;
				});
				if (i == -1) {
					z.groups.push(groupObj);
					z.context.fireEvent('ann_add', [z, groupObj.name]);
				}
			});
			
			this.paint();
		},
		getPageOffset: function(){
			var hi,$=this.context.$;
			if(this.type==D.ANNOTATION_TYPE_TEXT){
				hi= $('em.' + D.Highlighter.HIGHLIGHT_ID_CLASS + this.id);
			}else{
				hi= this.getEle();
			}
			if(hi.size()<=0){
				return {left:0,top:0};
			}
			
			var curwin=this.context.win,pOffset={left:0,top:0},scrollLeft=0,scrollTop=0;
			
			var re=hi.parents('div').filter(function(index){
				var over=curwin.getComputedStyle(this, null).getPropertyValue("position");
				if(over=='relative'){
					pOffset.left+=$(this).offset().left;
					pOffset.top+=$(this).offset().top;
					scrollLeft+=$(this).scrollLeft();
					scrollTop+=$(this).scrollTop();
				}
			});

			
			var offset=hi.offset();
			if(!offset){
				return {left:0,top:0};
			}
			var iconLeft=offset.left-pOffset.left+scrollLeft;
			var iconTop=offset.top-pOffset.top+scrollTop;
			return {left:iconLeft,top:iconTop};

		},
		
		/**
		 * get comments by filter mode
		 */
		getComments: function(filterMode) {
			return this.comments.filter(function(c) {
				return c.matchFilter(filterMode);
			});
		},
		
		jumpHere: function(scroll, edit) {
			Utils.dump('[Annotation] jump here');
			if (arguments.length == 0) scroll = true;
			
			if (!this.paintedSuccessfully) {
		  	D.notifyError(D.strings.getString('highlight.notFound'));
		  	return;
		  }
			
			var paintContext = this.getPaintContext();
			
			var doc = paintContext.doc;
			var $ = paintContext.$;
			var j = (this.type == D.ANNOTATION_TYPE_TEXT ? $('em.' + D.Highlighter.HIGHLIGHT_ID_CLASS + this.id) : this.getEle());
			
			if (!j || !j.size()) {
				//highlight not found on page
				D.notifyError(D.strings.getString('highlight.notFound'));
				return;
			}
			
			//scroll in to view
			if (scroll) {
				var o = j.eq(0).offset();
				paintContext.win.scrollTo(o.left, o.top - 40);
			}
			
			j = j.eq(j.size() - 1);
			o = j.offset();
			//show sticky note dialog
			
			//max because quirks mode and stand compliant mode are different
			var scrollTop = Math.max(doc.documentElement.scrollTop, doc.body.scrollTop);
			var scrollLeft = Math.max(doc.documentElement.scrollLeft, doc.body.scrollLeft);
			
			e = {clientX: o.left - scrollLeft, clientY: o.top - scrollTop + j.height() + 15};
			
			DlgIC.hide();
			
			this.activate(true,this.id);
						
			if (edit || this.comments.length > 0)
				DlgIC.show(e, this, edit);
		},
		
		mouseOvered: function(over,id) {
			var $ = this.context.$;
			var klass = D.Highlighter.HIGHLIGHT_ID_CLASS + id;
			
			$('.' + klass).each(function(i, v) {
				var j = $(v);
				if(over){
					j.addClass("mouseOvered");
				}else{
					j.removeClass("mouseOvered");
				}
			});
			
			/*if(over){
				if(this.oldMouseOverClass.length>0&&this.oldMouseOverClass!=klass){
					$('.' + this.oldMouseOverClass).each(function(i, v) {
						var j = $(v);
						j.removeClass("mouseOvered");
					});
				}
				this.oldMouseOverClass=klass;
			}*/
		},		
		
		/**
		 * Visually makes the highlight "activated" so that the user knows which is the "current" highlight.
		 * e.g. display outline
		 * 
		 * Instead of changing the highlight elements, which is slow, we try to alter css style
		 */
		activate: function(b,id) {
			/*var styleSheet = Utils.array.find(Utils.toArray(this.getPaintContext().doc.styleSheets), function(st) {
				return st.ownerNode.id == 'diigo-activeHighlight';
			});
			
			//cssRules[0] is the dummyRuleForDigg
			if(styleSheet.cssRules.length > 1) styleSheet.deleteRule(1);
			
			if (b) 
				styleSheet.insertRule(Utils.string.showTpl('.diigoHighlight.id_#{0} {outline: 2px dotted #ccc !important;}', [this.id]), 0);
				*/
			return;
			if(this.type==D.ANNOTATION_TYPE_IMAGE||this.type==D.ANNOTATION_TYPE_TEXT){
				this.bMouseOver=b;
				if(!b){
					This=this;
					setTimeout(function(){
						if(!This.bMouseOver){
							This.mouseOvered(false,id);
						}
					},300);
				}else{
					this.mouseOvered(true,id);
				}
			}else{
				
			}
				
			
		}
	
	});
	
	Utils.extend(Annotation, {
		/**
		 * Notice: Though the context for Annotation.add maybe a frame context, ann.context must be always the top level context
		 * @param {PageContext} context either a top level context or a frame context
		 */
		add: function(context, ann, options) {
			//Utils.dump('[Annotation] add', context, ann, options);
			var topContext = context.getTopLevelContext();
			
			if(ann.id&&ann.id.length>0){
				var exist = this.find(topContext,ann.id);
				if(exist!=null)return exist;
			}
			
			if (!context.isTopLevel())
				Utils.dump('[Annotation] add: annotation in a frame!', context.url);
				
			options = Utils.extend({
				dontPaint: false,
				dontSave: false,
				dontSort: false	
			}, options || {});
			
			//FIXME: of course it isObject ?!
			ann.context = topContext;
			
			if (Utils.isObject(ann)) {
				var ctor;
				switch (ann.type) {
					case D.ANNOTATION_TYPE_TEXT:
						ctor = TextHighlight;
						break;
					case D.ANNOTATION_TYPE_IMAGE:
						ctor = ImageHighlight;
						break;
					case D.ANNOTATION_TYPE_FLOATNOTE:
						ctor = FloatNote;
						break;
					case D.ANNOTATION_TYPE_FLASH:
						ctor = D.FlashClip;
						break;
					case D.ANNOTATION_TYPE_VIDEO:
						ctor = D.FlashClip;
						break;
//					case D.ANNOTATION_TYPE_CLIP:
//						ctor = D.Clip;
//						break;
					default:
						throw 'unknown annotation type';
				}
				ann = new ctor(ann);
			}
			
//			Utils.dump('[Annotation] add: new ctor(ann) returned');
			
			//it's unsaved, must be a newly made highlight
			if (!ann.saved) {
				if (!ann.id) {
					
					
					//make annotationId
					ann.id = Utils.string.md5(ann.content + ann.user + topContext.bookmark.urlId + ann.nth)
				}
				ann.extra.color=D.prefs.val('penColor');
				//in a frame
				if (!context.isTopLevel()) {
					ann.extra.url = context.url;
				}
			}
			
			topContext.annotations.push(ann);
			
			if (!options.dontPaint){
				ann.paint();
			}
			
			if (ann.paintedSuccessfully && !options.dontSort)
				Annotation.sort(ann.context);
			
			if (!ann.saved && !options.dontSave) {
				//save to server
				WebAPI.addAnnotation(ann, topContext);
			}
			
//			Utils.dump('[Annotation] add: about to fire events');
			
			if (!ann.onlyInGroup)
				topContext.fireEvent('ann_add', [ann]);
				
			if (ann.groups) {
				ann.groups.forEach(function(g) {
					topContext.fireEvent('ann_add', [ann, g.name]);
				});
			}
			
//			Utils.dump('[Annotation] add: returning');
			return ann;
		},
		
		/**
		 * Remove an annotation from smasher or a group
		 * 
		 * @param groupName If provided, the group will be removed otherwise the smasher annotation will be removed
		 */
		del: function(context, ann, groupName) {
			ann = this.find(context, ann);
			
			if (groupName) {
				//remove from a group
				var i = Utils.array.findIndex(ann.groups, function(g) { 
					return g.name == groupName && g.user == D.env.user;
				});
				
				//TODO delete from group server
				if (ann.saved) {
					//delete from server
					WebAPI.deleteAnnotation(ann, ann.groups[i].idInGroup, context);
				}
				
				ann.groups.splice(i, 1);
				context.fireEvent('ann_del', [ann, groupName]);
			} else {
			  if (ann.onlyInGroup) throw 'Annotation.del: onlyInGroup!';
			  
				//remove from smasher
				ann.onlyInGroup = true;
				ann.mode = D.PRIVACY_MODE_GROUP;
				
				//TODO delete from smasher server	
				if (ann.saved) {
					//delete from server
					WebAPI.deleteAnnotation(ann, null, context);
				}
				context.fireEvent('ann_del', [ann]);
			}
			
			if (ann.onlyInGroup && ann.groups.length == 0) {
				//the annotation is actually "empty" now, kick it away
				ann.unpaint();
				context.annotations.splice(context.annotations.indexOf(ann), 1);
			} else {
				ann.paint();
			}
			
			return ann;
		},
		
		find: function(context, id) {
			id = id.id || id;
			return Utils.array.find(context.annotations, function(a) {
				return a.id == id;
			});
		},
		
		/**
		 * 
		 * @param {Object} an InlineComment object or id
		 */
		findByInlineComment: function(context, c) {
			var id = c.id || c;
			return Utils.array.find(context.annotations, function(ann) {
				return Utils.array.find(ann.comments, function(ic) {
					return ic.id == id;
				});
			});
		},
		
		sort: function(context) {
			//Utils.dump('[Annotation] sorting...');
			context.annotations.sort(function(a, b) {
				if (!a.paintedSuccessfully) return 1;
				if (!b.paintedSuccessfully) return -1;
//				if (a.type != 0) return 1;
//				if (b.type != 0) return -1;
				return a.extra.top - b.extra.top > 0 ? 1 : (a.extra.top - b.extra.top == 0 ? (a.extra.left - b.extra.left) : -1);
			});
		}
	});
	
	var TextHighlight = D.TextHighlight = Class.create(Annotation, {
		initialize: function($super, data) {
			data = data || {};
			$super(data);
		}
	});
	
	Utils.extend(TextHighlight, {
		paint: function(context, ann) {
			ann = Annotation.find(context, ann);
			var paintContext = ann.getPaintContext();
			if (paintContext) {
				paintContext.textHighlighter.paint(ann);
			}
		},
		
		unpaint: function(context, ann) {
			ann = Annotation.find(context, ann);
			var paintContext = ann.getPaintContext();
			if (paintContext) {
				paintContext.textHighlighter.unpaint(ann);
			}
		}
	});
	
	var ImageHighlight = D.ImageHighlight = Class.create(Annotation, {
		initialize: function($super, data) {
			data = data || {};
			$super(data);
			
			this.type = D.ANNOTATION_TYPE_IMAGE;
			
			var defaultExtra = {};
			this.extra = Utils.extend(defaultExtra, this.extra || {});
			
			var paintContext = this.getPaintContext();
			var docUrl = paintContext.doc.URL;
			//if content is not provided, generate it from node
			if (!this.content) {
				if (this._focusedNode) {
					this.content = Utils.string.showTpl('<img src="#{0}" title="#{1}" alt="#{2}" />', [ImageHighlight.makeAbsUrl(this._focusedNode.src, docUrl), this._focusedNode.title, this._focusedNode.alt]);
				} else {
					throw('Must specify either content or _focusedNode for image highlight');
				}
			}
			
			//extract image url from content if not provided in extra
			if (!this.extra.imageUrl) {
				this.extra.imageUrl = ImageHighlight.makeAbsUrl(
					(this.content.match(/<img.*?\s+src=['"]?(.*?)['"]?[\s>]/im) || [])[1] || '',
					docUrl
				);
			}
			
			if (!this.extra.imageUrl)
				throw('invalid imageUrl');
				
			//if it is a newly added one, calculate extra.nth
			if (!this.saved && this._focusedNode) {
				var relUrl = ImageHighlight.makeRelUrl(this.extra.imageUrl, docUrl);
				var absUrl = ImageHighlight.makeAbsUrl(this.extra.imageUrl, docUrl);
					
				var images = paintContext.doc.getElementsByTagName('img');
					
				var n = 0; // the "nth" counter
				for (var i = 0, len = images.length, img; img = images[i], i < len; i++) {
					//a quick indexOf to improve performance
					if (img.src.indexOf(relUrl) > -1 && absUrl == ImageHighlight.makeAbsUrl(img.src, docUrl)) {
						
						if (img == this._focusedNode) {
							this.extra.nth = n;
							break;
						}
						n++;
					}
				}
			}
		},
		
		/**
		 * Gets the jquery object for this highlight
		 */
		getEle: function() {
			var j = null;
			var paintContext = this.getPaintContext();
			if (paintContext) {
				var $ = paintContext.$;
				j = $('img.' + D.Highlighter.HIGHLIGHT_ID_CLASS);
				
				//by comparing absolutized src
				if (j.size() == 0) {
					var relUrl = ImageHighlight.makeRelUrl(this.extra.imageUrl, paintContext.doc.URL);
					var absUrl = ImageHighlight.makeAbsUrl(this.extra.imageUrl, paintContext.doc.URL);
					
					//Utils.dump('[ImageHighlight] getEle: relUrl', relUrl, ' absUrl', absUrl);
					var images = paintContext.doc.getElementsByTagName('img');
					
					var n = 0; // the "nth" counter
					for (var i = 0, len = images.length, img; img = images[i], i < len; i++) {
						//a quick indexOf to improve performance
						if (img.src.indexOf(relUrl) > -1 && absUrl == ImageHighlight.makeAbsUrl(img.src, paintContext.doc.URL)) {
							
							//Utils.dump('[ImageHighlight] getEle: n', n, n == this.extra.nth);
							if (n == this.extra.nth)
								return $(img);
							n++;
						}
					}
					
				}
			}
			return j;
		}
	});
	
	Utils.extend(ImageHighlight, {
		paint: function(context, ann) {
			ann = Annotation.find(context, ann);
			
			var paintContext = ann.getPaintContext();
			if (paintContext) {
				var $ = paintContext.$;
				var j = ann.getEle();
				
				if (!j) return;
				var comments=ann.getComments(D.prefs.val('filter'));
				
				var classId=D.Highlighter.HIGHLIGHT_ID_CLASS + ann.id;
				
				j.addClass(D.Highlighter.HIGHLIGHT_CLASS)
				.addClass(classId)
				.addClass(D.Highlighter.HIGHLIGHT_TYPE_CLASS + ann.type)
				.removeClass('yellow blue green pink')
				.addClass((ann.extra.color&&ann.extra.color.length>0)?ann.extra.color:'yellow')
				.toggleClass('commented', comments.length > 0);
				//.toggleClass('public', ann.isPublic())
				//.toggleClass('group', ann.inAnyGroups());
				
				if (j.size() > 0) {
					var offset = j.offset();
					ann.extra.top = offset.top;
					ann.extra.left = offset.left;
				}
				var z=this;
				
				ann.painted = true;
				ann.paintedSuccessfully = j.size() > 0;
				
				function addIconClass(icon, ann) {
					var H=D.Highlighter;
					$(icon)
					.addClass(H.HIGHLIGHT_ICON_CLASS)
					.addClass(H.HIGHLIGHT_ID_CLASS + ann.id)
					.addClass(H.HIGHLIGHT_TYPE_CLASS + D.ANNOTATION_TYPE_ICON)
					.addClass("ImageIcon")
					.toggleClass('public', ann.isPublic())
					.toggleClass('private', ann.isPrivate())
					.toggleClass('group', ann.inAnyGroups())
					.toggleClass('commented', comments.length > 0)
					.toggleClass('diigoEdit', comments.length == 0)
					//.css({left:0,top:0})
					;
					if(comments.length > 0){
						$(icon).html("<span class='"+H.HIGHLIGHT_ID_CLASS + ann.id+" "+H.HIGHLIGHT_ICON_CLASS+" "+H.HIGHLIGHT_TYPE_CLASS + D.ANNOTATION_TYPE_ICON+"'>"+comments.length+"</span>");
						$(icon).attr("title",D.AnnotationContextMenu.tipMsg(ann,false));
						var offset=ann.getPageOffset();
						$(icon).css({left:offset.left,top:offset.top});
					}else{
						$(icon).html("");
						$(icon).attr("title","");
					}
					return icon;	
				}			
				
				if($('div.'+classId).size()<=0&&j.size() > 0){
					var icon=$(paintContext.doc.createElement('div'));
					icon.insertBefore(j);
					//j.parent().append(icon);
					addIconClass(icon,ann);
				}else{
					addIconClass('div.'+classId,ann);
				}

			}
		},
		
		unpaint: function(context, ann) {
			ann = Annotation.find(context, ann);
			
			var j = ann.getEle();
			
			if (!j) return;
			
			j.removeClass(D.Highlighter.HIGHLIGHT_ID_CLASS + ann.id)
			.removeClass(D.Highlighter.HIGHLIGHT_CLASS)
			.removeClass('commented');
			//.removeClass('public')
			//.removeClass('group');
			ann.context.$('div.' + D.Highlighter.HIGHLIGHT_ID_CLASS + ann.id).remove();
		},
		
		/**
		 * Returns the base url
		 * @return {String} with trailing /
		 * need testing!!!
		 */
		getBaseUrl: function(url, siteRoot) {
			var u = url;
			//TODO ^[a-zA-Z]+(:\/\/)
			var i = u.replace(/:\/\//, '123')[(siteRoot ? 'i' : 'lastI') + 'ndexOf']('/');
			if (i > 0) {
				u = u.slice(0, i);
			}
			return u + '/';
		},
		
		/**
		 * Returns a relative url
		 * need testing!!!
		 */
		makeRelUrl: function(url, docUrl) {
			//return directly if it is already a relative url
			if (!/^[a-zA-Z]+:\/\/|^\//.test(url)) return url;

			if (url.charAt(0) == '/') {
				//extend starting slash to full site root
				url = url.replace('/', this.getBaseUrl(url, true));
			}
			
			return url.replace(this.getBaseUrl(docUrl, false), '');
		},
		
		/**
		 * Returns an absolute url
		 * need testing!!!
		 */
		makeAbsUrl: function(url, docUrl) {
			if (url.indexOf('://') > 0) return url;
			var root = url.charAt(0) == '/';
				
			return this.getBaseUrl(docUrl, root) + (root ? url.slice(1) : url);
		}
	});
	
	var FloatNote = D.FloatNote = Class.create(Annotation, {
		initialize: function($super, data) {
			data = data || {};
			$super(data);
			
			this.saving = false; //flag, while a floatnote is being saved(waiting webapi response), closing the inline comment window should not remove the floatnote
			
			//generate a random id if necessary
			if (!this.id) {
				this.id = Utils.string.md5(Math.random() + Math.random() + new Date().getTime().toString());
			}
			
			this.type = D.ANNOTATION_TYPE_FLOATNOTE;
		}, 
		
		/**
		 * Add missing left, top
		 * update winWidth and winHeight
		 */
		fixExtra: function () {
			var topContext = this.context;
			var doc = this.context.doc;
			
			//avoid newly added floatnotes being piled in the same position
			var existingFloatNoteCount = topContext.annotations.filter(function(ann) {return ann.type == D.ANNOTATION_TYPE_FLOATNOTE;}).length;
			var offset = 5 * existingFloatNoteCount;
			
			//max because quirks mode and stand compliant mode are different
			var scrollTop = Math.max(doc.documentElement.scrollTop, doc.body.scrollTop);
			var scrollLeft = Math.max(doc.documentElement.scrollLeft, doc.body.scrollLeft);
			
			var docHeight = Math.max(doc.documentElement.clientHeight, doc.body.clientHeight);
			var docWidth = Math.max(doc.documentElement.clientWidth, doc.body.clientWidth);
			
			
			var defaultExtra = {
				left: offset + scrollLeft + 10, 
				top: offset + scrollTop + 50
			};
			
			this.extra = Utils.extend(defaultExtra, this.extra || {}, {winWidth: docWidth, winHeight: docHeight});
		},
		
		getEle: function() {
			var j = null;
			
			var paintContext = this.getPaintContext();
			if (paintContext) {
				var $ = paintContext.$;
				j = $('.' + D.Highlighter.HIGHLIGHT_ID_CLASS + this.id);
			}
			
			return j;
		}
	});
	
	Utils.extend(FloatNote, {
		paint: function(context, ann) {
			Utils.dump('[FloatNote] paint: called');
			ann = Annotation.find(context, ann);
			
			ann.fixExtra();
			
			var paintContext = ann.getPaintContext();
			
			if (paintContext) {
				var $ = paintContext.$;
				
				if (ann.painted) {
					var j = ann.getEle();
					if (j.length)
						j.find('span').text(ann.comments.length || '');
				} else {
					var j = $('<div><span></span></div>')
					.addClass(D.Highlighter.HIGHLIGHT_CLASS)
					.addClass(D.Highlighter.HIGHLIGHT_ID_CLASS + ann.id)
					.addClass(D.Highlighter.HIGHLIGHT_TYPE_CLASS + ann.type)
					
					.css({left: ann.extra.left, top: ann.extra.top}).appendTo(paintContext.doc.body)
					.find('span').text(ann.comments.length || '');
					
					this.bindEvents(ann.context, ann);
					
					ann.paintedSuccessfully = true;
				}
			}
			
			ann.painted = true;
		},
		
		unpaint: function(context, ann) {
			ann = Annotation.find(context, ann);
			
			var paintContext = ann.getPaintContext();
			if (paintContext) {
				var $ = paintContext.$;
				var j = ann.getEle();
				if (j) {
					j.unbind().remove();
					ann.painted = ann.paintedSuccessfully = false;
				}
			}
		},
	
		bindEvents: function(context, ann) {
			ann = Annotation.find(context, ann);
			
			var paintContext = ann.getPaintContext();
			if (paintContext) {
				var $ = paintContext.$;
				var j = ann.getEle();
				
				//HACK
				//mouseover div/span causes mouseout event on div to trigger
				//cancel the event in this case
				j.mouseout(function(e) {if (e.relatedTarget == this.firstChild) return false;});
				
				new $.Draggable(j, {
					cursor: 'default',
					beforeDrag: function() {
						context.clickedOnDiigolet = true;
						context.draggingFloatNote = true;
						
						D.utils.dump('[FloatNote] before drag: cancel show');
						
						//add new sticky note and then dragging empty bubble shouldn't hide DlgIC
						if (ann.saved)
							DlgIC.hide();
					},
					afterDrag: function(event) {
						D.utils.dump('[FloatNote] after drag');
						ann.extra.left = parseInt(j.css('left'));
						ann.extra.top = parseInt(j.css('top'));
						
						context.draggingFloatNote = false;
						
						if (ann.user == D.env.user) {
							WebAPI.updateExtra(ann, context);
							Annotation.sort(context);
						}
						//DlgIC.scheduleShow(event);
	//					DlgSticky.show();
	//					if (DlgSticky.editingHighlight == ann) {
	//						DlgSticky.updatePos(); 
	//					}
	//					if (ann.saved && checkSignIn() && ann.user == Ctx.user) {
	//						WebAPI.updateFloatNote(ann.id);
	//					}
					}
				});
			}
		}
	});
	
	
	//load subscripts
	var subscriptLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
						  .getService(Components.interfaces.mozIJSSubScriptLoader);
						  
	subscriptLoader.loadSubScript('chrome://diigotb/content/models.clip.js');
	
	
	var Comment = D.Comment = Class.create({
		initialize: function(data) {
			Utils.extend(this, {
				id: null,
				user: '',
				mode: D.PRIVACY_MODE_PRIVATE,
				saved: false,
				datetime: '',
				groups: [],
				onlyInGroup: false,
				
				context: null
			}, data || {});
			
			if (this.onlyInGroup && !(this.id > 0)) {
				this.id = Math.random().toString().substr(2);
			}
			
			if (!this.context)
				throw '[Comment] initialize: Invalid context';
		},
		
		del: function(groupName) {
			return this.constructor.del(this.context, this, groupName);
		},
		
		canDelete: function() {
			return this.user == D.env.user;
		},
		
		/**
		 * Adds groups to this comment.
		 * This method does not call WebAPI.
		 * If the group(identified by idInGroup) to be added already exists, no changes will be made.
		 * 
		 */
		addGroups: function(groupObjects) {
			var z = this;
			groupObjects.forEach(function(groupObj) {
				var i = Utils.array.findIndex(z.groups, function(g) { 
					return g.idInGroup == groupObj.idInGroup;
				});
				if (i == -1) {
					z.groups.push(groupObj);
					z.context.fireEvent(((z instanceof PageComment) ? 'pc' : 'ic') + '_add', [z, groupObj.name]);
				}
			});
		},
	});
	
	var PageComment = D.PageComment = Class.create(Comment, {
		initialize: function($super, data) {
			$super(data);
		}
	});
	
	Utils.extend(PageComment, {
		add: function(context, c) {
			c.context = context;
			
			if (!(c instanceof this)) c = new this(c);
			
			context.pageComments.push(c);
			
			if (!c.saved) {
				throw '[PageComment] add: adding an unsaved comment is probably and error';
			}
			
			context.fireEvent('pc_add', [c]);
			return c;
		},
		
		del: function(context, c, groupName) {
			c = this.find(context, c);
			
			if (groupName) {
			  //remove from a group
			  var i = Utils.array.findIndex(c.groups, function(g) { 
					return g.name == groupName && g.user == D.env.user;
				});
				
				if (c.saved) {
					//delete from server
					WebAPI.deletePageComment(c, c.groups[i].idInGroup, context);
				}
				
				c.groups.splice(i, 1);
				context.fireEvent('pc_del', [c, groupName]);
			} else {
			  //remove from smasher
			  if (c.onlyInGroup) throw 'PageComment.del: onlyInGroup!';
  
				//remove from smasher
				c.onlyInGroup = true;
				c.mode = D.PRIVACY_MODE_GROUP;
				
				if (c.saved) {
					//delete from server
					WebAPI.deletePageComment(c, null, context);
				}
				context.fireEvent('pc_del', [c]);
			}
			
			if (c.onlyInGroup && c.groups.length == 0) {
				//the comment is actually "empty" now, kick it away
				context.pageComments.splice(context.pageComments.indexOf(c), 1);
			}
			
			return c;
		},
		
		find: function(context, id) {
			id = id.id || id;
			return Utils.array.find(context.pageComments, function(c) {
				return c.id == id;
			});
		},
		
		/**
		 * Returns comments that have been shared to no groups
		 */
//		getNoneGroupComments: function(context) {
//			return context.pageComments.filter(function(c) {
//				return !c.inAnyGroups();
//			});
//		},
		
//		/**
//		 * Returns comments shared to a group
//		 */
//		getGroupComments: function(context, groupName) {
//			return context.pageComments.filter(function(c) {
//				return c.inGroup(groupName);
//			});
//		}
	});
	
	var InlineComment = D.InlineComment = Class.create(Comment, {
		initialize: function($super, data) {
			data = Utils.extend({
				annotationId: null
			}, data);
			$super(data);
			
		},
		
		getAnnotation: function() {
			var context = this.context;
			if (!this.annotation) {
				this.annotation = Annotation.find(context, this.annotationId);
			}
			return this.annotation;
		}
	});
	
	Utils.extend(InlineComment, {
		add: function(context, c) {
			c.context = context;
			
			if (Utils.isObject(c)) c = new this(c);
			
			//a reference to annotation
			var ann = c.getAnnotation();
			if(ann == null) return;
			
			if (ann.getComments('_noGroups').length > 0) {
				//if the annotation already has comments
				//the new comment must not have a different mode
				if (c.mode != ann.mode && c.mode != D.PRIVACY_MODE_GROUP)
					c.mode = ann.mode;
			} else {
				//annotation mode follows the mode of the first comment
				if (c.mode != ann.mode && c.mode != D.PRIVACY_MODE_GROUP) {
					ann.mode = c.mode;
					context.fireEvent('ann_changeMode', [ann]);
				}
			}
			
			ann.comments.push(c);
			
			if (!c.saved) {
				//save to server
				//add is called by WebAPI callback
				//nothing to do here
			}
			
			//repaint(update)
			if (ann.paintedSuccessfully)
				ann.paint();
				
			context.fireEvent('ic_add', [c]);
			return c;
		},
		
		del: function(context, c, groupName) {
			c = this.find(context, c);
			Utils.dump('[InlineComment.del]', c, groupName, c.onlyInGroup ? 'onlyInGroup' : '');
			
			var callback = function() {
				if (f) f();
			};
			
			if (groupName) {
			  //remove from a group
			  var i = Utils.array.findIndex(c.groups, function(g) { 
					return g.name == groupName && g.user == D.env.user;
				});
				
				if (c.saved) {
					//delete from server
					WebAPI.deleteInlineComment(c, c.groups[i].idInGroup, context, callback);
				}
				
				c.groups.splice(i, 1);
				context.fireEvent('ic_del', [c, groupName]);
			} else {
			  //remove from smasher
			  if (c.onlyInGroup) throw 'InlineComment.del: onlyInGroup!';
			  
				//remove from smasher
				c.onlyInGroup = true;
				c.mode = D.PRIVACY_MODE_GROUP;
				
				if (c.saved) {
					//delete from server
					WebAPI.deleteInlineComment(c, null, context, callback);
				}
				context.fireEvent('ic_del', [c]);
			}
			
			if (c.onlyInGroup && c.groups.length == 0) {
				//the comment is actually "empty" now, kick it away
				var ann = c.getAnnotation();
				ann.comments.splice(ann.comments.indexOf(c), 1);
				
				//if the annotation has no comments. make sure it is private
				if (ann.isPublic() && ann.getComments('_smasher').length == 0) {
					ann.mode = D.PRIVACY_MODE_PRIVATE;
					context.fireEvent('ann_changeMode', [ann]);
				}
				
				//repaint(update)
				if (ann.paintedSuccessfully)
					ann.paint();
				
				//remove "empty bubble"
				var sectionComments = ann.getComments(groupName ? groupName : '_smasher');
				if (ann.type == D.ANNOTATION_TYPE_FLOATNOTE && sectionComments.length == 0) {
					var f = function() {
						ann.saved=false;
						ann.del(groupName);
						ann.saved=true;
					};
				}
			}
			
			return c;
			
		},
		
		find: function(context, id) {
			id = id.id || id;
			var ann = Annotation.findByInlineComment(context, id);
			if (!ann) return null;			
			
			return Utils.array.find(ann.comments, function(c) {
				return c.id == id;
			});
		}
	});
	
	
	/**
	 * mixin  
	 */
	[Bookmark, Annotation, Comment].forEach(function(v) {
		Class.extend(v, {
			isPublic: function() {
				return this.mode == D.PRIVACY_MODE_PUBLIC && !this.onlyInGroup;
			},
			 
			isPrivate: function() {
				return this.mode == D.PRIVACY_MODE_PRIVATE && !this.onlyInGroup;
			},
			
			/**
			 * Tells whether the item has been shared to a group (by a user)
			 * @param {String} user Optional, if specified, it will be compared with group user
			 */
			inGroup: function(groupName, user) {
				var z = this;
				return z.groups.some(function(g) {
			   		return g.name == groupName && (user ? g.user == user : true);
				});
			},
			
			inAnyGroups: function() {
				return this.groups.length > 0;
			},
			
			isMine: function() {
				return this.user == D.env.user;
			},			
			/**
			 * Tells whether the item can pass given filter mode
			 */
			matchFilter: function(filterMode) {
				try{
			  if (!filterMode) throw 'matchFilter: illegal filter mode!';
			  var select=D.isSelectedURL(D.getCurrentContext().bookmark.url);
			  if(select&&this.isPublic()){
				  return false;
			  }
			  if (filterMode == '_all')
		    	return this.isPublic()||this.isPrivate()||this.inAnyGroups();
			  else if (filterMode == '_nonepublic')
			    return !this.isPublic();
			  else if (filterMode == '_smasher')
		    	return this.isPublic() || this.isPrivate();
			  else if (filterMode == '_public') 
				return this.isPublic();
			  else if (filterMode == '_private')
		    	return this.isPrivate();
			  else 
				return this.inGroup(filterMode); //filter is a group name
				}catch(e){
					Utils.dump('[matchFilter]'+e);
					return false;
				}
			}
		});	
	});
})();