(function() {
	var D = window.diigo;
	

	var Utils;
	Utils = D["utils"];
	var Class;
	Class = Utils["Class"];

	var Bookmark;
	var TextHighlight;
	var Annotation;
	var Toolbar;
	var Annotation;
	var Csm;
	var DlgIC;
	var PageContext;
	D.registerRunLater(function () {
		Bookmark = D["Bookmark"];
		Annotation = D["Annotation"];
		TextHighlight = D["TextHighlight"];
		Toolbar = D["Toolbar"];
		Csm=D["Csm"];
		DlgIC=D["DlgIC"];
		PageContext=D["PageContext"];
		Sidebar = D["Sidebar"];
	});
		
	/**
	 * Holds context data for a document
	 * @param {Object}
	 * required keys: doc
	 */
	var PageContext = D.PageContext = Class.create({
		initialize: function(doc) {
			Utils.extend(this, {
				doc: doc,
				topDoc: doc.defaultView.top.document,
				win: doc.defaultView,
				url: doc.URL,
				
				isPdfReader: D.isLocalPDFReader(doc.URL),
				highlightable: D.isDocumentHighlightable(doc),
				textHighlighter: null,
				$: null
			});

			if (this.highlightable){
				this.textHighlighter = new D.TextHighlighter(this);
				this.$ = D.j(doc.defaultView);
			}
			
			if(this.isPdfReader){
				this.$ = D.j(doc.defaultView);
			}
			
			//only top level windows can have a real PageContext
			//PageContexts for frames are merely for highlighting
			if (this.isTopLevel()) {
				Utils.extend(this, {
					bookmark: null,
					bookmarkInfoLoaded: false, //flag,
					bookmarkInfoLoading: false,
					
					pageComments: [],
					pageCommentsLoaded: false,
					pageCommentsLoading: false,
					
					annotations: [],
					annotationsLoaded: false, //flag
					annotationsLoading: false,
					
					recommendedTags: [],
					recommendedTagsLoaded: false, //flag
					
					//readers
					pageReaderCount: 0,
					siteReaderCount: 0,
					pageReaders: [],
					siteReaders: [],
					siteDomain: '',
					readersLoaded: false, //flag
					readersLoading: false, //flag
					
					imageClipable: false,
					
					annotatedLinkCache: {},
					captureItem:{
						isCaptureMode:false,
						editpanel:null,
						colorpanel:null,
						bgImageUrl:null,
						inputPanel:null,
						urlId:'',
						pageDoc:doc,
						operation:0,//1:rect,2:round,3:text,4:arrow
						color:'capture-red',
						operationList:[],
						drawReady:false,
						textInputing:false,
						captureleft:0,
						capturetop:0,
						capturewidth:0,
						captureheight:0,
						beginDraw:false,
						scrollTop:0,
						scrollLeft:0,
						offsetY:31,
						currentBtnX:28,
						resizeStartX:0,
						resizeStartY:0,
						iMovingBox:false,
						iResizingBox:false,
						mAdjustHorz:0,
						mAdjustVert:0,
						rect:{left:0,top:0,right:0,bottom:0},
						startX:0,
						startY:0,
						endX:0,
						endY:0,
						drawStart:false,
						selectionBeginresize:false
					},
					
					loading: false,
					loaded: false,
					isModerator: false,
					isHighlightPen: false,
					
					draggingFloatNote: false,
					filterMode: D.prefs.val('filter'), 
					
					_events: [],
					supressEvents: false
				});
				
				this.bookmark = new Bookmark({
					url: (this.isPdfReader?D.getOriginalPDFURL(this.url):this.url),
					title: doc.title,
					contentType: doc.contentType,
					domain: this.isPdfReader?"":doc.domain,
					//TODO: get description 
					description: function() {return '';}(),
					context: this
				});				
				
				this.imageClipable = this.isPdfReader?false:(!this.isDiigoSite(doc.domain));//&&this.highlightable;
				
				//special urls
				//cached page
				//proxied page
				if (/^https?:\/\/([^.]+\.)?diigo\.(com|cn)\/cached\?url=/i.test(this.bookmark.url)) {
					this.bookmark.url = D.utils.string.parseQuery(this.bookmark.url).url;
				}
			}
			
			
			//bind event handlers
//			Utils.each(this.eventHandlers, function(v, k) {
//				this[k] = Utils.func.bind(v, this);
//			}, this);
		},
		
		/**
		 * Page context event dispatcher
		 * Sidebar and DlgIC can listener for these events to get notified.
		 * 
		 * UI action -> call model method -> context event -> UI update
		 * model change -> context event -> UI update
		 * 
		 * Event              listener
		 * 'pc_add'           listener(context, pc, groupName)
		 * 'pc_del'           listener(context, pc, groupName)
		 * 
		 * 'ic_add'           listener(context, ic, groupName)
		 * 'ic_del'           listener(context, ic, groupName)
		 * 
		 * 'ann_add'          listener(context, ann, groupName)
		 * 'ann_del'          listener(context, ann, groupName)
		 * 'ann_changeMode'   listener(context, ann)
		 */
		/**
		 * example callback
		 * listener(context, eventSpecificArg, ...)
		 */
		addEventListener: function(eventName, listener) {
			if (!this._events[eventName])
				this._events[eventName] = [];
				
			var listenerList = this._events[eventName];
			if (listenerList.indexOf(listener) == -1)
				listenerList.push(listener);
		},
		
		removeEventListener: function(eventName, listener) {
			var listenerList = this._events[eventName];
			if (listenerList) {
				if (arguments.length == 2) {
					//listener provided
					var i = listenerList.indexOf(listener);
					if (i > -1) {
						listenerList.splice(i);
					}
				} else {
					//listener provided, remove all
					delete this._events[eventName];
				}
			}
		},
		
		fireEvent: function(eventName, paramArray) {
			if (this.supressEvents) return;
			
			Utils.dump('[context event]', eventName);
			var listenerList = this._events[eventName];
			if (listenerList) {
				for (var i = 0, listener, len = listenerList.length; listener = listenerList[i], i < len; i++) {
					(typeof listener == 'function' ? listener : listener['on' + eventName])
					.apply(listener, [this].concat(paramArray));
				}
			}
		},
		
		/**
		 * Tells whether the associated document is top level, not a frame or iframe
		 */
		isTopLevel: function() {
			return this.doc == this.topDoc||/https?:\/\/www.google/.test(this.url);	
		},
		isDiigoSite: function(domain){
			if(domain){
				return domain.indexOf(D.urls.getCookieDomain())>=0;
			}
			return false;
		},
		
		
		getTopLevelContext: function() {
			return this.topDoc.__diigo;
		},
		
		/**
		 * Recursively collect all embedded frames
		 * @return {Array}
		 */
		getSubFrames: function(filterFunc) {
			var f = filterFunc || function() {return true;};
			var win = this.doc.defaultView;
			return recur(win);
			
			function recur(w) {
				var r = [];
				for (var i = 0, fr; fr = w.frames[i], i < w.frames.length; i++ ) {
					if (f(fr)) {
						r.push(fr);
						Utils.array.concatMe(r, recur(fr));
					}
				}
				return r;
			}
		},
		
		/**
		 * Returns an array of subcontexts of a top level context
		 */
		getSubContexts: function(filterFunc) {
			if (!this.isTopLevel()) {
				throw '[Page context] getSubContexts: called on non-toplevel context';
			}
			var f = filterFunc || function() {return true;};
			
			var contexts = [];
			this.getSubFrames().forEach(function(fr) {
				var context = fr.document.__diigo;
				if (context && f(context))
					contexts.push(context);
			});
			return contexts;
		},
		
		/**
		 * Get the first sub context with specifed url
		 */
		getSubContextByUrl: function(url) {
			if(this.url==url)
				return this;
			return this.getSubContexts(function(context) {
				return context.url == url;
			})[0];
		},
		getSubContextByDoc: function(doc) {
			if(this.doc===doc)
				return this;
			return this.getSubContexts(function(context) {
				return context.url == doc.URL;
			})[0];
		},		
		log: function(s) {
			var args = [].slice.call(arguments);
			
			var console = (this.win.wrappedJSObject || this.win).console;
			if (console) {
				console.log.apply(console, args);
//			} else if (console = window.content.console) {
//				args.unshift('[from: ' + this.win.location.href + ']');
//				console.log.apply(console, args);
			} else {
				Utils.dump('[from: ' + this.doc.URL + ']\n' + s);
			}		
		},
		
		getFilteredItems: function (items) {
			var szfilter=D.prefs.val('filter');
			  return items.filter(function (item) {
			    var re= item.matchFilter(szfilter);
			    return re;
			  });
		},
		
		/**
		 * Applicable for top level context
		 * Paints all unpainted annotations
		 * 
		 * This is reenterable.
		 */
		paintAllAnnotations: function(onlyInFrames) {
			if (D.prefs.val('hl.dontPaint')) return;
			var z = this;
			
			z.getFilteredItems(z.annotations).forEach(function (ann) {
				try{
				if (onlyInFrames && (!ann.extra.url || ann.extra.url == z.bookmark.url)) return;

				if (!ann.painted) {
					ann.paint();
				}
				}catch(e){throw(e);}
			});

			Annotation.sort(this);
		},
		
		unpaintAllAnnotations: function() {
			this.annotations.forEach(function(ann) {
				if (ann.painted && ann.paintedSuccessfully) {
					ann.unpaint();
				}
			});
		},
		
		documentMouseUp: function(e) {
			//this.log('mouseup');
			
			var z = this;
			
			D.runtime._focusedContext = z;
			D.runtime._focusedNode = e.target;
			if(z.isPdfReader){
				if(D.runtime._focusedNode.id == "diigo-i-know"){
	                z.$(D.runtime._focusedNode).parent().hide();
	                D.prefs.val('isPdfTipClicked', true);
				}else if(D.runtime._focusedNode.id == "diigo-enter-pdf"){
					Components.utils.import("resource://gre/modules/AddonManager.jsm");
					AddonManager.getAddonByID('{fc2b8f80-d9a5-4f51-8076-7c7ce3c67ee3}', function(addon){
						var addonLocation = addon.getResourceURI("").QueryInterface(Components.interfaces.nsIFileURL).file.path;
						z.win.location.href="chrome:\/\/diigotb\/content\/pdf\/viewer.html?file="+z.url;
					});
				}else{
				}
			}
			
			
				//left click highlightable document
				//show csm?
				if (this.highlightable && 
					e.button == 0 && 
					e.target.ownerDocument.designMode.toLowerCase() != 'on') {
					
					var h = this.textHighlighter;
					
					//have to clone
					var e = {screenX: e.screenX, screenY: e.screenY};
					
					//setTimeout because selection clears after event triggering
					setTimeout(function() {
						if (h.hasSelectionChanged() &&
							h.isSelectionHighlightable()){
							if(!z.isHighlightPen&&Csm.okToPopup()){
								Csm.show(e);
							}else if(z.isHighlightPen&&h.checkSelection().ok){
								D.handle('highlight');
							}
						}
						
					}, 1);
				}
				var win=D.runtime._focusedNode.ownerDocument;
				if(D.runtime._focusedNode.id == "screenshot"){
					D.handle('capture',D.runtime._focusedNode);
					
					D.runtime._focusedNode.checked=true;
				}
				
		},
		
		documentMouseDown: function(e) {
			//this.log('mousedown');
			
			var z = this;
			//left click
			if (this.highlightable && e.button == 0) {
				//setTimeout because selection clears after event triggering
				setTimeout(function() {
					z.textHighlighter.detectSelectionChange();
				}, 1);
			}
		},
		
		documentClick: function(event) {
			//			if (D.runtime.inClipMode) { 
//				var ele = D.runtime.clip_currentNode;
//				if (!ele) return;
//				D.notifyOK('Clipped');
//				
//				event.stopPropagation();
//				event.preventDefault();
//			}
		},
		
		documentMouseOver: function(e) {
			//mouseover highlight
			var z = this;
			
//			if (D.runtime.inClipMode) { 
//				z.documentMouseOver_clip(e);
//				return;
//			}
			
			z.documentMouseOver_highlight(e);
		},
		
//		documentMouseOver_clip: function(event) {
//			var z = this;
//			if (!z.highlightable) return;
//			
//			event.stopPropagation();
//			event.preventDefault();
//			
//			var ele = event.target; 
//			
//			if (Clip.isClippable(ele)) {
//				ele.style.outline = 'solid 4px orange';
//				
//				D.runtime.clip_currentNode = ele;
//			}
//		},
		
		documentMouseOver_highlight: function(event) {
			var z = this;
			var topContext = z.getTopLevelContext();
			
			
			if(D.prefs.val('showimagepanel')&&topContext.imageClipable/*||bgimage!='none'*/)
			{
				var tar=event.target;
				var url=tar.ownerDocument.URL;
				var keyName=tar.tagName.toLowerCase();
				if(keyName=="img"){
					if(tar.width>160&&tar.height>120){
						if(D.ImagePanel.target!=tar)
							D.ImagePanel.hide();
						if(url.match(/maps\.google\./i)||url.match(/\.google\.(.*)\/maps/i)){
							return ;
						}
						
						D.ImagePanel.reset(topContext.getSubContextByDoc(tar.ownerDocument));
						D.ImagePanel.scheduleShow(tar);
					}
				}else if(url.match(/\.flickr\.com\//i)){
					var parent=z.$(tar).parents('#photo')[0];
					if(parent){
						function a(p){
							var kids=z.$(p).children();
							if(kids&&kids.size()>0){
								for(var i=0;i<kids.size();i++){
									var kid=kids[i];
									if(kid
											&&kid.tagName.toLowerCase()=='img'
											&&kid.width>160
											&&kid.height>120){
										return kid;
									}
									var img=a(kid);
									if(img
											&&img.tagName.toLowerCase()=='img'
											&&img.width>160
											&&img.height>120){
										return img;
									}
								}
							}
							return null;
						}
						tar=a(parent);
						if(tar){
							if(D.ImagePanel.target!=tar)
								D.ImagePanel.hide();
							D.ImagePanel.reset(topContext.getSubContextByDoc(tar.ownerDocument));
							D.ImagePanel.scheduleShow(tar);
							z.$(parent).mouseout(function(){
								D.ImagePanel.scheduleHide();
							});
						}
					}
				}
			}
			
			if (!z.highlightable) return;
			
			
			if (topContext.draggingFloatNote) return;
			
			var hInfo = D.Highlighter.isHighlightElement(event.target);
			
			if (hInfo) {
				var id = hInfo.ids[0]; //take the first, forsake the reset
				var ann = Annotation.find(topContext, id);
				ann.activate(true,id);
				
				if (ann.comments.length == 0) {
					var IconAnnChanged = D.AnnotationHeader.ann != ann;
					if(hInfo.type==D.ANNOTATION_TYPE_TEXT||
						hInfo.type==D.ANNOTATION_TYPE_IMAGE){
						if (IconAnnChanged || !D.AnnotationHeader.shown){
							D.AnnotationHeader.reset(ann);
							D.AnnotationHeader.scheduleShow(event, ann);
						}else{
							D.AnnotationHeader.cancelHide();
							D.AnnotationHeader.cancelToggleEdit();
						}
					}
				} else if(hInfo.type==D.ANNOTATION_TYPE_FLOATNOTE
						||hInfo.type==D.ANNOTATION_TYPE_ICON){
					
					D.AnnotationHeader.removeEditMode();
					D.AnnotationHeader.ann=null;
					//with comments, show DlgIC
					var annChanged = D.DlgIC.ann != ann;
					
					if (!D.DlgIC.pinned && !D.DlgIC.editing) {
						if (annChanged || !D.DlgIC.shown)
							D.DlgIC.scheduleShow(event, ann);
						else
							D.DlgIC.cancelHide();
					}
				}
				
				
				if(ann.comments.length > 0&&(hInfo.type==D.ANNOTATION_TYPE_TEXT||
						hInfo.type==D.ANNOTATION_TYPE_IMAGE)){
					D.AnnotationHeader.reset(ann);
					D.AnnotationHeader.scheduleToggleEdit(true);
				}
			}
		},
		
		documentMouseOut: function(event) {
			//mouseout highlight
			
			var z = this;
			
//			if (D.runtime.inClipMode) { 
//				z.documentMouseOut_clip(event);
//				return;
//			}
			
			z.documentMouseOut_highlight(event);
		},
		
//		documentMouseOut_clip: function(event) {
//			var z = this;
//			if (!z.highlightable) return;
//			
//			event.target.style.outline = '';
//			
//			D.runtime.clip_currentNode = null;
//			
//			event.stopPropagation();
//			event.preventDefault();
//		},
		
		documentMouseOut_highlight: function(event) {
			var z = this;
			
			if (this.highlightable) {
				var topContext = this.getTopLevelContext();
				if (topContext.draggingFloatNote) return;
				
				var keyName=event.target.tagName.toLowerCase();
				if(keyName=="img")
				{
					D.ImagePanel.scheduleHide();
				}				
				
				var hInfo = D.Highlighter.isHighlightElement(event.target);
				if (hInfo) {
					var id = hInfo.ids[0];
					var ann = Annotation.find(topContext, id);
					if(ann)
						ann.activate(false,id);

					//cancel scheduled showing
					if (!DlgIC.pinned && !DlgIC.editing && DlgIC.aboutToShow()) {
						DlgIC.scheduleHide();
					}
					//this.textHighlighter.hideIcon(ann);

					if(D.AnnotationHeader.aboutToShow()&&ann.comments.length <= 0){
						D.AnnotationHeader.scheduleHide();
					}
					D.AnnotationHeader.scheduleToggleEdit(false);						
					//if (D.AnnotationContextMenu.aboutToShow())
					//	D.AnnotationContextMenu.scheduleHide();
				}
			}
		}
	});
	
	Utils.extend(PageContext, {
		/**
		 * Diigofy the document
		 */
		diigofy: function (doc) {
			var url = doc.URL;
			if (doc.__diigo || (!D.isDocumentDiigoable(doc) && !D.isLocalPDFReader(url))) return;
			
			Utils.dump('diigofying...', doc.defaultView == doc.defaultView.top ? 'top' : 'frame', D.isDocumentHighlightable(doc) ? '' : 'not highlightable', url);
			
			var context = doc.__diigo = new PageContext(doc);
			
			//for debugging
			//TODO remove these expandos
			var unsafeWin = doc.defaultView.wrappedJSObject || doc.defaultView;
			
			unsafeWin.diigorocks = function() {
				unsafeWin.c = unsafeWin.context = context;
				unsafeWin.d = unsafeWin.D = unsafeWin.diigo = D;
				unsafeWin.utils = unsafeWin.Utils = D.utils;
			}
			
			if(context.isPdfReader){
				unsafeWin.diigoUserName = D.env.user;
			}

			if(!D.isSlidesDiigoable(doc))
				unsafeWin.diigoToolbarVersion = D.version;			
			
			//TODO add unload listener so that we can clear things
			//TODO action according to url(permlink, ageslide, sandbox...)
			//only load bookmark for top level document
			if (context.isTopLevel()) {
				//if the page being diigofied is the current page
				//set _focusedContext
				if (context.win == window.content) {
					D.runtime._focusedContext = context;	
				}
				
				D.WebAPI.loadBookmark({url: context.bookmark.url, what: 'pageComments annotations ' + D.Sidebar.needAnythingFromLoadBookmark()}, context, function() {
					//why do I put the code here instead of inside WebAPI.callback_bm_loadBookmark_success() ?
					//the first loadBookmark of a page
					D.updateBookmarkableState();
		
					Toolbar.refresh('comment readLater');
				});
			
				//D.handle('newInlineComment',context,context.bookmark);
				//
				Sidebar.changeLocation();
				DlgIC.changeLocation();
			}
			//inserting style elements too early results in weird broken styles
			//the large font size bug
			//TODO restarting firefox: page loaded from cache does not trigger DOMContentLoaded?
			Utils.dom.once(document, 'DOMContentLoaded', function() {
				if (context.highlightable) {
					try{
						PageContext.addCSSFile(doc);
					}catch(e){
						//alert(e);
					}
					
					if(D.isDocumentAnnotated(doc.URL)){
						Utils.dom.HTML_addDiv(doc, {id:'diigo-chrome-installed'});
					}
				}
				
			});
			
			return context;
		},
		addCSSFile: function(doc){
			if(!doc.getElementById('diigo-activeHighlight')){
				Utils.dom.HTML_addCSS(doc, window.PAGE_CSS);
						//see Comments on Digg.com not shown in Toolbar Dev Notes
				Utils.dom.HTML_addCSS(doc, 'dummyRuleForDigg{}', 'diigo-activeHighlight');
			}
		},
		
		undiigofy: function (doc, unpaint) {
			var context = doc.__diigo;
			if (!context) return;
			
			[
				D.AnnotationHeader,
				D.Csm, 
				D.AnnotationContextMenu, 
				D.ImagePanel, 
				D.PageMessage
			].forEach(function(v) {
				v.clear();
			});

			
			Utils.dump('undiigofying...', doc.URL, doc.defaultView == doc.defaultView.top ? 'top' : 'frame');
			
			if (unpaint && context.isTopLevel()) {
				//unpaint all highlights
				context.annotations.forEach(function(ann) {
					ann.unpaint();
				});
				
				//undiigofy all sub contexts
				context.getSubContexts().forEach(function(v) {
					PageContext.undiigofy(v.doc);
				});
			}
			
			doc.__diigo = null;
			//TODO remove expandos on window and style sheets
		}
	});

})();