/**
 * The Quick Access Filter feature is only supported in Firefox 3
 */

(function() {

var D = diigo, U = D.utils, $ = D.$;

D.QuickAccessFilter = D.QAF = {
	TYPE_UNREAD: 0,
  TYPE_RECENT: 1,
  TYPE_TAG:    2,
  TYPE_LIST:   3,
	
	bmCache: {},
	
	jContainer: null,
	currentVersion: 1,
	//dirty: {}, //hash container dirty flags (whether menu needs to be refreshed dirty.id !=== false). filter.id as keys
	filters: [],
	loadNum: 0,
	currentBmitem: null,
	
	enabled: false, //flag. see checkEnabled
	
	//checks whether this feature is enabled
	//sets the flag and take action
	//need to call this when toolbar.showItems.diigotb-tb-quickAccessFilter-item changes
	checkEnabled: function() {
		var oldEnabled = this.enabled;
		var item, enabled = D.ff3up && D.hasSignedIn() && (item = document.getElementById('navigator-toolbox')) && item.getAttribute('diigohidden') != 'true';
		this.enabled = enabled;
		if (oldEnabled != enabled) {
			//if changed
			enabled ? this.doEnable() : this.doDisable();
		}
	},
	
	doEnable: function() {
		try{
		var z = this;
		if (D.hasSignedIn()&&D.prefs.val('toolbar.showItems.diigotb-tb-quickAccessFilter-item')) z.load();
		z.jContainer = $('#diigotb-tb-filterContainer');
		z.refreshUI();
		D.QAF.DNDFilterOrder.setup();
		
		D.addEventListener('bm_save', z);
		D.addEventListener('bm_del', z);
		}catch(e){throw(e);}
	},
	
	doDisable: function(setPref) {
		var z = this;
//		$('#diigotb-tb-quickAccessFilter-item').attr('diigohidden', 'true');
		if (setPref) D.prefs.val('toolbar.showItems.diigotb-tb-quickAccessFilter-item', false);
		D.removeEventListener('bm_save', z);
		D.removeEventListener('bm_del', z);
	},
	
	init: function() {
		if (!D.ff3up) this.doDisable(true);
	},
	
	
	load: function() {
		var z = this;
		D.WebAPI.loadFilters(function(data) {
			if(data == null) return;
			z.filters = data.filters;
			z.filters.forEach(function (f,i) {
				setTimeout(function(){z.loadBookmarksForFilter(f);},50+500*i);
			});
			z.refreshUI();
		});
	},
	
//	save: function() {
//		var z = this, a = z.filters;
//		for (var i = a.length, f; f = a[--i], i >= 0;) {
//				if (!z.isValidFilter(f)) {
//					U.dump('[QAF.save] invalid filter detected. will not save', f);
//					return false;
//				}
//		}
//		
//		D.prefs.set('bmfilters', U.json.toJSON(this.filters), true);
//	},
	
//	isValidFilter: function(f) {
//		return !!(f.id && f.pos && f.type);
//	},
	
	/**
	 * generates a random id
	 * //Math.max() returns -Infinity so we add another layer of Math.max(0, ...) to ensure we get a valid number
	 */
	_nextFilterId: function() {
		return -Math.floor(Math.random() * 1000);
//		return Math.max(0, Math.max.apply(null, this.filters.map(function(f) {return f.id;}))) + 1;
	},
	_nextFilterPos: function() {
		return Math.max(0, Math.max.apply(null, this.filters.map(function(f) {return f.pos;}))) + 10;
	},
	_TempFilter:null,
	compareFilter: function(f,f2) {
		var filter=(typeof f2 == "object")?f2:D.QAF._TempFilter;
		if(f.type == filter.type){
			if(f.type == D.QAF.TYPE_TAG && D.unparseTags(filter.criteria.tags) == D.unparseTags(f.criteria.tags))
				return true;
			else if(f.type == D.QAF.TYPE_RECENT)
				return true;
			else if(f.type == D.QAF.TYPE_LIST && f.criteria.listId == filter.criteria.listId)
				return true;
			else if(f.type == D.QAF.TYPE_UNREAD)
				return true;
		}
		return false;
	},
	isAlreadyExist: function(filter) {
		var z = this;
		z._TempFilter=filter;
		res=U.array.find(this.filters, this.compareFilter);
		z._TempFilter=null;
		if (res) {
			return false;
		}
		return true;
	},
	//return false means already exists
	addTagFilter: function(tags,unread,position,count) {
		if (U.array.find(this.filters, function(f) {
			return f.type == D.QAF.TYPE_TAG && f.criteria.unread == unread && D.unparseTags(tags) == D.unparseTags(f.criteria.tags);
		})) {
			return false;
		} else {
			var f = {
				id: this._nextFilterId(),
				pos: position?position:this._nextFilterPos(),
				type: D.QAF.TYPE_TAG,
				v: this.currentVersion,
				bmcount: count,
				name: D.unparseTags(tags),
				criteria: {tags: tags,unread:unread}
			};
			this.filters.push(f);
			
			this.refreshUI(f);
			this.loadBookmarksForFilter(f);
			this._addFilter(f,position);
			return true;
		}
	},
	
	addRecentFilter: function(position,count) {
		if (U.array.find(this.filters, function(f) {return f.type == D.QAF.TYPE_RECENT;})) {
			return false;
		} else {
			var f = {
				id: this._nextFilterId(),
				pos: position?position:this._nextFilterPos(),
				type: D.QAF.TYPE_RECENT,
				v: this.currentVersion,
				bmcount: count,
				name: 'Recent',
				criteria: {}
			};
			this.filters.push(f);
			
			this.refreshUI(f);
			this.loadBookmarksForFilter(f);
			this._addFilter(f,position);
			return true;
		}
	},
	
	addListFilter: function(listId, listName,position,count,timeout) {
		if (U.array.find(this.filters, function(f) {return f.type == D.QAF.TYPE_LIST && f.criteria.listId == listId;})) {
			return false;
		} else {
			var f = {
				id: this._nextFilterId(),
				pos: position?position:this._nextFilterPos(),
				type: D.QAF.TYPE_LIST,
				v: this.currentVersion,
				bmcount: count,
				name: listName,
				criteria: {listId: listId}
			};
			this.filters.push(f);
			this.refreshUI(f);
			if(timeout){
				z=this;
				setTimeout(function() {
					z.loadBookmarksForFilter(f);
				}, timeout);
			}else{
				this.loadBookmarksForFilter(f);
			}
			this._addFilter(f,position);
			return true;
		}
	},
	
	addUnreadFilter: function(position,count) {
		if (U.array.find(this.filters, function(f) {return f.type == D.QAF.TYPE_UNREAD;})) {
			return false;
		} else {
			var f = {
				id: this._nextFilterId(),
				pos: position?position:this._nextFilterPos(),
				type: D.QAF.TYPE_UNREAD,
				bmcount: count,
				v: this.currentVersion,
				name: 'Unread',
				criteria: {}
			};
			this.filters.push(f);
			this.refreshUI(f);
			this.loadBookmarksForFilter(f);
			this._addFilter(f,position);
			return true;
		}
	},
	
	_addFilter: function(f,position) {
		var z = this;
		function task() {
			//filter.id will be updated if it's an default id
			D.WebAPI.addFilter({
				name: f.name,
				pos: f.pos,
				type: f.type,
				criteria: f.criteria
			}, function(data) {
				f.id = data.id;
				z.refreshUI(f);
				if(position){
					z.afterReordering();
				}
			});
			z.refreshUI(f);
		}
		
		z.saveDefaultsAndRun(task);
	},
	
	_importFilter: function(lists) {
		D.importing=true;
		function a(l){
			D.WebAPI.createBmList(l,function(result){
				if(result&&result.lists){
					U.each(result.lists,function(lis){
						D.env.outliners.push(lis);
						D.QAF.addListFilter(lis.id, lis.title,null,20,5000);
					});					
				}

				if(lists.length>0){
					var data={
						lists:[lists.shift()]
					};
					a(data);
				}else{
					D.importing=false;
				}
			});			
		}
		var data={
			lists:[lists.shift()]
		};		
		a(data);
		D.notifyImport("Importing your bookmarks");
	},
	
	edit: function(id) {
		
	},
	
	remove: function(id) {
		var z = this;
		var filter = z.find(id);
		if (filter.type == D.QAF.TYPE_UNREAD){
			//var j = $('#diigotb-tb-readitLater');
			//j.attr('hidden', 'true');
		}
		
		function task() {
			//filter.id will be updated if it's an default id
			D.WebAPI.delFilter(filter.id);
			z.filters.splice(z.filters.indexOf(filter), 1);
			z.refreshUI();
		}
		
		z.saveDefaultsAndRun(task);
		
	},
	
	find: function(id) {
		return id.id ? id : U.array.find(this.filters, function(f) {return f.id == id;});
	},
	
	_getCacheKey: function(filter) {
		filter = this.find(filter);
		if (filter.type == D.QAF.TYPE_LIST) {
			return 'list_' + filter.criteria.listId;
		} else if (filter.type == D.QAF.TYPE_TAG) {
			return 'tag_' + filter.criteria.tags.join(',')+filter.criteria.unread;
		} else {
			throw "[QAF] _getCacheKey: invalid filter type " + filter.type;
		}
	},
	
	getBookmarksForFilter: function(filter) {
		var z = this, t, k;
		filter = z.find(filter);
		if (filter.type == D.QAF.TYPE_UNREAD) {
			return D.env.unreadBookmarks;
		} else if (filter.type == D.QAF.TYPE_RECENT) {
			return D.env.recentlySavedBookmarks;
		} else if (filter.type == D.QAF.TYPE_LIST || filter.type == D.QAF.TYPE_TAG) {
			k = z._getCacheKey(filter);
			t = z.bmCache[k];
			if (!t) t = z.bmCache[k] = [];
			return t;
		}
	},
	
	//load from webapi and save to cache
	loadBookmarksForFilter: function(filter) {
	    var z = this;
	    filter = z.find(filter);
		var type = filter.type;
	    if (type == D.QAF.TYPE_UNREAD) {
	        //
	    } else if (type == D.QAF.TYPE_RECENT) {
	    } else if (type == D.QAF.TYPE_LIST) {
	        D.WebAPI.loadBookmarksInList(filter.criteria.listId, filter.bmcount, function(r){
	        	if(r){
	        		z.bmCache[z._getCacheKey(filter)] = r.bookmarks;
	        	}
	        });
	    } else if (type == D.QAF.TYPE_TAG) {
	        D.WebAPI.searchBookmarkbyTag({
	        							tags: filter.criteria.tags,
	        							bmcount: filter.bmcount,
	        							unread: filter.criteria.unread}, function(r){ 
	        		if(r){
	        			z.bmCache[z._getCacheKey(filter)] = r.bookmarks;
	        		}
	        	});
	    }
	},
	
	//tells whether a bookmark matches a filter
	matchFilter: function(bm, filter) {
		var z = this;
		filter = z.find(filter);
		if (filter.type == D.QAF.TYPE_UNREAD) {
			return bm.unread;
		} else if (filter.type == D.QAF.TYPE_RECENT) {
			return true;
		}  else if (filter.type == D.QAF.TYPE_LIST) {
			return bm.inList && bm.inList(filter.criteria.listId);
		} else if (filter.type == D.QAF.TYPE_TAG) {
			var a = filter.criteria.tags;
			for (var l = a.length, i = 0, t; t = a[i], i < l; i++) {
				if (bm.tags.indexOf(t) == -1) return false;
			}
			return true;
		}
	},
	//////////////////////////////////////////////////
	onbm_save: function(bm) {
		if (!this.enabled) return;
		
		U.dump('[QAF] bm_saved', bm);
		var z = this;
		z.filters.forEach(function(f) {
			if (f.type == D.QAF.TYPE_UNREAD || f.type == D.QAF.TYPE_RECENT) return; //WebAPI.js takes care of unread and recent bookmarks
			
			U.dump('[QAF] bm_saved filter', f);
			U.dump('[QAF] bm_saved match', z.matchFilter(bm, f));
			if (z.matchFilter(bm, f)) {
				var a = z.getBookmarksForFilter(f), i = U.array.findIndex(a, function(v) {return v.url == bm.url;});
				
				//add or update
				var data = {
					datetime: bm.datatime,
					mode: bm.mode,
					title: (bm.getTitle ? bm.getTitle() : bm.title) || bm.url,
					url: bm.url,
					user: bm.user,
					userOnline: bm.userOnline
				};
					
				if (i == -1) {
					a.unshift(data);
				} else {
					a[i] = data;
				}
			}
		});
	},
	
	onbm_del: function(bm) {
		if (!this.enabled) return;
		var z = this;
		z.filters.forEach(function(f) {
			if (f.type == D.QAF.TYPE_UNREAD || f.type == D.QAF.TYPE_RECENT) return; //WebAPI.js takes care of unread and recent bookmarks
			if (z.matchFilter(bm, f)) {
				var a = z.getBookmarksForFilter(f), i = U.array.findIndex(a, function(v) {return v.url == bm.url;});
				
				if (i > -1) a.splice(i, 1);
			}
		});
	},
	
	//reordered by dnd, sort filters by elements' position and save
	afterReordering: function() {
		try{
		var z = this;
		z.refreshPosition();
		
		function task() {
			
			D.WebAPI.updateFilterPos(z.filters.map(function(f) {
				return [f.id, f.pos];
			}));
		}
		
		z.saveDefaultsAndRun(task);
		}catch(e)
		{
		}
	},
	//reordered by dnd, sort filters by elements' position and save
	refreshPosition: function() {
		var z = this;
		var ids = z.jContainer.find('toolbarbutton[filterid]').get().map(function(b) {return Number(b.getAttribute('filterid'));});
		z.filters.sort(function(a, b) {return ids.indexOf(a.id) - ids.indexOf(b.id);});
		//reassign pos
		for (var l = z.filters.length, i = 0; i < l; i++) {
			z.filters[i].pos = i*5 + 10;
			
		}
		U.dump('[QAF afterReordering]', ids, z.filters.map(function(f) {return f.name;}));
	},	
	//////////////////////////////////////////////////
	/**
	 * Save default filters (if necessary) and call f
	 * @param {Function} f
	 */
	saveDefaultsAndRun: function(f) {
		var z = this;
		var anyDefault = z.filters.filter(function(f) {return f['default'];}).length > 0;
		if (anyDefault) {
			D.WebAPI.saveDefaultFilters(function(data) {
				U.dump('[QAF] default filters saved', data.ids);
				var idsMapping = data.ids;
				idsMapping.forEach(function(m) {
					var oldId = m[0], newId = m[1];
					var f = z.find(oldId);
					if (f) {f.id = newId; f['default'] = false;}
				});
				z.refreshUI();
				f();
			});
		} else {
			f();
		}
	},
	
	///////////////////////////////////////////////////////////////////////////////
	refreshUI: function() {
		var z = this;
		z.jContainer.find('toolbarbutton[filterid]').remove();
		var unreadshown=false;
		z.filters
			.sort(function(a, b) {return a.pos - b.pos;})
			.forEach(function(f) {
				z.buildMenu(f);
				if (f.type == D.QAF.TYPE_UNREAD){
					unreadshown=true;
				}
			});
		var anyDefault = z.filters.filter(function(f) {return f['default'];}).length > 0;
		if (!anyDefault) {
			$('#diigotb-tb-addfilter').attr('class','diigotb-toolbarButton noText diigotb-icon-addFilter');
		}else{
			$('#diigotb-tb-addfilter').attr('class','diigotb-toolbarButton diigotb-icon-addFilter');
		}
		setTimeout(function(){
			if(document.getElementById('diigotb-tb-filterContainer')!=null
					&&z.jContainer.find('toolbarbutton[filterid]').length<=0
					&&z.loadNum<3){
				z.loadNum++;
				D.QAF.doEnable();
			}
		},15000);
		/*
		var j = $('#diigotb-tb-readitLater');
		if(unreadshown){
			j.attr('hidden', 'false');
		}else{
			j.attr('hidden', 'true');
		}*/
		//chevron
		//TODO don't trigger chevron when unnecessary
		return D.Chevron.doResize();
	},
	
	//build the menebutton element and insert it to filter container
	buildMenu: function(f) {
		var iconClass = '';
		var unread=f.criteria.unread;
		if (f.type == D.QAF.TYPE_TAG){
			if(unread){
				iconClass = 'diigotb-icon-tag-unread';
			}else{
				iconClass = 'diigotb-icon-tag';
			}
		} 

		return $(U.dom.buildOne(
			'toolbarbutton', {
				type: 'menu', 
				'class': 'diigo-bookmark-item ' + iconClass, 
				label: f.name, 
				container: 'true', 
				filterid: f.id, 
				context: 'diigotb-filterContextMenu'}, 
				['menupopup', {onpopupshowing: 'if (event.target == this) diigo.QuickAccessFilter.onPopupShowing(event);'}, [
					'menuseparator', {},
					'menuitem', {
						label: 'Still empty, drag a link here to save',
						'class': 'empty-item',
						disabled: 'true'
					},
					'menuitem', {
						label: 'Switch to sidebar view',
						'class': 'open-switch',
						accesskey: 't',
						oncommand: "diigo.handle('switchTagsidebar', '"+f.id+"');"
					},
					'menuitem', {
						label: 'See More On the Web',
						'class': 'open-see-more',
						accesskey: 'S',
						onmouseup :"if(event.button==0||event.button==1) diigo.handle('viewFilterbm', event,'"+f.id+"');"
					},
					'menuitem', {
						label: 'Open All in Tabs',
						'class': 'open-all',
						accesskey: 'O',
						oncommand: "diigo.handle('" + (f.type == D.QAF.TYPE_UNREAD ? 'openAllUnreadInTabs' : 'openAllInTabs') + "', event);"
					}
					]
				]
			)).appendTo(this.jContainer);
	},
	
	//fill menu with bookmarks
	/**
	 * 
	 * @param {Object} filterId
	 * @param {Object} the menupopup to fill
	 */
	fillMenu: function(filterId, menu) {
		var id = filterId.id || filterId, filter = this.find(id);
		var Cc = Components.classes, Ci = Components.interfaces;
		var ioservice = Components.classes["@mozilla.org/network/io-service;1"]
						                          .getService(Components.interfaces.nsIIOService);
		
		var faviconsvc = PlacesUtils.favicons;//Cc["@mozilla.org/browser/favicon-service;1"].getService(Ci.nsIFaviconService);														 
		                         
		var z = this;
		var jMenuPopups = $(menu);
		
		jMenuPopups.each(function() {
			var jMenuPopup = $(this);
			jMenuPopup.find('>.bookmark-item').remove();
		
			//recently opened menu for unread filter
			if (filter.type == D.QAF.TYPE_UNREAD) {
				var jRecent = $('menu.recentlyOpened', jMenuPopup);
				//remove the menu if there are no srecently opened bookmarks
				if (D.env.recentlyReadBookmarks.length == 0) 
					jRecent.remove();
				else {
					if (jRecent.length == 0) {
						jRecent = $(U.dom.buildOne('menu', {
							label: 'Recently Opened',
							'class': 'recentlyOpened menu-iconic bookmark-item',
							container: 'true'
						}, ['menupopup', {}])).prependTo(jMenuPopup[0]); //add to top
					}
					
					jRecent.find('menupopup').empty().append(D.env.recentlyReadBookmarks.map(function(bm){
						return U.dom.buildOne('menuitem', {
							'class': 'menuitem-iconic bookmark-item',
							label: bm.title,
							url: bm.url,
							onmouseup: "if(event.button==0||event.button==1){diigo.handle('openMenuItemUrl', event);}"
						});
					}));
				}
			}
			
			var above18=diigoUtil.firefox.ff18OrAbove();
			var isEmpty=true;
			$(z.getBookmarksForFilter(id).map(function(bm) {
				
				var item = U.dom.buildOne('menuitem', {
					'class': 'menuitem-iconic bookmark-item openAllInTabs', 
					label: bm.title, 
					url: bm.url, 
					belongfilter: id,
					context: 'diigotb-bookmark-menu',
					onmouseup: "if(event.button==0||event.button==1){diigo.handle('openMenuItemUrl', event, " + (filter.type == D.QAF.TYPE_UNREAD ? "'unread'" : 'null') + ");}else{diigo.QAF.currentBmitem=this;}"
				});
				isEmpty=false;
				var pageUri=ioservice.newURI(bm.url , null, null);
				faviconsvc.getFaviconURLForPage(pageUri, function(uri){
					if(uri){
						if(above18){
							faviconsvc.setAndFetchFaviconForPage(pageUri, uri, true, faviconsvc.FAVICON_LOAD_NON_PRIVATE);
						}else{
							faviconsvc.setAndFetchFaviconForPage(pageUri, uri, true);
						}
						
						item.style.listStyleImage = 'url(' + uri.spec + ')';
					}
				});
				return item;
			})).insertBefore(jMenuPopup.find('menuseparator')[0]); //insert before the menu separator before Open in All Tabs
			/*if(filter.type == D.QAF.TYPE_TAG){
				
			}else{
				jMenuPopup.find('>.open-switch')[0].hidden=true;
			}*/
			jMenuPopup.find('>.open-switch')[0].hidden=false;
			
			if(isEmpty){
				jMenuPopup.find('menuseparator')[0].hidden=true;
				jMenuPopup.find('>.open-all')[0].hidden=true;
				jMenuPopup.find('>.empty-item')[0].hidden=false;
				jMenuPopup.find('>.open-see-more')[0].hidden=true;
				jMenuPopup.find('>.open-switch')[0].hidden=true;
			}else{
				jMenuPopup.find('menuseparator')[0].hidden=false;
				jMenuPopup.find('>.open-all')[0].hidden=false;
				jMenuPopup.find('>.empty-item')[0].hidden=true;
				jMenuPopup.find('>.open-see-more')[0].hidden=false;
			}
		});
	},
	
	onPopupShowing: function(event) {
		U.dump('[QAF onPopupShowing]');
		var btn = event.target.parentNode;
		var filterId = btn.getAttribute('filterid');
		U.dump('[QAF onPopupShowing] filterId:', filterId);
		
		//the dirty attr is set, we need to refresh
//		if (this.dirty.filterId !== false) {
			this.fillMenu(filterId, event.target);
//			this.dirty.filterId = false;
//		}
	},
	
	/**
	 * Adds a bookmark to a filter
	 * @param {Object} data {url: '', title: ''}
	 * @param {Object} filter
	 */
	addBookmark: function(data, filter) {
		var c = D.getCurrentContext(), bm=c.bookmark, moreOptions = {};
		
		//if not current url
		if (!(c && D.urlEqual(c.bookmark.url, data.url))) {
			c = null; //make sure we don't pass context to addBookmark window
			bm = new D.Bookmark({
				title: data.title,
				url: data.url,
				context: true
			});
			bm.context = null;
		}
		
		var callback=null,z=this;
		//more options
		if (filter.type == D.QAF.TYPE_UNREAD) {
			bm.unread = true;
		} else if (filter.type == D.QAF.TYPE_TAG) {
			bm.tags = [].slice.call(filter.criteria.tags);
			callback=function(){
				D.fireEvent('bm_save', [bm]);
			};
		} else if (filter.type == D.QAF.TYPE_LIST) {
			bm.toAddToLists = [filter.criteria.listId];
			callback=function(){
				D.fireEvent('bm_save', [bm]);
			};
		}
		D.WebAPI.saveBookmark(bm, {}, c, callback);
		/*
		window.openDialog(
			'chrome://diigotb/content/addBookmark.xul', 
			'addBookmark', 
			'width=530,height=380,chrome,resizable=no,minimizable',
			c,
			bm,
			moreOptions
		).focus();*/
	}
};
//
//var Filter = D.QuickAccessFilter.Filter = {
//	
//};
	
	//context menu for filter menubutton
	D.QAF.FilterContextMenu = {
		contextEle: null,
		filter: null,
		onPopupShowing: function(event) {
			this.contextEle = document.popupNode;
			this.filter = D.QAF.find(this.contextEle.getAttribute('filterid'));
		},
		
		onCmd_remove: function() {
			var f = this.filter;
			//if (confirm('Are you sure you want to remove the filter "' + f.name + '"?')) {
				D.QAF.remove(f);
			//}
		},
		onCmd_edit: function() {
			var f = this.filter;
			openDialog('chrome://diigotb/content/quickAccessFilter/addFilter.xul', 'dlgaddFilter', '',f).focus();
		},
		onCmd_add: function() {
			var f = this.filter;
			openDialog('chrome://diigotb/content/quickAccessFilter/addFilter.xul', 'dlgaddFilter', '',null,f.pos+0.5).focus();
		},
		onCmd_bookmarkHere: function() {}
	};
	
	/**
	 * If a menubutton is opened, mouse hovering another menubutton should open it automatically
	 * listen for mouseover event on the container hbox
	 * inspired by delicious menu autoopen code in toolbar.js which is ganked from /browser/components/bookmarks/content/bookmarksMenu.js
	 */
	D.QAF.AutoOpenMenu = {
		_openedMenuButton: null,
		autoOpen: function(event) {
			var z = D.QAF.AutoOpenMenu;
			var t = event.target;
	    if (z._openedMenuButton != t && t.localName == "toolbarbutton" && t.type == "menu") {
	      z._openedMenuButton.open = false;
	      t.open = true;
	    }
		},
		
		menuOpened: function(event) {
			var t = event.target;
			if (t.parentNode.localName == 'toolbarbutton') {
				this._openedMenuButton = t.parentNode;
				event.currentTarget.addEventListener('mouseover', this.autoOpen, true);
			}
		},
		
		menuClosed: function(event) {
			var t = event.target;
			if (t.parentNode.localName == 'toolbarbutton') {
				this._openedMenuButton = null;
				event.currentTarget.removeEventListener('mouseover', this.autoOpen, true);
			}
		}
	};
	
	//order filters using dnd
	D.QAF.DNDFilterOrder = {
		setupDone: false, //flag to avoid setting up multiple times
		ind: null, //drop indicator (dropmarker)
		buttonDragged: null, //menubutton being dragged
		whereToDrop: {
			ele: null, //reference drop target
			before: true //insert before or after the drop target
		},
		
		setup: function() {
			if (this.setupDone) return;
			this.setupDone = true;
			
			var z = this, container = D.id('diigotb-tb-filterContainer');
			z.ind = $(container).find('hbox.diigoToolbarDropMarker')[0];
			
			if(container){
				container.addEventListener('draggesture', function(event) {
					var t = event.target;
					if (t.localName == 'toolbarbutton' && t.hasAttribute('filterid')) {
						t.open = false;
						z.buttonDragged = t;
						z.whereToDrop.ele = null;
						nsDragAndDrop.startDrag(event, z);
					}
				}, false);
				
				container.addEventListener('dragover', function(event) {
					nsDragAndDrop.dragOver(event, z);
				}, false);
				
				container.addEventListener('dragdrop', function(event) {
					nsDragAndDrop.drop(event, z);
				}, false);
				
				container.addEventListener('dragexit', function(event) {
					nsDragAndDrop.dragExit(event, z);
				}, false);
			}
		},
		
		//////////////observer/////////////////////
		onDragStart: function (event, transferData, action) {
			U.dump('[onDragStart]');
			action.action = Ci.nsIDragService.DRAGDROP_ACTION_MOVE;
	    var fid = event.target.getAttribute("filterid");
	    transferData.data = new TransferData();
	    transferData.data.addDataForFlavour("text/x-diigo-filter", fid); //data is actually unused
	  },
		
		onDragOver: function (event, flavour, session) {
			var z = this, t = event.target;
			var ind = z.ind;
			
			U.dump('[onDragOver]', flavour);
			
			//dragging over a filer button, either insert the dropmarker before it or after it, denpending on mouse x
			if (flavour.contentType == 'text/x-diigo-filter' && t.localName == 'toolbarbutton' && t != z.buttonDragged) {
			  	z.whereToDrop.ele = t;
			  	if (event.clientX - (t.boxObject.x + t.boxObject.width / 2) < 0) {
			  		t.parentNode.insertBefore(ind, t);
			  		z.whereToDrop.before = true;
			  	} else {
			  		if (t.nextSibling)
			  			t.parentNode.insertBefore(ind, t.nextSibling);
			  		else
			  			t.parentNode.appendChild(ind);
							
			  		z.whereToDrop.before = false;
			  	}
			  	
			  	ind.hidden = false;
			}else if (flavour.contentType == 'text/x-moz-url'){
	  	
			}else if (flavour.contentType == 'text/x-moz-place'){
			}
				
		},
		
		onDrop: function (event, dropdata, session) {
			U.dump('[onDrop]', dropdata, session);
			var z = this, ct = dropdata.flavour.contentType;
			if (ct == 'text/x-diigo-filter') {
				z.ind.hidden = true;
		  	
		  		var a = z.buttonDragged, b = z.whereToDrop.ele;
		  		if (b && b != a) {
		  			//we need to insert a before/after b
					if (z.whereToDrop.before) {
						a.parentNode.insertBefore(a, b);
					}
					else {
						if (b.nextSibling) 
							a.parentNode.insertBefore(a, b.nextSibling);
						else 
							a.parentNode.appendChild(a);
					}
					//now that reordering is done, save the new order
					D.QAF.afterReordering();
				}
			} else if (ct == 'text/x-moz-url' || ct == 'text/x-moz-place') {
				var filter = D.QAF.find(event.target.getAttribute('filterid')), data;
				if (ct == 'text/x-moz-url') {
				//'url\ntitle'
					var t = dropdata.data.split('\n');
					data = {url: t[0], title: t[1]};
				} else if (ct == 'text/x-moz-place') {
					//"{\"title\":\"欢迎光临白云黄鹤BBS站\",\"id\":160,\"parent\":3,\"dateAdded\":1210219677172375,\"type\":\"text/x-moz-place\",\"uri\":\"http://bbs.whnet.edu.cn/main.html\"}"
					var json = U.json.parseJSON(dropdata.data);
					data = {url: json.uri, title: json.title};
				}
				U.dump('[onDrop] link/bm', data, filter);
				D.QAF.addBookmark(data, filter);
			}
				
			z.whereToDrop.ele=null;
	  },
		
		onDragExit: function(event, session) {
			var z = this;
			
			//do nothing if the relatedTarget is z.ind to avoid flickers
			if (event.relatedTarget != z.ind) {
		  	z.ind.hidden = true;
		  	//z.whereToDrop.ele = null;
		  }
		},
		
		getSupportedFlavours : function () {
	    var flavours = new FlavourSet();
			var types = ['text/x-moz-place', 'text/x-moz-url', 'text/x-diigo-filter'];
	    for (var i = 0; i < types.length; ++i) {
	      flavours.appendFlavour(types[i]);
	    }
	    return flavours;
	  },
		
		canDrop: function(event, session) {
			var types = ['text/x-moz-place', 'text/x-moz-url', 'text/x-diigo-filter'];
			var foundType = false;
	    for (var i = 0; i < types.length && !foundType; ++i) {
	      if (session.isDataFlavorSupported(types[i]))
	        foundType = true;
	    }
	
      return foundType;
		}

		////////////////////////////////////////////
	};

})();
