(function() {
	var D = window.diigoUtil;
	
/////////////////////////////////////////////////////////////////////////////////

	D.FileIO = 
	{

		localfileCID  : '@mozilla.org/file/local;1',
		localfileIID  : Components.interfaces.nsIFile,

		finstreamCID  : '@mozilla.org/network/file-input-stream;1',
		finstreamIID  : Components.interfaces.nsIFileInputStream,

		foutstreamCID : '@mozilla.org/network/file-output-stream;1',
		foutstreamIID : Components.interfaces.nsIFileOutputStream,

		sinstreamCID  : '@mozilla.org/scriptableinputstream;1',
		sinstreamIID  : Components.interfaces.nsIScriptableInputStream,

		suniconvCID   : '@mozilla.org/intl/scriptableunicodeconverter',
		suniconvIID   : Components.interfaces.nsIScriptableUnicodeConverter,

		open   : function(path) {
			try {
				var file = Components.classes[this.localfileCID]
								.createInstance(this.localfileIID);
				file.initWithPath(path);
				return file;
			}
			catch(e) {
				return false;
			}
		},

		read   : function(file, charset) {
			try {
				var data     = new String();
				var fiStream = Components.classes[this.finstreamCID]
									.createInstance(this.finstreamIID);
				var siStream = Components.classes[this.sinstreamCID]
									.createInstance(this.sinstreamIID);
				fiStream.init(file, 1, 0, false);
				siStream.init(fiStream);
				data += siStream.read(-1);
				siStream.close();
				fiStream.close();
				if (charset) {
					data = this.toUnicode(charset, data);
				}
				return data;
			} 
			catch(e) {
				return false;
			}
		},

		write  : function(file, data, mode, charset) {
			try {
				var foStream = Components.classes[this.foutstreamCID]
									.createInstance(this.foutstreamIID);
				if (charset) {
					data = this.fromUnicode(charset, data);
				}
				var flags = 0x02 | 0x08 | 0x20; // wronly | create | truncate
				if (mode == 'a') {
					flags = 0x02 | 0x10; // wronly | append
				}
				foStream.init(file, flags, 0664, 0);
				foStream.write(data, data.length);
				foStream.close();
				return true;
			}
			catch(e) {
				return false;
			}
		},

		create : function(file) {
			try {
				file.create(0x00, 0664);
				return true;
			}
			catch(e) {
				return false;
			}
		},

		unlink : function(file) {
			try {
				file.remove(false);
				return true;
			}
			catch(e) {
				return false;
			}
		},

		path   : function(file) {
			try {
				return 'file:///' + file.path.replace(/\\/g, '\/')
							.replace(/^\s*\/?/, '').replace(/\ /g, '%20');
			}
			catch(e) {
				return false;
			}
		},
		toUnicode   : function(charset, data) {
			try{
				var uniConv = Components.classes[this.suniconvCID]
									.createInstance(this.suniconvIID);
				uniConv.charset = charset;
				data = uniConv.ConvertToUnicode(data);
			} 
			catch(e) {
			}
			return data;
		},

		fromUnicode : function(charset,data) {
			var data=null;
			try {
				var uniConv = Components.classes[this.suniconvCID]
									.createInstance(this.suniconvIID);
				uniConv.charset = charset;
				data = uniConv.ConvertFromUnicode(data);
				
			}
			catch(e) {
				//alert(e);
			}
			return data;
		}
	};

	D.PathIO = 
	{

		sep        : '\\',

		dirservCID : '@mozilla.org/file/directory_service;1',
	
		propsIID   : Components.interfaces.nsIProperties,
	
		fileIID    : Components.interfaces.nsIFile,

		get    : function(type) {
			try {
				var dir = Components.classes[this.dirservCID]
								.createInstance(this.propsIID)
								.get(type, this.fileIID);
				return dir;
			}
			catch(e) {
				return false;
			}
		},

		open   : function(path) {
			return D.FileIO.open(path);
		},
		
		GetPrefsDir : function()
		{
			return this.get('ProfD');
		},
		GetExtentionPath : function(id)
		{
		    var strExtPath = "";
	        try
	        {
	            var ext = Components.classes["@mozilla.org/extensions/manager;1"]
	                    .getService(Components.interfaces.nsIExtensionManager)
	                    .getInstallLocation(id)
	                    .getItemLocation(id); 
	            strExtPath = ext.path;
	        }
	        catch(e)
	        {
	            strExtPath = "";
	        }

	        return strExtPath;
	    },
		create : function(dir) {
			try {
				dir.create(0x01, 0755);
				 //make sure we can read and write to and from the directory
				if (!dir.isReadable() || !dir.isWritable())
				{
					dir.permissions = 0755;
				}
				return true;
			}
			catch(e) {
				return false;
			}
		}
	};	
	
	D.searchBox = {
		xmlFileName: 'diigo--google.xml',
		imgFileName: 'favicon.ico',
		copyFile: function(filePos, profileEnginePath, filename) {
	
		    var aLocal = Components.classes["@mozilla.org/file/local;1"]
				.createInstance(Components.interfaces.nsIFile);
		    if (!aLocal) return false;
		
		    aLocal.initWithPath(profileEnginePath);
		    aLocal.appendRelativePath(filename);
		
		    try {
		    	aLocal.remove(false);
		    }catch(e){ }
		
		    aLocal.initWithPath(profileEnginePath);
		
		    try {
		        if (aLocal.isWritable()){
		        	filePos.copyTo(aLocal, filename);
		        }else{
		            return false;
		        }
		    }catch(e){
		        //return false;
		    }
		
		    return true;
		},
	    CreateFolder: function(strPath) {
	        var oPath = D.PathIO.open(strPath);
	        if (!oPath.exists())
	        	D.PathIO.create(oPath);
	    },
	
	    getProfileSearchFolder: function() {
	        var plugDir = D.PathIO.GetPrefsDir();
	        var f = Components.classes["@mozilla.org/file/local;1"]
		         		                  .createInstance(Components.interfaces.nsIFile);
	        f.initWithPath(plugDir.path);
	        f.append('searchplugins');
	        return f.path;
	    },

		SetSearchEngine: function() {
	    	try{

	        var strDestPath = this.getProfileSearchFolder();
	        this.CreateFolder(strDestPath);

	        var extPath = D.PathIO.GetExtentionPath('{fc2b8f80-d9a5-4f51-8076-7c7ce3c67ee3}');

	        var engineSrc = D.FileIO.open(extPath);
	        engineSrc.append('Search');
	        var engineGif = D.FileIO.open(extPath);
	        engineGif.append('Search');

	        var bIsSuccess = false;

	        engineSrc.appendRelativePath(this.xmlFileName);
	        engineGif.appendRelativePath(this.imgFileName);
	        bIsSuccess = this.copyFile(engineSrc, strDestPath, this.xmlFileName);

	        if (!bIsSuccess) {
				window.external.AddSearchProvider('https://www.diigo.com/diigoresources/downloads/diigo_customize.xml');
	        }
			diigoUtil.prefs.setPref('extensions.diigotb.search',true);
	    	}catch(e){
	    		//alert('SetSearchEngineL: '+e);
	    	}
		}
	};

/////////////////////////////////////////////////////////////////////////////////
})();