(function() {
	var D = window.diigo;
	var Utils = D.utils;

	var $ = D.$;
	
	var Sidebar = D.Sidebar = {
		//currentUrl: '',
		currentContext: '',
		
		//window object of sidebar.html
		//maintained by the sidebar page
		pageWin: null,
		
		urlLocked: false,
		
//		/**
//		 * Returns the current url of gBrowser if it is diigoable, otherwise false
//		 */
//		getValidCurrentUrl: function() {
//			if (this.urlLocked) return this.currentUrl;
//			return D.isDocumentDiigoable(gBrowser.contentDocument) ? gBrowser.contentDocument.location.href : null;
//		},
		
		/**
		 * Returns the window object of sidebar
		 */
		getSidebarWindow: function() {
			return this.isShown() ? this.pageWin : null;
		},
		
		/**
		 * Call me after signIn
		 */
		onSignIn: function() {},
		onSignOut: function() {
			this.close();
		},
		
		/**
		 * Tells whether the side is shown
		 */
		isShown: function() {
			var sidebarBox = document.getElementById('sidebar-box');
			return !sidebarBox.hidden && !sidebarBox.collapsed && this.pageWin;
		},
		
		open: function(tab, callback) {
			var z = this;
			
			//if tab is not specified, use the last selected tab stored in prefs
			tab = tab || D.prefs.val('sidebar.selectedTab') || 'bookmarks';
			
			if (!this.isShown()) window.toggleSidebar('viewDiigoSidebar');
			
			
			function ready (event) {
				var doc = event.target;
				//just for document
				if (doc.nodeName == '#document' && doc.URL == 'chrome://diigotb/content/sidebar.html') {
					document.getElementById("sidebar").removeEventListener('DOMContentLoaded', ready, false);
					z.getSidebarWindow().selectTab(tab, callback);
				}
			}
			
			var w = this.getSidebarWindow();
			if (w && w.loaded) {
				w.selectTab(tab, callback);
			} else {
				//if sidebar window is not ready yet, we wait
				document.getElementById("sidebar").addEventListener('DOMContentLoaded', ready, false);
			}
		},
		
		close: function() {
			if (this.isShown()) window.toggleSidebar('viewDiigoSidebar');
		},
		
		lockUrl: function(lock) {
			Utils.dump('[lock url]', lock);
			
			this.urlLocked = lock;
			
			if (!lock)
				this.changeLocation();
			
			this.refresh('lock');
		},
		
		/**
		 * Call me when the current location of gBrowser changes
		 */
		changeLocation: function() {
			//Utils.dump('[sidebar] changeUrl called...');
			if (this.urlLocked) return;
			var t = D.getCurrentContext();
			
			if (this.currentContext != t) {
				//changing current context
				//unlisten old context and listen to new context
				if (this.currentContext) {
					this.currentContext.removeEventListener('ann_add', this);
					this.currentContext.removeEventListener('ann_del', this);
					this.currentContext.removeEventListener('ann_changeMode', this);
					
					this.currentContext.removeEventListener('ic_add', this);
					this.currentContext.removeEventListener('ic_del', this);
//						this.currentContext.removeEventListener('ic_edit', this);
					
					this.currentContext.removeEventListener('pc_add', this);
					this.currentContext.removeEventListener('pc_del', this);
//						this.currentContext.removeEventListener('pc_edit', this);
				}
				
				this.currentContext = t;
				
				if (this.currentContext) {
					this.currentContext.addEventListener('ann_add', this);
					this.currentContext.addEventListener('ann_del', this);
					this.currentContext.addEventListener('ann_changeMode', this);
					
					this.currentContext.addEventListener('ic_add', this);
					this.currentContext.addEventListener('ic_del', this);
//					this.currentContext.addEventListener('ic_edit', this);
				
					this.currentContext.addEventListener('pc_add', this);
					this.currentContext.addEventListener('pc_del', this);
//					this.currentContext.addEventListener('pc_edit', this);
				}

				//Utils.dump('[sidebar] calling refresh("url")...');
				this.refresh('url');
			}
			
		},
		
		/**
		 * Refreshes the sidebar UI (and states)
		 * @params {String}
		 * | 'list' | 'url'
		 */
		refresh: function(what, context, force) {
			if (!this.isShown()) return;
			
			//if no context is provided, do refresh
			//if a context is provided and it's the current context, ignore the refresh unless force
			if (context && (context == this.currentContext ? !force : true)) return;
			
			Utils.dump('[Sidebar] refresh:', what, force ? ' force' : '');
			
			var w = this.getSidebarWindow();
			
			// do nothing if the window has not been loaded
			if (!w.loaded) return;
			
			if (what == 'myBookmarks') {
				if(w.BookmarkTab.isUnread()){
					w.BookmarkTab.populateMyUnreadBookmarks();
				}else{
					w.BookmarkTab.populateMyBookmarks();
				}
			}
			
			if (what == 'bmResult') {
				w.BookmarkTab.selectBookmarkFilter();
			}
			
			if (what == 'myGroups') {
				w.AnnTab.refreshGroups();
			}
			
			if (what == 'list') {
				w.populateMyLists();
			}
			
			if (what == 'url') {
				w.changeUrl();
			}
			
			if (what == 'cached') {
				w.populateCached();
			}
			
			if (what == 'readers') {
				w.populateReaders_async(force);
			}
			
			if (what == 'annotations') {
				w.populateAnnotations_async(force);
			}
			
			if (what == 'pageComments') {
				w.populatePageComments_async(force);
			}
			
			if (what == 'thisUrl') {
				w.populateThisUrl(force);
			}
			
			if (what == 'lock') {
				w.UrlLock.refresh();
			}
			
			if (what == 'tagAutoComplete') {
				w.BookmarkTab.setupTagAutoComplete();
			}
			
			if (what == 'bookmarkSearch') {
				w.BookmarkTab.refreshBookmarkSearch();
			}
			
		},
		
		/**
		 * Asks the sidebar what data it needs
		 * @return {String} 'a b c'
		 */
		needAnythingFromLoadBookmark: function() {
			if (this.isShown()) {
				var w = this.getSidebarWindow();
				var currentTab = w.currentTab;
				if (currentTab == 'annotations' || currentTab == 'readers') {
					return 'pageComments bookmarkInfo readers'
				}
			}
			return '';
		},
		
		//context event listeners
//		onann_add: function(context, ann) {
//			if (!this.isShown()) return;
//			var w = this.getSidebarWindow();
//		},
//		
//		onann_del: function(context, ann) {
//			if (!this.isShown()) return;
//			var w = this.getSidebarWindow();
//		},
//		
//		onann_addGroup: function(context, ann, groupNames) {
//			if (!this.isShown()) return;
//			var w = this.getSidebarWindow();
//		},
//		
//		onann_changeMode: function(context, ann) {
//			if (!this.isShown()) return;
//			var w = this.getSidebarWindow();
//		},
//		
//		onic_add: function(context, c) {
//			if (!this.isShown()) return;
//			var w = this.getSidebarWindow();
//		},
//		
//		onic_del: function(context, c) {
//			if (!this.isShown()) return;
//			var w = this.getSidebarWindow();
//		},
//		
//		onic_edit: function(context, c) {
//			if (!this.isShown()) return;
//			var w = this.getSidebarWindow();
//		},
//		
//		onpc_add: function(context, c) {
//			if (!this.isShown()) return;
//			var w = this.getSidebarWindow();
//		},
//		
//		onpc_del: function(context, c) {
//			if (!this.isShown()) return;
//			var w = this.getSidebarWindow();
//		},
//		
//		onpc_edit: function(context, c) {
//			if (!this.isShown()) return;
//			var w = this.getSidebarWindow();
//		}
	};
	
	//context event handlers
	'pc_add pc_edit pc_del ic_add ic_edit ic_del ann_add ann_del ann_changeMode'.split(' ')
	.forEach(function(eventName) {
		Sidebar['on' + eventName] = function() {
			if (!this.isShown()) return;
			var w = this.getSidebarWindow();
			
			var f = w.AnnTab['on' + eventName];
			if (f) f.apply(w.AnnTab, [].slice.call(arguments, 1));
		}
	});
})();