(function() {
	//items customizable in diigo options
	var TOOLBAR_ITEMS = 'diigotb-tb-searchBox,diigotb-tb-highlight,diigotb-tb-capture,diigotb-tb-comment,diigotb-tb-share,diigotb-tb-message,diigotb-tb-readitLater,diigotb-tb-quickAccessFilter-item,diigotb-tb-options,diigotb-tb-search';
	
	var D = window.diigo;

	var Utils;
	Utils = D["utils"];

	
	var $ = D.$;
	
	var Toolbar = D.Toolbar = {
		init: function() {			
			this.migrateFromHiddenButtonsPref();
		},
		
		onSignIn: function() {
			this.refresh('buttons');
		},
		onSignOut: function() {
			this.refresh('premium');
			//clearHistory also updates colorful hignlights
			//but after signing out, ColorfulFind.currentDoc will not be set properly (onLocationChange not called), causing an exception
			try {
				D.Search.clearHistory();
			} catch(e) {}
		},
		
		migrateFromHiddenButtonsPref: function() {
			var pref = 'toolbar.hiddenButtons'; //hiddenButtonsPrefName
			//only need to migrate if the user has changed the pref
			if (!D.prefs.hasUserValue(pref)) return;
			var a = D.utils.string.split(D.prefs.val(pref) || '', ',');
			if (a.length > 0) {
				D.utils.dump('[toolbar] legacy hiddenButtons pref found and has been changed by user. migrating...', a);
				for (var i = a.length, v; v = a[--i], i >= 0;) {
					D.prefs.val('toolbar.showItems.' + v, false);
				}
				D.prefs.del(pref);
			}
		},
		
		/**
		 * Returns an array of id of items to hide.
		 * Used in options.js
		 */
		getItemsToHide: function() {
			return TOOLBAR_ITEMS.split(',').filter(function(v) {
				return D.prefs.val('toolbar.showItems.' + v) === false;
			});
		},
		
		updaters: {
			bookmark: function() {
				var bookmarked = D.isCurrentLocationBookmarked();
				
				var star = $('#diigotb-tb-bookmarkStar');
				star.attr('bookmarked', !!bookmarked);
				//toolbar tip changes with bookmarked state
				star.attr('tooltiptext', star.attr('tooltiptext' + (bookmarked ? 2 : 1)));
				
				$('diigotb-tb-menuitem-bookmark').attr('label', bookmarked ? 'Edit Bookmark...' : 'Bookmark this Page');
			},
			
			readLater: function() {
				
				if(!D.isCurrentLocationBookmarkable())return;
				//read later button
				var j = $('#diigotb-tb-readitLater');
				var ctx = D.getCurrentContext();
				j.attr('disabled', !ctx);
				
				if (!ctx) return;
				
				var curBm = ctx.bookmark;
				var unread = D.isBookmarkInUnreadList(curBm);
				j.attr({
					unread: unread
				});
			},			
			/**
			 * lights up the star
			 */
			star: function() {
				document.getElementById('diigotb-tb-bookmarkStar').setAttribute('bookmarked', true);
				//var j = $('#diigotb-tb-bookmarkStar');
				//j.attr('bookmarked', !true);
				/*var ele;
				if(document.getElementsByName)
					ele=document.getElementsByName('diigotb-tb-bookmarkStar');
				else
					ele=document.getElementsByClassName("starBtn");
				$(ele).each(function(i,v){
					v.setAttribute('bookmarked', true);
				});*/
			},
			
			'comment': function() {
				$('#diigotb-tb-comment').attr('commented', !!D.isCurrentLocationCommented());
			},
			
			HighlightPen: function() {
				context=D.runtime._focusedContext;
				if(context&&context.isHighlightPen){
					$('#diigotb-tb-highlight').attr('checked','true');
				}else{
					$('#diigotb-tb-highlight').attr('checked','false');
				}
			},
			
			clipMode: function() {
//				var b = D.Clip.inClipMode;
//				$('#diigotb-tb-clip').attr('checked', b);
			},
			
			server: function() {
				var server = D.prefs.val('server');
				$('#diigotb-tb-server').attr({label: 'Server: ' + server})
				.find('[value=' + server + ']').attr({checked: true});
			},
			
			premium: function(){
				try{
				if(D.env.permissions.premium&&D.hasSignedIn()){
					$('#diigotb-tb-upgradebtn').attr({collapsed:false});
					$('#diigotb-menuitem-upgrade').attr({hidden:false});
					$('#diigotb-menuitem-upgrade2').attr({hidden:false});
				}else{
					$('#diigotb-tb-upgradebtn').attr({collapsed:true});
					$('#diigotb-menuitem-upgrade').attr({hidden:true});
					$('#diigotb-menuitem-upgrade2').attr({hidden:true});
				}
				
				//alert(D.env.permissions.capture);
				if(!D.env.permissions.capture){
					$('#diigotb-tb-capture').attr({
						tooltiptext:'Premium feature ~ please upgrade. Enable screenshot capture & annotation. ',
						disabled:true
					});
				}else{
					$('#diigotb-tb-capture').attr({
						tooltiptext:'Click to enter capture mode'
					});
					if(D.isCurrentLocationBookmarkable()){
						$('#diigotb-tb-capture').attr({
							disabled:false
						});						
					}
				}
				}catch(e){
					//alert(e);
				}
			},
			
			trace: function() {
				var trace = D.prefs.val('trace');
				$('#diigotb-tb-trace').attr({checked: trace});
			},
			
			//show/hide toolbar buttons according to prefs
			buttons: function() {
				try{
				//add buttons checked in option window to toolbar if necessary
				
				var itemsToShow = TOOLBAR_ITEMS.split(',').filter(function(v) {
					return D.prefs.val('toolbar.showItems.' + v) !== false;
				});
				//items that should be shown
								
				//if an item that should be shown is not found anywhere in #navigator-toolbox, add it to #diigotb-toolbar
				var toolbox = document.getElementById('navigator-toolbox');
				var diigoToolbar = document.getElementById('diigotb-toolbar');
				var deltaSet = '';
				for (var l = itemsToShow.length, i = 0, v; v = itemsToShow[i], i < l; i++) {
					if (!Utils.dom.isChildOf(document.getElementById(v), toolbox)) {
						Utils.dump('[Toolbar]', v, 'should be shown but not shown');
						
						//if(v == "diigotb-tb-quickAccessFilter-item"){
						//	continue;
						//}
						//the button is not in navigator-toolbox, we add it to diigo toolbar
						deltaSet += ',' + v;
					}
				}
				
				if (deltaSet) {
					Utils.dump('[Toolbar] deltaSet', deltaSet);
					Toolbar.setCurrentSet(Toolbar.getCurrentSet() + deltaSet);
				}

				//set diigohidden attr for each item
//				D.utils.dump(itemsToShow);
				TOOLBAR_ITEMS.split(',').forEach(function(id) {
					var item = document.getElementById(id)
					if(item){
						if (itemsToShow.indexOf(id) > -1)
							item.removeAttribute('diigohidden');
						else{
							item.setAttribute('diigohidden', 'true');
							//alert(id);
						}
					}
				});
				
				D.Chevron.doResize();	
				}catch(e){
					throw(e)
				}
			},
			
			//only enable the update button
			expire: function() {
				$('#diigotb-toolbar *').attr({hidden: true});
				this.update();
			},
			
			highlightPen: function(){
				var szColor=D.prefs.val('penColor');
				D.Highlighter.colorPen.forEach(function(v) {
					$('.diigotb-icon-highlight').toggleClass(v,szColor==v);
				});				
			},
			
			/**
			 * Makes the update button (which is hidden by default) visible
			 */
			update: function() {
				$('#diigotb-tb-update').attr({hidden: false});
			},
			
			toggleHighlight: function() {
				var off = !!D.prefs.val('hl.dontPaint');
				$('#diigotb-tb-menuitem-toggleHighlight').attr('checked', off);
				$('#diigotb-tb-highlight').toggleClass('dontPaint', off);
			},
			
			silentMode: function(){
				var off = D.prefs.val('silentMode');
				$('#diigo-tb-menu-silent').attr({checked: off});
			},
			
			filter: function() {
				D.QuickAccessFilter.refreshUI();
			}
		},
		/**
		 * @param {String} Space separated
		 * e.g. 'user bookmark comment'
		 */
		refresh: function(what) {
			var z = this;
			what.split(' ').forEach(function(v) {
				z.updaters[v]();
			});
		},
		
		/**
		 * @param {String} setName 'default' | 'beginner' | 'advanced'
		 */
		restoreCurrentSet: function(setName) {
			this.setCurrentSet(document.getElementById('diigotb-toolbar').getAttribute(setName + 'set'));
		},
		
		getCurrentSet: function() {
			var tb = $('#diigotb-toolbar')[0];
			//tb.setAttribute("currentset", value);
     		return tb.getAttribute("currentset");
 		},		
 		
		setCurrentSet: function(value) {
			var tb = $('#diigotb-toolbar')[0];
			tb.setAttribute("currentset", value);
     		tb.currentSet = value;
     		document.persist(tb.id, "currentset");
     		
     		// If you don't do the following call, funny things happen
		    try {
		    	BrowserToolboxCustomizeDone(true);
		    }
		    catch (e) { }
		},
		
		restoreMode: function(mode) {
			var tb = $('#diigotb-toolbar')[0];
			
			tb.setAttribute("mode", mode);
     		document.persist(tb.id, "mode");
     		
     		try {
		    	BrowserToolboxCustomizeDone(true);
		    }
		    catch (e) { }
		}
		
	};
	
	/**
	 * The Chevron is something that puts toolbar buttons to a popup menu(chevron popup) 
	 * when there's not enough room in the toolbar.
	 * The chevron popup is a menu button with label ">>", shown in the right most.
	 * Google toolbar does the chevron thing, too.
	 * 
	 * I'm using the word chevron as verb here. 
	 * To chevron means to hide a button in toolbar and show it in the chevron popup popup.
	 * To unchevron means to show a button in toolbar.
	 * 
	 * To chevron a toolbarbutton,
	 * 1. collapse it in toolbar
	 * 2. add an item in the chevron popup, copying styles from toolbarbutton
	 * 3. append toolbarbutton's child menupopup to the chevron item(don't forget to add it back)
	 * 
	 */
	var Chevron = D.Chevron = {
		winWidth: 0,
		_menus: {}, /*{chevid: menuPopupDOMObject, ...}*/
		
		init: function() {
			var z = this;
			
			z.winWidth = window.innerWidth;
						
			$(window).resize(function(event) {
				//only relayout when the width has actually changed
				var newWidth = window.innerWidth;
				
				if (newWidth != z.winWidth) {
					z.winWidth = newWidth;//D.QAF.refreshUI();//
					var re=z.doResize();
					return re;
				}
			});
			
			
			//CAUTION
			//NOTICE
			// we need to reflow buttons after firefox toolbar customization done
			// I hate this kind of overriding, but have no choice
			// Stumple upon is doing the same
			var orig = window.BrowserToolboxCustomizeDone;
			if (orig) {
				window.BrowserToolboxCustomizeDone = function() {
					Utils.dump('[Chevron] BrowserToolboxCustomizeDone');
					try {z.doResize();} catch(e) {Utils.dump('[Chevron] BrowserToolboxCustomizeDone', e);}
					return orig.apply(window, Utils.toArray(arguments));
				};
			}
			
			// Toolbar context menu
			// See function onViewToolbarsPopupShowing in browser.js
			// e.g. resize -> hide -> resize -> show
			document.getElementById('toolbar-context-menu').addEventListener('popupshown', function(event) {
				//find diigo toolbar index
				var i = Utils.array.findIndex(
					Utils.toArray(document.getElementById("navigator-toolbox").childNodes), 
					function(v) {
						return v.getAttribute('toolbarname') == 'Diigo Toolbar';
					}
				);
				
				Utils.dump('[Chevron] toolbar-context-menu diigo toolbar index', i);
				
				if (i > -1) {
					// find the menuitem
					$('menuitem[toolbarindex=' + i + ']', event.target)
						.bind('command', function() {
							Utils.dump('[Chevron] toolbar-context-menu diigo: calling doResize()');
							z.doResize();	
						});
				}
			}, false);
		},
		
		/**
		 * Tells whether a toolbar element can be moved to the chevron menu
		 */
		mayChevron: function(ele) {
			return ele.nodeName != "tooltip" && 
				   ele.nodeType != Node.TEXT_NODE && 
				   //ele.getAttribute('donotchevron') != 'true' &&
				   ele.id != 'diigoChevron';
		},
		
		/**
		 * Returns an array of elements in toolbar that may be chevroned.
		 * @param {Boolean} alreadyChevroned an optional filter flag
		 * 
		 */
		whatToChevron: function(alreadyChevroned) {
			var z = this;
			var eles = [];
			var j = $('#diigotb-toolbar>*');
			j.get().forEach(function(ele) {
				if (z.mayChevron(ele)) {
//					Utils.dump('toolbar resize', 'whatToChevron:', ele.id);
					eles.push(ele);
				}
	
				if(ele.id == 'diigotb-tb-search'&&!$(ele).attr('diigohidden')){
					if(!Utils.toBoolean($('#diigotb-tb-toggleSearchHighlight').attr('diigohidden2'))){
						eles.push($('#diigotb-tb-toggleSearchHighlight')[0]);
					}
					
					var c=$('#Diigo-X-SearchWordsContainer').children('toolbarbutton').each(function(){
						eles.push(this);
					});	
				}
				
				//buttons to chevron:
				if (ele.id == 'diigotb-tb-quickAccessFilter-item'&&!$(ele).attr('diigohidden')) {
					$('*', ele).each(function() {
						if ($(this).is('toolbarbutton,toolbarseparator'))
							eles.push(this);
					});
				}
			});

			if (alreadyChevroned !== undefined) {
				eles = eles.filter(function(ele) {
					return Utils.toBoolean(ele.getAttribute('chevron')) == alreadyChevroned;
				});
				//Utils.dump('toolbar resize', 'alreadyChevroned:', eles.length);
			}

			return eles;
		},

		/**
		 * Becareful! Very subtle logic.
		 */
		doResize: function() {
			//try{
			
			var z = this;
			var tb = document.getElementById('diigotb-toolbar');

			//do nothing if the toolbar is collapsed
			if (tb.getAttribute('collapsed')=='true') return;
			
			var chevron = document.getElementById("diigoChevron");
			var upgrad=$("#diigotb-tb-upgradebtn")[0];
			if(!chevron||!upgrad)return;
			
			var roomLeft;
			function isOverflowing() {
				//Utils.dump('toolbar resize', 'overflowing: ',D.ff4up, tb.boxObject.width+" : "+upgrad.boxObject.width +" : "+ chevron.boxObject.x +" : "+ chevron.boxObject.width);
				var existWidth=chevron.boxObject.x + chevron.boxObject.width + (upgrad?upgrad.boxObject.width-4:0);
				roomLeft = tb.boxObject.width - existWidth;
				Utils.dump('toolbar resize', 'overflowing: ', roomLeft, existWidth);
				return roomLeft < 0;
			}
			
			Utils.dump('toolbar resize', 'overflowing: ', isOverflowing());
			
			function tryToSqueezeSearchBox() {
				Utils.dump('[Chevron] squeezing search box', roomLeft);
				
				var j = $('#diigotb-tb-searchBox');
				if (!j.attr('diigohidden') && !Utils.toBoolean(j.attr('chevron'))) {
					jj = $('#Diigo-X-SearchBox-Container');
					var minWidth = parseInt(jj[0].getAttribute('minwidth'));
					var squeeze = jj[0].boxObject.width - minWidth;
					
					Utils.dump('[Chevron] max squeeze ', squeeze, squeeze >= -roomLeft);
					if (squeeze >= -roomLeft){
						var newlength=jj[0].boxObject.width - (-roomLeft);
						jj.parent().attr('width', newlength);
						jj.attr('width', newlength);
					}
				}
				return !isOverflowing();
			}
			
			if (isOverflowing()) {
				chevron.collapsed = false;
				var whatToChevron = z.whatToChevron(false);
				//[  ] -> >>
				//try to chevron util it's not overflowing
				for (var i = whatToChevron.length - 1, ele; ele = whatToChevron[i], i >= 0; i--) {
					if (ele.id == 'diigotb-tb-searchBox' && tryToSqueezeSearchBox()) {
						break;
					} else {
						Utils.dump('[Chevron] hidden id', ele.id);
						z.chevron(ele, true);
						if (!isOverflowing()) break;
					}
				}
			} else {
				//[  ] <- >>
				chevron.collapsed = true;
				var chevroned = z.whatToChevron(true);
				
				//so we still have some room, try to unchevron to fill the free space
				for (var i = 0, len = chevroned.length, ele; ele = chevroned[i], i < len; i++) {
					z.chevron(ele, false);
					
					//if the unchevron action makes it overflow, undo
					if (isOverflowing()) {
						if (ele.id == 'diigotb-tb-searchBox' && tryToSqueezeSearchBox()) {
						} else {
							z.chevron(ele, true);
							chevron.collapsed = false;
						}
						
						break;
					}
				}
			}
			//}catch(e){alert(e);}
		
		},
		
		chevron: function(ele, on) {
			on = !!on;
			if (Utils.toBoolean(ele.getAttribute('chevron')) == on) return;
			
			var j = $(ele);
			j.attr({chevron: on, collapsed: on});
			
			if (on) {
				//set an chevron id attribute
				var chevid = j.attr('chevid');
				
				if (!chevid) {
					chevid = ele.id || Math.random().toString().replace('.', '');
					j.attr({chevid: chevid});
				}
				
				//save a reference to the menupopup for menu(buttons)
				if (!this._menus[chevid] && (ele.type == 'menu' || ele.type == 'menu-button')) {
					var popup = ele.getElementsByTagName('menupopup')[0];
					if (popup)
						this._menus[chevid] = popup;
				}
			} else {
				var chevid = ele.getAttribute('chevid');
				var popup = this._menus[chevid];
				
				//take back my child!
				if (popup) {
					//NOTICE
					//I'm not sure why, but firefox sets visibility to 'collapsed'
					//we need to change it to 'visible' or the popupmenu can't not been seen
					popup.style.visibility = 'visible';
					ele.appendChild(popup);
				}
			}
		},
		
//		getWidth: function(ele) {
//			return ele.boxObject.width;
//		},
		
		
		/**
		 * Build the chevron popup
		 */
		onChevronMenuPopupShowing: function(event, menu) {
			var z = this;
			var eles = z.whatToChevron(true);
			
			//empty the menu
			while (menu.firstChild)
				menu.removeChild(menu.firstChild);
				
			eles.forEach(function(ele) {
				if (ele.getAttribute('diigohidden') != 'true')
					addMenuItem(ele);
			});
			
			function addMenuItem(ele) {
				//Utils.dump('[Chevron] adding menuitem', ele.localName, ele.type);
				
				if (Utils.toBoolean(ele.getAttribute('dontshowinchevron'))) return;
				var item;
				
				if (ele.localName == "toolbarseparator" && menu.firstChild) {
					item = document.createElement("menuseparator");
				} else if(ele.localName == 'toolbarbutton') {
					var j = $(ele);
					
					if (ele.type == 'menu' || ele.type == 'menu-button') {
						item = Utils.dom.buildOne('menu', {
							'class': 'menu-iconic',
						});
						
						//may I borrow your child?
						var popup = z._menus[ele.getAttribute('chevid')];
						popup && item.appendChild(popup);
					} else {
						item = Utils.dom.buildOne('menuitem', {
							'class': 'menuitem-iconic'
						});
						
//						if (j.hasClass('diigotb-searchKeyword')) {
//							//colorful search keyword highlight
//							item.style.backgroundColor = j.css('backgroundColor');
//						}
					}
					
					//filterid is used by quick access filter
					copyAttrs(item, ele, ['chevid', 'checked', 'disabled', 'command', 'oncommand', 'tooltiptext', 'label', 'image', 'filterid']);
					item.style.listStyleImage = j.css("listStyleImage");
					item.style.MozImageRegion = j.css("MozImageRegion");
				} else {
					return;
				}
				
				menu.appendChild(item);
			}
			
			function copyAttrs(to, from, attrs) {
				for (var i = 0, len = attrs.length, attr; attr = attrs[i], i < len; i++) {
					if (from.hasAttribute(attr))
						to.setAttribute(attr, from.getAttribute(attr));
				}
			}
		} 
	};
	
	/**
	 * Colorful in page find
	 * Many thanks to google toolbar
	 */
	var ColorfulFind = D.ColorfulFind = {
		//flag, whether to highlight find terms
		highlightOn: false,
		currentDoc: null,
		currentKeywords: [],
		
		//avoid refreshing keywords too frequently
		_keyUpTimer: null,
		
		init: function() {
			var z = this;
			
			z.updateButtons();
			
			$('#diigotb-txt-search').keyup(function() {
				clearTimeout(z._keyUpTimer);
				z._keyUpTimer = setTimeout(function() {
					z.keywordsChanged();
					z._keyUpTimer = null;
				}, 365); //the timeout should be the average interval between 2 key strokes in normal typing
				
			});
		},
		
		/**
		 * Detects keyword input changes and takes action
		 */
		keywordsChanged: function () {
			var val = Utils.string.strip2(document.getElementById('diigotb-txt-search').value);
			this.currentKeywords = this.searchTermParser._splitUpSearchQuery(val);
			this.updateButtons();
			
			this.toggleHighlight(this.highlightOn);
		},
		
		updateButtons: function() {
			var z = this;
			var keywords = z.currentKeywords;
			var jContainer = $('#Diigo-X-SearchWordsContainer');
			
			//var addedWidth=jContainer[0].boxObject.width+$('#diigotb-tb-toggleSearchHighlight')[0].width;
			
			keywords.slice(0, 5).forEach(function(kw, i) {
				var j = $(jContainer[0].childNodes[i] || 
						  jContainer[0].appendChild(Utils.dom.buildOne(
						  	'toolbarbutton', {
						  		'class': 'diigotb-toolbarButton noIcon diigotb-searchKeyword',
						  		crop: 'end',
						  		n: i % 6, 
						  		'diigotb-show-text': "always",
						  		'oncommand': 'diigo.ColorfulFind.selectNextKeyword(this.label, !event.shiftKey)'
						  	}))
						);
				j.attr({label: kw});	
			});
			
			jContainer.find('>toolbarbutton:gt(' + (keywords.length - 1) + ')').remove();
			
			//Don't show "word find and highlight icons" unless there is something in the search bar
			$('#Diigo-X-SearchWordsContainer,#diigotb-tb-toggleSearchHighlight')
			 .attr({diigohidden2: keywords.length == 0});
			
			//var newWidth=$('#diigotb-tb-search')[0].boxObject.width + jContainer[0].boxObject.width+$('#diigotb-tb-toggleSearchHighlight')[0].width - addedWidth;
			
		//	Utils.dump('[Colorful find]', $('#diigotb-tb-search')[0].boxObject.width +" : "+ jContainer[0].boxObject.width+" : "+addedWidth);
			//$('#diigotb-tb-search').attr('width', newWidth);
			
			D.Chevron.doResize();
		},
		
		onReadyLocationChange: function() {
			this.currentDoc = content.document;
			
			if (Utils.toBoolean($('#diigotb-tb-search').attr('diigohidden')) || !D.isURIDiigoable(this.currentDoc.URL)) return;
			
			context=D.runtime._focusedContext;
			var domain=(context&&context.bookmark)?context.bookmark.domain:this.currentDoc.domain;
			var termStr=D.ColorfulFind.searchTermSniffer.capture(domain,content.location.toString());
			if (termStr) {
				document.getElementById('diigotb-txt-search').value = termStr;
				this.keywordsChanged();
			}
			
			this.toggleHighlight(this.highlightOn);
		},
		
		toggleHighlight: function(on) {
			var z = this;
			
			on = on === undefined ? !z.highlightOn : on;
			this.highlightOn = on;
			$('#diigotb-tb-toggleSearchHighlight').attr('checked', on);
			$('#Diigo-X-SearchWordsContainer').attr({diigocolorful: on});
			this.currentDoc = content.document;
			
			z.unhighlight();
			if (on) {
				z.currentKeywords.forEach(function(kw, i) {
					z.highlightKeyword(kw, $('#Diigo-X-SearchWordsContainer toolbarbutton').eq(i).css('backgroundColor'));
				});
				
			} else {
				
			}
			
		},
		
		highlightKeyword: function(keyword, color) {
			//gFindBar.highlightDoc(color, null, keyword);
			this._highlightWindow(this.currentDoc.defaultView, keyword, color);
		},
		
		_highlightWindow: function(win, text, color/*, idIndex*/) {
			var count = 0;
			if (win.frames != null) {
				for (var i = 0;i < win.frames.length; i++) {
					var numHighlight = this._highlightWindow(win.frames[i], text,
							color/*, idIndex + count*/);
					count += numHighlight
				}
			}
			var doc = win.document;
			if (doc != null) {
				var body = doc.body;
				if (body != null) {
					var docRange = doc.createRange();
					docRange.selectNode(body);
					var startRange = docRange.cloneRange();
					startRange.collapse(true);
					var stopRange = docRange.cloneRange();
					stopRange.collapse(false);
					var findRange = Components.classes["@mozilla.org/embedcomp/rangefind;1"]
							.createInstance();
					if (findRange != null
							&& findRange instanceof Components.interfaces.nsIFind) {
						findRange.caseSensitive = false;
						findRange.findBackwards = false;
						var result = findRange.Find(text, docRange, startRange,
								stopRange);
						while (null != result) {
							if (result.endContainer != null
									&& result.startContainer != null
									&& result.endContainer.nodeType == result.endContainer.TEXT_NODE
									&& result.startContainer.nodeType == result.startContainer.TEXT_NODE
									&& !this
											._isNodeInputNodeOrChildOf(result.startContainer)
									&& !this
											._isNodeInputNodeOrChildOf(result.endContainer)) {
								var startChar = null;
								if (result.startOffset > 0) {
									startChar = result.startContainer.nodeValue
											.charAt(result.startOffset - 1)
								}
								var endChar = result.endContainer.nodeValue
										.charAt(result.endOffset);
								if (!this._isWordCharacter(startChar)
										&& !this._isWordCharacter(endChar)) {
									var spanNode = this
											._highlightRange(result, color, 'diigotb-findTermId'
													/*this._GOOGLE_TOOLBAR_ID
															+ (idIndex + count)*/);
									startRange.selectNode(spanNode);
									count++
								} else {
									startRange = result
								}
							} else {
								startRange = result
							}
							startRange.collapse(false);
							result = findRange.Find(text, docRange, startRange,
									stopRange)
						}
					}
				}
			}
			return count
		},
		
		_isWordCharacter: function(ch) {
			if (ch == null || typeof ch != "string" || ch.length != 1) {
				return false
			}
			var code = ch.charCodeAt(0);
			return code >= 48 && code <= 57 || code >= 65 && code <= 90 || code >= 97
					&& code <= 122 || code >= 138 && code <= 142 || code >= 154
					&& code <= 159 || code >= 192 && code <= 255
		},
		_isNodeInputNodeOrChildOf: function(node) {
			if (node.nodeName.toUpperCase() == "INPUT") {
				return true
			}
			if (node.parentNode != null) {
				return this._isNodeInputNodeOrChildOf(node.parentNode)
			}
			return false
		},
		
		_highlightRange: function(range, color, id) {
			var startContainer = range.startContainer, spanNode = startContainer.ownerDocument
					.createElement("layer");
			spanNode.style.backgroundColor = color;
			spanNode.style.color = "black";
			spanNode.id = id;
			var docfrag = range.extractContents();
			spanNode.appendChild(docfrag);
			range.insertNode(spanNode);
			return spanNode
		},
		
		unhighlight: function(doc) {
			var doc = doc || this.currentDoc;
			var win = doc.defaultView;
			
			for (var i = 0; win.frames && i < win.frames.length; i++) {
		      this.unhighlight(win.frames[i].document)
		    }
		    
		    var span;
		    while (span = doc.getElementById('diigotb-findTermId')) {
		    	while (span.firstChild)
		    		span.parentNode.insertBefore(span.firstChild, span);
		    	span.parentNode.removeChild(span);
		    }
		},
		
		/**
		 * Previous version: unknown
		 * FF2: gFindBar.find(keyword); gFindBar.findNext(); gFindBar.findPrevious();
		 * FF3: gFindBar._find(keyword); gFindBar.onFindAgainCommand(previous);
		 * @param {Boolean} next true: findNext, false: findPrevious
		 */
		selectNextKeyword: function(keyword, next) {
			try{
				if (D.ff3up) {
					if (gFindBar._findField.value == keyword) {
						gFindBar.onFindAgainCommand(!next);
					} else {
						gFindBar._findField.value = keyword;
						gFindBar._find();
					}
				} else {
					var newV =  typeof gFindBar != "undefined";
				
					if(getBrowser().findString == keyword) 
					{
						if(next) 
						{
							newV ? gFindBar.findNext() : findNext();
						}else{
							newV ? gFindBar.findPrevious() : findPrevious();
						}
					}else{
						newV ? gFindBar.find(keyword) : find(keyword);
						if(!next) 
						{
							newV ? gFindBar.findPrevious() : findPrevious()
						}
					}
				}
				
				return;
			}catch(e){
				diigo.utils.dump("[Search] selectNextKeyword", e); 
			}
		}
	};
	
	/**
	 * Splits a search term string into an array of key words.
	 * Call _splitUpSearchQuery
	 */
	ColorfulFind.searchTermParser = {
		_isWhitespace: function(ch) {
			return /^\s$/.test(ch)
		},
		_isPunctuation: function(ch) {
			return /[`~!@#$%^&\*()-+={}\[\]|\\:;\"<>,\.\?\/]/.test(ch)
		},
		_isWordCharacter: function(ch) {
			if (ch == null || typeof ch != "string" || ch.length != 1) {
				return false
			}
			var code = ch.charCodeAt(0);
			return code >= 48 && code <= 57 || code >= 65 && code <= 90 || code >= 97
					&& code <= 122 || code >= 138 && code <= 142 || code >= 154
					&& code <= 159 || code >= 192 && code <= 255
		},
		_isHebrewCharacter: function(ch) {
			return "\u0590" <= ch && "\u05ff" >= ch
		},
		_isBetweenHebrewCharacters: function(str, i) {
			return i > 0 && i < str.length - 1 && this._isHebrewCharacter(str[i - 1])
					&& this._isHebrewCharacter(str[i + 1])
		},
		_tokenizeSearchQuery: function(input) {
			var tokens = [], state = 0;
			for (var i = 0;i < input.length; ++i) {
				var ch = input[i];
				if (state == 0) {
					if (ch == '"') {
						tokens.push(["", true]);
						state = 2
					} else if (!this._isWhitespace(ch) && !this._isPunctuation(ch)) {
						tokens.push([ch, false]);
						state = 1
					}
				} else if (state == 1) {
					if (ch == '"' && this._isBetweenHebrewCharacters(input, i)) {
						tokens[tokens.length - 1][0] += ch
					} else if (this._isWhitespace(ch) || this._isPunctuation(ch)) {
						state = 0
					} else {
						tokens[tokens.length - 1][0] += ch
					}
				} else if (state == 2) {
					if (ch == '"' && !this._isBetweenHebrewCharacters(input, i)) {
						state = 0
					} else {
						tokens[tokens.length - 1][0] += ch
					}
				}
			}
			return tokens
		},
		_removeOperators: function(tokens) {
			var results = [];
			for (var i = 0;i < tokens.length; ++i) {
				var token = tokens[i][0];
				if (tokens[i][1]) {
					if (token.length > 0) {
						results.push(token)
					}
					continue
				}
				if (token.toUpperCase() == "OR") {
					continue
				}
				if (token[0] == "-") {
					continue
				}
				if (token == "*" || token == "/" || token == "^" || token == "%") {
					continue
				}
				if (/\d+\.\.\.\d+/.test(token)) {
					continue
				}
				if (token[0] == "~") {
					token = token.substring(1)
				}
				results.push(token)
			}
			return results
		},
		_normalizeSpacing: function(tokens) {
			var results = [];
			for (var i = 0;i < tokens.length; ++i) {
				var token = tokens[i];
				token = token.replace(/^\s+/, "");
				token = token.replace(/\s+$/, "");
				token = token.replace(/\s+/g, " ");
				if (token.length > 0) {
					results.push(token)
				}
			}
			return results
		},
		_removeDuplicates: function(list) {
			var member = function(array, item) {
				for (var index in array) {
					if (item == array[index]) {
						return true
					}
				}
				return false
			}, result = [];
			for (var i = 0;i < list.length; ++i) {
				if (!member(result, list[i])) {
					result.push(list[i])
				}
			}
			return result
		},
		_splitUpSearchQuery: function(query) {
			var tokens = [];
			tokens = this._tokenizeSearchQuery(query);
			tokens = this._removeOperators(tokens);
			tokens = this._normalizeSpacing(tokens);
			return this._removeDuplicates(tokens)
		}
	};
	
	
	/**
	 * When the user navigates to a search result page,
	 * e.g. http://www.google.com/search?hl=en&q=doctor+daniel&btnG=Google+Search
	 * 
	 * We try to extract the search term and fill our search box
	 * 
	 * Hard to understand? think they are from our codebase!
	 */
	ColorfulFind.searchTermSniffer = {
		__p: [
			/[\/\.]?bing\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?amazon\.[\S]+keywords=([^\&\?\:\=]+)/i,
			/[\/\.]?diigo\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?google\.[\S]+[?&]q=([^\&\?\=]+)/i,
			/[\/\.]?google\.[\S]+as_lq=([^\&\?\:\=]+)/i/*,
			
			/[\/\.]?yahoo\.[\S]+p=([^\&\?\=]+)/i,
			/[\/\.]?yahoo\.[\S]+s=([^\&\?\=]+)/i,
			/[\/\.]?yahoo\.[\S]+keywords_all=([^\&\?\:\=]+)/i,
			
			/[\/\.]?msn\.[\S]+q=([^\&\?\=]+)/i,
			/[\/\.]?msn\.[\S]+ss=([^\&\?\:\=]+)/i,
			
			/[\/\.]?ask\.[\S]+q=([^\&\?\=]+)/i,
			
			/[\/]?del\.icio\.us\/tag\/([^\&\?\:\=]+)/i,
			/[\/]?del\.icio\.us\/search\/\?all=([^\&\?\:\=]+)/i,
			/[\/\.]?technorati\.\/search\/([^\*]+)/i,
			
			/[\/\.]?bloglines\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?feedster\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?feedster\.[\S]+url=([^\&\?\:\=]+)/i,
			/[\/\.]?blogpulse\.[\S]+query=([^\&\?\:\=]+)/i,
			/[\/\.]?diigo\.[\S]+what=([^\&\?\:\=]+)/i,
			/[\/\.]?searcho\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?furl\.[\S]+keyword=([^\&\?\:\=]+)/i,
			/[\/\.]?zniff\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?indeed\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?monster\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?simplyhired\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?workzoo\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?alltheweb\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?lycos\.[\S]+query=([^\&\?\:\=]+)/i,
			/[\/\.]?podscope\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?answers\.[\S]+\/([^\&\?\:\=]+)/i,
			/[\/\.]?about\.[\S]+terms=([^\&\?\:\=]+)/i,
			/[\/\.]?encyclopedia\.[\S]+target=([^\&\?\:\=]+)/i,
			/[\/\.]?shopping\.[\S]+KW=([^\&\?\:\=]+)/i,
			/[\/\.]?become\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?mysimon\.[\S]+qt=([^\&\?\:\=]+)/i,
			/[\/\.]?pricewatch\.[\S]+s=([^\&\?\:\=]+)/i,
			/[\/\.]?download\.[\S]+qt=([^\&\?\:\=]+)/i,
			/[\/\.]?msn\.[\S]+Symbol=([^\&\?\:\=]+)/i,
			/[\/\.]?nasdaq\.[\S]+Symbol=([^\&\?\:\=]+)/i,
			/[\/\.]?imdb\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?blinkx\.[\S]+q=([^\&\?\:\=]+)/i,
			/[\/\.]?singingfish\.[\S]+query=([^\&\?\:\=]+)/i*/
		],
		
		__do: [
			/diigo/i,
			/amazon/i,
			/bing/i,
			/google/i/*,
			/yahoo/i,
			/msn/i,
			/ask/i,
			/del\.icio\.us/i,
			/technorati/i,
			/bloglines/i,
			/feedster/i,
			/blogpulse/i,
			/furl/i,
			/zniff/i,
			/indeed/i,
			/monster/i,
			/simplyhired/i,
			/workzoo/i,
			/alltheweb/i,
			/lycos/i,
			/podscope/i,
			/answers/i,
			/about/i,
			/encyclopedia/i,
			/shopping/i,
			/become/i,
			/mysimon/i,
			/pricewatch/i,
			/download/i,
			/nasdaq/i,
			/imdb/i,
			/blinkx/i,
			/singingfish/i*/
		],
		
		
		/**
		 * Returns a search term string
		 */
		capture: function(domain,url) {
			try {
				if (this.__CheckSearchDomain(domain)) {
					//var url = doc.URL;
					var len = this.__p.length;
					for(var i = 0; i < len; i++){
						var m = url.match(this.__p[i]);
						if (m) {
							return decodeURIComponent(m[1].replace(/\+/g, ' '));
							break;
						}
					}
					
					return '';
				}
			} catch(e) {
				Utils.dump('[capture search term] capture: ', e);
			}
		},
		
		/**
		 * Is the domain of a search engine?
		 */
		__CheckSearchDomain: function (domain) {
			var isdomain = false;
			try{
				var len = this.__do.length;
				for (var i = 0; i < len; i++){ 
					var n = domain.search(this.__do[i]);
					if (n >= 0){
						if (i != 1 || (i == 1 && (domain.search("mail") < 0 || domain.search("search") >= 0))) {
							isdomain=true;
							break;
						}
					}
				}
			} catch(e) {
				Utils.dump('[capture search term] __CheckSearchDomain: ', e);
			}
			return isdomain;
		}
	};
	
	/**
	 * Multi-category search
	 */
	var Search = D.Search = {
		init: function () {
			
		},
		
		/**
		 * Returns the default search category name
		 * Just for convenience
		 */
		getDefaultCategoryName: function() {
			//the first one is the default one
			var cat = D.env.myCustomizedSearch[0];
			return cat ? cat.name : '';	
		},
		
		getSearchMenuLabel: function(selectionText) {
			return 'Search for \"'+(selectionText.length > 14 ? (selectionText.substr(0, 14) + '...') : selectionText).replace(/[\r\n]+/g, ' ')+'\"';// + this.getDefaultCategoryName();
			//return Utils.string.showTpl('Search for "#{0}" in', [(selectionText.length > 14 ? (selectionText.substr(0, 14) + '...') : selectionText).replace(/[\r\n]+/g, ' ')]);
		},
		
		//builds the search popup menu
		buildMenu: function(menu, useNewTab) {
			var newTab = useNewTab ? 'true' : 'false';
			var j = $(menu).empty();
			
			var cats = D.env.myCustomizedSearch;
			
			cats.forEach(function(cat) {
				var m = Utils.dom.buildOne('menu', {
					label: cat.name,
					url: cat.engines[0].url,
					engine: cat.engines[0].name,
					category: cat.name,
					onclick: 'if (event.target == this) diigo.Search.menuSearch(event, ' + newTab + ')'
				}, ['menupopup', {}]);
				
				var popup = m.firstChild;
				cat.engines.forEach(function(en) {
					popup.appendChild(Utils.dom.buildOne('menuitem', {
						label: en.name,
						url: en.url,
						engine: en.name,
						category: cat.name,
						oncommand: 'diigo.Search.menuSearch(event, ' + newTab + ')'
					}));
					
					if (popup.childNodes.length == 1) {
						popup.firstChild.style.fontWeight = 'bold'; //bold the first(default) item
						popup.appendChild(document.createElement('menuseparator'));
					}
				});
				
				menu.appendChild(m);
				
				if (menu.childNodes.length == 1) {
					menu.firstChild.style.fontWeight = 'bold'; //bold the first(default) item
					menu.appendChild(document.createElement('menuseparator'));
				}
			});
			
			menu.appendChild(document.createElement('menuseparator'));
//			menu.appendChild(Utils.dom.buildOne('menuitem', {
//				label: 'Customize',
//				oncommand: 'diigo.utils.firefox.openTab(diigo.urls.getCustomizedSearchURL());'
//			}));
		},
		
		/**
		 * Handles search menu click
		 * type in search box and click search: use current tab
		 * right click menu: open in new tab
		 */
		menuSearch: function(event, useNewTab) {
			var z = this;
			var j = $(event.target);
			
			//hide popup
			//seems that the popup doesn't hide itself when onclick=xxx is used (oncommand is OK)
			$(event.target).parents().andSelf().filter('menupopup,popup').each(function() {this.hidePopup();});
			
			z.goSearch(j.attr('url'), j.attr('engine'), j.attr('category'), z.getSearchTerm(event), useNewTab);
		},
		
		/**
		 * Gets an array of keywords of the search action.
		 * The search keywords may come from either the search input box on toolbar or selected text in current document
		 * 
		 * @param event  Used for deciding where to get the search keywords when forWhat is omitted.
		 * @param {String} forWhat  Optional, may be 'input' | 'selection'.
		 * 
		 * @return {String}
		 */
		getSearchTerm: function(event, forWhat) {
			var ele = event.target;
			var kw = '';
			if (forWhat == 'input' /*|| Utils.dom.isChildOf(ele, document.getElementById('diigotb-tb-searchWeb'))*/) {
				//toolbar input
				kw = $('#diigotb-txt-search').val();
			} else {
				//document selection
				kw = D.getFocusedFrame().getSelection().toString();
			}
			//add to autocomplete history
			this.addToHistory(kw);
			
			return kw;
		},
		
		goSearchDefaultCategory: function(event, forWhat) {
			var cat = D.env.myCustomizedSearch[0];
			var eng = cat.engines[D.prefs.val('curSearchEngine')];
			
			if (!cat || !eng) return;
			
			//hide popup
			$(event.target).parents().andSelf().filter('menupopup,popup').each(function() {this.hidePopup()});
			
			this.goSearch(eng.url, eng.name, cat.name, this.getSearchTerm(event, forWhat), D.prefs.val('opensearchintab')||forWhat != 'input',forWhat == 'input');//
		},
		
		
		insertMyTag: function(){
			searchText=$('#diigotb-txt-search').val();
			searchPopup=document.getElementById("diigotb-txt-search").popup;
			
			function lastIndexOfMultipleSeparator(str) {
				var index = -1, sep = ' ', tail = '';
				arr=D.parseTags(str);
				
				if(arr.length>0){
					tail=arr.pop();
				}
				newarr=D.unparseTags(arr);
				index=newarr.length;
				return [ index, sep ,newarr, tail];
			}
			
			function requestData(q) {
				var sep = lastIndexOfMultipleSeparator(q);
				if (sep[0] > 0) {
					q = $.trim(sep[3]);
				}

				var q = q.toLowerCase();
				if(q[0]=='\"'||q[0]=='\''){
					q=q.substr(1);
				}
				var candidates = [];
				var finded=false;
				var tags=D.env.myTags;
				for ( var i = 0, v; v = tags[i], i < tags.length; i++) {
					if (v.toLowerCase().indexOf(q) == 0) {
						candidates.push(v);
						finded=true;
					}else{
						if(finded)
							break;
					}
					if(candidates.length>10)
						break;
				}
				return candidates;
			}
			
			data=requestData(searchText);
		},
		
		goSearchFindbar : function() {
			var findText=document.getAnonymousElementByAttribute(
					document.getElementById("FindToolbar"), "anonid", "findbar-textbox");

			var cat = D.env.myCustomizedSearch[0];
			var eng = cat.engines[D.prefs.val('curSearchEngine')];

			this.goSearch(eng.url, eng.name, eng.name, findText.value, true,true);
		},		
		
		goSearch: function(urlPattern, engineName, categoryName, term, useNewTab,focus) {
			var url = this.buildSearchUrl(urlPattern, engineName, categoryName, term);
			Utils.dump('[Search] ', engineName, categoryName, term, url);
			if (useNewTab) {
				if(focus)
					Utils.firefox.openTab(url);
				else
					openNewTabWith(url, null, null, null, true);
			}
			else {
				loadURI(url);
			}
		},
		
		buildSearchUrl: function(urlPattern, engineName, categoryName, term) {
			Utils.dump('[Search] buildSearchUrl', urlPattern, engineName, categoryName, term);
			var url = urlPattern;
//			var keywords = keywords.map(function(kw) {
//				if (kw.indexOf(' ') > -1)
//					kw = '"' + kw + '"';
//				return kw;
//			}).join(' ');
			
			if (categoryName == "Current Site") {
				//alert(keywords);
				var p1 = /([^\&\:]+)site\:([^\&]+)/;
				var m = term.match(p1);
				if(m){
					url=url.replace(/diigosearch/g,encodeURIComponent(m[1]));
					url+=m[2];
				}else{
					url=url.replace(/diigosearch/g,encodeURIComponent(term));
					var p2=/(http:\/\/)?([^\/\?\&\:]+)/i;
					url+=content.document.URL.match(p2)[2];
				}
			}else{
				url=url.replace(/diigosearch/g,encodeURIComponent(term));
				url=url.replace(/diigowho/g, D.env.user);
			}
			return url;
		},
		
		/**
		 * Adds a search term to autocomplete history
		 */
		addToHistory: function(term) {
			try{
			var searchparam = document.getElementById("diigotb-txt-search")
								.getAttribute("autocompletesearchparam");
			
			//add tags to autocomplete candidates
			var fhistory=null;
			if("nsIFormHistory" in Components.interfaces){
				fhistory = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory); 
			}else{
				fhistory = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
			}
			
			//fhistory.removeEntriesForName("Diigo-Main-BP-Tag");
			fhistory.addEntry('Diigo-X-SearchBox', term);
			}catch(e){
				//alert(e);
			}
		},
		
		clearHistory: function() {
			var ele = document.getElementById("diigotb-txt-search");
			if (!ele) return;
			
			ele.value = ''; //also clears the text input
			
			var searchparam = ele.getAttribute("autocompletesearchparam");
								
			var fhistory=null;
			if("nsIFormHistory" in Components.interfaces){
				fhistory = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory); 
			}else{
				fhistory = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
			}
			
			fhistory.removeEntriesForName('Diigo-X-SearchBox');
			
			ColorfulFind.keywordsChanged();
		}
	};
	
})();