var diigoUtil;


(function() {
const Ci = Components.interfaces;
const Cc = Components.classes;

/**
 * utils
 */
var Utils = diigoUtil = {
	dump: function(aMessage) {
		if (window.diigo.prefs&&!window.diigo.prefs.val('trace')) return;
		
		var args = [].slice.call(arguments);
		args.unshift('DIIGO');
		
	    this.consoleService = this.consoleService || Cc["@mozilla.org/consoleservice;1"]
	                                 .getService(Ci.nsIConsoleService);
	    //logStringMessage sometimes causes Error: Component is not available = NS_ERROR_NOT_AVAILABLE
		//the error has sth to do with args.join(' ') part
	    
	    this.consoleService.logStringMessage("[diigo]: " + args.map(function(v) {return v ? v.toString() : v;}).join(' '));
	    //this.consoleService.logStringMessage("[diigo]: " + args.join(' '));
	    
	    if (window.Firebug) {
	    	try {
	    		Firebug.Console.logFormatted(args);
	    	} catch(e) {}
	    }
	},
	
	fork: function(f) {
		setTimeout(function() {
			f();
		}, 13);
	}
};

Utils.lang = {
	/**
	 * From YUI, with my love
	 */
	/**
     * Determines whether or not the provided object is an array.
     * Testing typeof/instanceof/constructor of arrays across frame 
     * boundaries isn't possible in Safari unless you have a reference
     * to the other frame to test against its Array prototype.  To
     * handle this case, we test well-known array properties instead.
     * properties.
     * @method isArray
     * @param {any} o The object being testing
     * @return Boolean
     */
    isArray: function(o) { 

        if (o) {
           var l = this;
           return l.isNumber(o.length) && l.isFunction(o.splice) && 
                  !l.hasOwnProperty(o.length);
        }
        return false;
    },

    /**
     * Determines whether or not the provided object is a boolean
     * @method isBoolean
     * @param {any} o The object being testing
     * @return Boolean
     */
    isBoolean: function(o) {
        return typeof o === 'boolean';
    },
    
    /**
     * Determines whether or not the provided object is a function
     * @method isFunction
     * @param {any} o The object being testing
     * @return Boolean
     */
    isFunction: function(o) {
        return typeof o === 'function';
    },
        
    /**
     * Determines whether or not the provided object is null
     * @method isNull
     * @param {any} o The object being testing
     * @return Boolean
     */
    isNull: function(o) {
        return o === null;
    },
        
    /**
     * Determines whether or not the provided object is a legal number
     * @method isNumber
     * @param {any} o The object being testing
     * @return Boolean
     */
    isNumber: function(o) {
        return typeof o === 'number' && isFinite(o);
    },
      
    /**
     * Determines whether or not the provided object is of type object
     * or function
     * @method isObject
     * @param {any} o The object being testing
     * @return Boolean
     */  
    isObject: function(o) {
		return (o && (typeof o === 'object' || this.isFunction(o))) || false;
    },
        
    /**
     * Determines whether or not the provided object is a string
     * @method isString
     * @param {any} o The object being testing
     * @return Boolean
     */
    isString: function(o) {
        return typeof o === 'string';
    },
        
    /**
     * Determines whether or not the provided object is undefined
     * @method isUndefined
     * @param {any} o The object being testing
     * @return Boolean
     */
    isUndefined: function(o) {
        return typeof o === 'undefined';
    },
    
    /**
     * Determines whether or not the property was added
     * to the object instance.  Returns false if the property is not present
     * in the object, or was inherited from the prototype.
     * This abstraction is provided to enable hasOwnProperty for Safari 1.3.x.
     * There is a discrepancy between YAHOO.lang.hasOwnProperty and
     * Object.prototype.hasOwnProperty when the property is a primitive added to
     * both the instance AND prototype with the same value:
     * <pre>
     * var A = function() {};
     * A.prototype.foo = 'foo';
     * var a = new A();
     * a.foo = 'foo';
     * alert(a.hasOwnProperty('foo')); // true
     * alert(YAHOO.lang.hasOwnProperty(a, 'foo')); // false when using fallback
     * </pre>
     * @method hasOwnProperty
     * @param {any} o The object being testing
     * @return Boolean
     */
    hasOwnProperty: function(o, prop) {
        if (Object.prototype.hasOwnProperty) {
            return o.hasOwnProperty(prop);
        }
        
        return !this.isUndefined(o[prop]) && 
                o.constructor.prototype[prop] !== o[prop];
    },
	
	/**
	 * The following are not from YUI!
	 */
	extend: function(/* dst, src1, src2, ... */) {
		var args = [].slice.call(arguments);
		var dst = args.shift();
		
		for (var i = 0, l = args.length, src; src = args[i], i < l; i++) {
			for (var k in src) {
				dst[k] = src[k];
			}
		}
		return dst;
	},
	
	protectiveExtend: function(dst, src) {
		for (var k in src)
			if (dst[k] !== undefined) dst[k] = src[k];
	},
	
	/**
	 * @param {Function} filterFn
	 * filterFn(key) -> newKey
	 * if !newKey, the property is skipped
	 * if newKey === true, the original key is used
	 * else dst[newKey] = src[key]
	 */
	extendFiltered: function(/* dst, src1, src2, ..., srcn, filterFn*/) {
		var args = [].slice.call(arguments),
		dst = args.shift(),
		filterFn = args.pop();
		
		for (var i = 0, l = args.length, src; src = args[i], i < l; i++) {
			for (var k in src) {
				var kk = filterFn(k);
				
				if (kk) {
					dst[kk === true ? k : kk] = src[k];
				}
			}
		}
		return dst;
	},
	
	toArray: function(obj) {
		return [].slice.call(obj);
	},
	
	toBoolean: function(a) {
		//false, undefined, null, 0 and '' evaluate to false. !a is true and false is returned.
		
		//if a evaluates to true, it may be an object(not null), a nonzero number or a string and only for '0' and 'false' we should return false. 
		
		return !a ? false : a != 'false' && a != '0';
	},
	
	each: function(obj, fn, thiz) {
		if (obj.forEach)
			obj.forEach(fn, thiz);
		else
		for (var k in obj) {
			fn.call(thiz, obj[k], k);
		}
	}
};

//map Utils.lang.* to Utils.*
Utils.lang.extend(Utils, Utils.lang);

/**
 * utils.array
 */
Utils.array = {
	findIndex: function(arr, fn, start) {
		start = start || 0;
		for (var i = start, l = arr.length, v; v = arr[i], i < l; i++) {
			if (fn(v)) return i;
		}
		return -1;
	}, 
	
	find: function(arr, fn, start) {
		var i = this.findIndex(arr, fn, start);
		return i > -1 ? arr[i] : null;
	},
	
	/**
	 * Tells whether something is in an array. Comparison uses ===
	 * @param {Array} a
	 * @param {Object} e
	 */
	inArray: function(a, e) {
		var i = a.length;
		while(i--)
			if (a[i] === e) return true;
		return false;
	},
	
	/**
	 * Selfmodifying concat
	 * a.concat!(b)
	 */
	concatMe: function (a, b) {
		[].splice.apply(a, [a.length, 0].concat(b));
		return a;
	},
	
	//returns a copy
	unique: function(arr, fn) {
		var r = [];
	
		for (var i = 0, len = arr.length, v; v = arr[i], i < len; i++) {
			if (!fn) {
				if (r.indexOf(v) == -1)
					r.push(v);
			}
			else if (!this.find(r, function(vv) {return fn(vv, v);}))
				r.push(v);
		}
		return r;
	},
	
	//returns a copy
	reverse: function(arr) {
		var r = new Array(arr.length);
		for (var i = arr.length - 1, j = 0; i >=0; i--, j++) {
			r[j] = arr[i];
		}
		return r;
	}
};

//shortcut
Utils.a = Utils.array;

/**
 * utils.string
 */
Utils.string = {
	md52: function(istream) { 

		var ch = Components.classes["@mozilla.org/security/hash;1"]
	                            .createInstance(Components.interfaces.nsICryptoHash);
	         // we want to use the MD5 algorithm
	         ch.init(ch.MD5);
	         // this tells updateFromStream to read the entire file
	         const PR_UINT32_MAX = 0xffffffff;
	         ch.updateFromStream(istream, PR_UINT32_MAX);
	         // pass false here to get binary data back
	         var hash = ch.finish(false);
	         
	         // return the two-digit hexadecimal code for a byte
	         function toHexString(charCode)
	         {
	           return ("0" + charCode.toString(16)).slice(-2);
	         }

	         // convert the binary hash data to a hex string.
	 		var v=[];
			for(i in hash){
				if(typeof hash[i] === "string"){
					var hex=hash.charCodeAt(i);
					var n=toHexString(hex);
					v.push(n);
				}
			}
			var s = v.join("");

	    return s;
	},
	md53: function(string) { 
		var converter =
		  Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
		    createInstance(Components.interfaces.nsIScriptableUnicodeConverter);

		// we use UTF-8 here, you can choose other encodings.
		converter.charset = "UTF-8";
		// result is an out parameter,
		// result.value will contain the array length
		var result = {};
		// data is an array of bytes
		var data = converter.convertToByteArray(string, result);
		var ch = Components.classes["@mozilla.org/security/hash;1"]
		                   .createInstance(Components.interfaces.nsICryptoHash);
		ch.init(ch.MD5);
		ch.update(data, data.length);
		var hash = ch.finish(false);

		// return the two-digit hexadecimal code for a byte
		function toHexString(charCode)
		{
		  return ("0" + charCode.toString(16)).slice(-2);
		}

		// convert the binary hash data to a hex string.
		var v=[];
		for(i in hash){
			if(typeof hash[i] === "string"){
				var hex=hash.charCodeAt(i);
				var n=toHexString(hex);
				v.push(n);
			}
		}
		var s = v.join("");

	    return s;
	},
	md54: function(path) { 
		var f = Components.classes["@mozilla.org/file/local;1"]
		                  .createInstance(Components.interfaces.nsIFile);
		f.initWithPath(path);
		var istream = Components.classes["@mozilla.org/network/file-input-stream;1"]           
		                        .createInstance(Components.interfaces.nsIFileInputStream);
		// open for reading
		istream.init(f, 0x01, 0444, 0);

		var ch = Components.classes["@mozilla.org/security/hash;1"]
	                            .createInstance(Components.interfaces.nsICryptoHash);
	         // we want to use the MD5 algorithm
	         ch.init(ch.MD5);
	         // this tells updateFromStream to read the entire file
	         const PR_UINT32_MAX = 0xffffffff;
	         ch.updateFromStream(istream, PR_UINT32_MAX);
	         // pass false here to get binary data back
	         var hash = ch.finish(false);
	         
	         // return the two-digit hexadecimal code for a byte
	         function toHexString(charCode)
	         {
	           return ("0" + charCode.toString(16)).slice(-2);
	         }

	         // convert the binary hash data to a hex string.
	 		var v=[];
			for(i in hash){
				if(typeof hash[i] === "string"){
					var hex=hash.charCodeAt(i);
					var n=toHexString(hex);
					v.push(n);
				}
			}
			var s = v.join("");

	    return s;
	},
	md5: function(sMessage) { 
		function RotateLeft(lValue, iShiftBits) {  
			return (lValue<<iShiftBits) | (lValue>>>(32-iShiftBits));  
		} 
		function AddUnsigned(lX,lY) { 
			var lX4,lY4,lX8,lY8,lResult; 
			lX8 = (lX & 0x80000000); 
			lY8 = (lY & 0x80000000); 
			lX4 = (lX & 0x40000000); 
			lY4 = (lY & 0x40000000); 
			lResult = (lX & 0x3FFFFFFF)+(lY & 0x3FFFFFFF); 
			if (lX4 & lY4) return (lResult ^ 0x80000000 ^ lX8 ^ lY8); 
			if (lX4 | lY4) { 
			if (lResult & 0x40000000) return (lResult ^ 0xC0000000 ^ lX8 ^ lY8); 
			else return (lResult ^ 0x40000000 ^ lX8 ^ lY8); 
			} else return (lResult ^ lX8 ^ lY8); 
		} 
		function F(x,y,z) { return (x & y) | ((~x) & z); } 
		function G(x,y,z) { return (x & z) | (y & (~z)); } 
		function H(x,y,z) { return (x ^ y ^ z); } 
		function I(x,y,z) { return (y ^ (x | (~z))); } 
		function FF(a,b,c,d,x,s,ac) { 
			a = AddUnsigned(a, AddUnsigned(AddUnsigned(F(b, c, d), x), ac)); 
			return AddUnsigned(RotateLeft(a, s), b); 
		} 
		function GG(a,b,c,d,x,s,ac) { 
			a = AddUnsigned(a, AddUnsigned(AddUnsigned(G(b, c, d), x), ac)); 
			return AddUnsigned(RotateLeft(a, s), b); 
		} 
		function HH(a,b,c,d,x,s,ac) { 
			a = AddUnsigned(a, AddUnsigned(AddUnsigned(H(b, c, d), x), ac)); 
			return AddUnsigned(RotateLeft(a, s), b); 
		} 
		function II(a,b,c,d,x,s,ac) { 
			a = AddUnsigned(a, AddUnsigned(AddUnsigned(I(b, c, d), x), ac)); 
			return AddUnsigned(RotateLeft(a, s), b); 
		} 
		function ConvertToWordArray(sMessage) { 
			var lWordCount; 
			var lMessageLength = sMessage.length; 
			var lNumberOfWords_temp1=lMessageLength + 8; 
			var lNumberOfWords_temp2=(lNumberOfWords_temp1-(lNumberOfWords_temp1 % 64))/64; 
			var lNumberOfWords = (lNumberOfWords_temp2+1)*16; 
			var lWordArray=Array(lNumberOfWords-1); 
			var lBytePosition = 0; 
			var lByteCount = 0; 
			while ( lByteCount < lMessageLength ) { 
				lWordCount = (lByteCount-(lByteCount % 4))/4; 
				lBytePosition = (lByteCount % 4)*8; 
				lWordArray[lWordCount] = (lWordArray[lWordCount] | (sMessage.charCodeAt(lByteCount)<<lBytePosition)); 
				lByteCount++; 
			} 
			lWordCount = (lByteCount-(lByteCount % 4))/4; 
			lBytePosition = (lByteCount % 4)*8; 
			lWordArray[lWordCount] = lWordArray[lWordCount] | (0x80<<lBytePosition); 
			lWordArray[lNumberOfWords-2] = lMessageLength<<3; 
			lWordArray[lNumberOfWords-1] = lMessageLength>>>29; 
			return lWordArray; 
		}
		 
		function WordToHex(lValue) { 
			var WordToHexValue="",WordToHexValue_temp="",lByte,lCount; 
			for (lCount = 0;lCount<=3;lCount++) { 
				lByte = (lValue>>>(lCount*8)) & 255; 
				WordToHexValue_temp = "0" + lByte.toString(16); 
				WordToHexValue = WordToHexValue + WordToHexValue_temp.substr(WordToHexValue_temp.length-2,2); 
			} 
			return WordToHexValue; 
		} 
	    var x=Array(); 
	    var k,AA,BB,CC,DD,a,b,c,d; 
	    var S11=7, S12=12, S13=17, S14=22; 
	    var S21=5, S22=9 , S23=14, S24=20; 
	    var S31=4, S32=11, S33=16, S34=23; 
	    var S41=6, S42=10, S43=15, S44=21; 
	    // Steps 1 and 2.  Append padding bits and length and convert to words 
	    x = ConvertToWordArray(sMessage); 
	    // Step 3.  Initialise 
	    a = 0x67452301; b = 0xEFCDAB89; c = 0x98BADCFE; d = 0x10325476; 
	    // Step 4.  Process the message in 16-word blocks 
	    for (k=0;k<x.length;k+=16) { 
	        AA=a; BB=b; CC=c; DD=d; 
	        a=FF(a,b,c,d,x[k+0], S11,0xD76AA478); 
	        d=FF(d,a,b,c,x[k+1], S12,0xE8C7B756); 
	        c=FF(c,d,a,b,x[k+2], S13,0x242070DB); 
	        b=FF(b,c,d,a,x[k+3], S14,0xC1BDCEEE); 
	        a=FF(a,b,c,d,x[k+4], S11,0xF57C0FAF); 
	        d=FF(d,a,b,c,x[k+5], S12,0x4787C62A); 
	        c=FF(c,d,a,b,x[k+6], S13,0xA8304613); 
	        b=FF(b,c,d,a,x[k+7], S14,0xFD469501); 
	        a=FF(a,b,c,d,x[k+8], S11,0x698098D8); 
	        d=FF(d,a,b,c,x[k+9], S12,0x8B44F7AF); 
	        c=FF(c,d,a,b,x[k+10],S13,0xFFFF5BB1); 
	        b=FF(b,c,d,a,x[k+11],S14,0x895CD7BE); 
	        a=FF(a,b,c,d,x[k+12],S11,0x6B901122); 
	        d=FF(d,a,b,c,x[k+13],S12,0xFD987193); 
	        c=FF(c,d,a,b,x[k+14],S13,0xA679438E); 
	        b=FF(b,c,d,a,x[k+15],S14,0x49B40821); 
	        a=GG(a,b,c,d,x[k+1], S21,0xF61E2562); 
	        d=GG(d,a,b,c,x[k+6], S22,0xC040B340); 
	        c=GG(c,d,a,b,x[k+11],S23,0x265E5A51); 
	        b=GG(b,c,d,a,x[k+0], S24,0xE9B6C7AA); 
	        a=GG(a,b,c,d,x[k+5], S21,0xD62F105D); 
	        d=GG(d,a,b,c,x[k+10],S22,0x2441453); 
	        c=GG(c,d,a,b,x[k+15],S23,0xD8A1E681); 
	        b=GG(b,c,d,a,x[k+4], S24,0xE7D3FBC8); 
	        a=GG(a,b,c,d,x[k+9], S21,0x21E1CDE6); 
	        d=GG(d,a,b,c,x[k+14],S22,0xC33707D6); 
	        c=GG(c,d,a,b,x[k+3], S23,0xF4D50D87); 
	        b=GG(b,c,d,a,x[k+8], S24,0x455A14ED); 
	        a=GG(a,b,c,d,x[k+13],S21,0xA9E3E905); 
	        d=GG(d,a,b,c,x[k+2], S22,0xFCEFA3F8); 
	        c=GG(c,d,a,b,x[k+7], S23,0x676F02D9); 
	        b=GG(b,c,d,a,x[k+12],S24,0x8D2A4C8A); 
	        a=HH(a,b,c,d,x[k+5], S31,0xFFFA3942); 
	        d=HH(d,a,b,c,x[k+8], S32,0x8771F681); 
	        c=HH(c,d,a,b,x[k+11],S33,0x6D9D6122); 
	        b=HH(b,c,d,a,x[k+14],S34,0xFDE5380C); 
	        a=HH(a,b,c,d,x[k+1], S31,0xA4BEEA44); 
	        d=HH(d,a,b,c,x[k+4], S32,0x4BDECFA9); 
	        c=HH(c,d,a,b,x[k+7], S33,0xF6BB4B60); 
	        b=HH(b,c,d,a,x[k+10],S34,0xBEBFBC70); 
	        a=HH(a,b,c,d,x[k+13],S31,0x289B7EC6); 
	        d=HH(d,a,b,c,x[k+0], S32,0xEAA127FA); 
	        c=HH(c,d,a,b,x[k+3], S33,0xD4EF3085); 
	        b=HH(b,c,d,a,x[k+6], S34,0x4881D05); 
	        a=HH(a,b,c,d,x[k+9], S31,0xD9D4D039); 
	        d=HH(d,a,b,c,x[k+12],S32,0xE6DB99E5); 
	        c=HH(c,d,a,b,x[k+15],S33,0x1FA27CF8); 
	        b=HH(b,c,d,a,x[k+2], S34,0xC4AC5665); 
	        a=II(a,b,c,d,x[k+0], S41,0xF4292244); 
	        d=II(d,a,b,c,x[k+7], S42,0x432AFF97); 
	        c=II(c,d,a,b,x[k+14],S43,0xAB9423A7); 
	        b=II(b,c,d,a,x[k+5], S44,0xFC93A039); 
	        a=II(a,b,c,d,x[k+12],S41,0x655B59C3); 
	        d=II(d,a,b,c,x[k+3], S42,0x8F0CCC92); 
	        c=II(c,d,a,b,x[k+10],S43,0xFFEFF47D); 
	        b=II(b,c,d,a,x[k+1], S44,0x85845DD1); 
	        a=II(a,b,c,d,x[k+8], S41,0x6FA87E4F); 
	        d=II(d,a,b,c,x[k+15],S42,0xFE2CE6E0); 
	        c=II(c,d,a,b,x[k+6], S43,0xA3014314); 
	        b=II(b,c,d,a,x[k+13],S44,0x4E0811A1); 
	        a=II(a,b,c,d,x[k+4], S41,0xF7537E82); 
	        d=II(d,a,b,c,x[k+11],S42,0xBD3AF235); 
	        c=II(c,d,a,b,x[k+2], S43,0x2AD7D2BB); 
	        b=II(b,c,d,a,x[k+9], S44,0xEB86D391); 
	        a=AddUnsigned(a,AA); b=AddUnsigned(b,BB); c=AddUnsigned(c,CC); d=AddUnsigned(d,DD); 
	    } 
	    // Step 5.  Output the 128 bit digest 
	    var temp= WordToHex(a)+WordToHex(b)+WordToHex(c)+WordToHex(d); 
	    return temp.toLowerCase(); 
	},

	strip: function(s) {
		return s.replace(/^\s+/, '').replace(/\s+$/, '');
	},
	
	trim: function(s) {return this.strip(s);},
	
	/**
	 * Leading and trailing
	 * whitespace is removed, and consecutive whitespace is replaced by a single
	 * space.
	 */
	strip2: function(s) {
		return s.replace(/^\s+/, "")
                .replace(/\s+$/, "")
                .replace(/\s+/g, " ");
	},
	
	trim2: function(s) {return this.strip2(s);},
	
	showTpl: function(s, data) {
		if (data.toTemplateReplacements)
			data = data.toTemplateReplacements();
			
		return s.replace(/(^|.|\r|\n)(#\{(.*?)\})/g, function(m, m1, m2, m3) {
			if (data == null) return '';
  				
			//escape sequence
			var before = m1 || '';
			if (before == '\\') return m2;
			  
			var ctx = data, expr = m3;
			var pattern = /^([^.[]+|\[((?:.*?[^\\])?)\])(\.|\[|$)/, match = pattern.exec(expr);
			if (match == null) return '';
			
			while (match != null) {
				var comp = match[1][0] == '[' ? match[2].replace(/\\\\]/g, ']') : match[1];
				ctx = ctx[comp];
				if (null == ctx || '' == match[3]) break;
				expr = expr.substring('[' == match[3] ? match[1].length : match[0].length);
			    match = pattern.exec(expr);
			}
			  
			return before + (ctx == null ? '' : String(ctx));
		})
	},
	
	parseQuery: function(s, separator) {
	    var match = Utils.string.strip(s).match(/([^?#]*)(#.*)?$/);
	    if (!match) return { };
	    
	    var hash = {};
	    match[1].split(separator || '&').forEach(function(pair) {
	      if ((pair = pair.split('='))[0]) {
	        var key = decodeURIComponent(pair.shift());
	        var value = pair.length > 1 ? pair.join('=') : pair[0];
	        if (value != undefined) value = decodeURIComponent(value);
	        
	        if (key in hash) {
	          if (!Utils.isArray(hash[key])) hash[key] = [hash[key]];
	          hash[key].push(value);
	        }
	        else hash[key] = value;
	      }
	    });
	    
	    return hash;
	},
	
	toQueryString: function(obj) {
		var add = function(key, value) {
		  key = encodeURIComponent(key);
		  if (value === undefined) parts.push(key);
		  else parts.push(key + '=' + (value == null ? '' : encodeURIComponent(value)));
			};
  
		    var parts = [];
		    
		    Utils.each(obj, function(value, key) {
		      if (!key) return;
		      
		      if (value && typeof value == 'object') {
	        if (Utils.isArray(value)) value.forEach(function(value) {
	          add(key, value);
	        });
	        return;
	      }
	      add(key, value);
	    });
	    
	    return parts.join('&');
	},
	
	base64encode: function(str) {
		var base64EncodeChars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
	    var out, i, len;
	    var c1, c2, c3;

	    len = str.length;
	    i = 0;
	    out = "";
	    while(i < len) {
	    c1 = str.charCodeAt(i++) & 0xff;
	    if(i == len)
	    {
	        out += base64EncodeChars.charAt(c1 >> 2);
	        out += base64EncodeChars.charAt((c1 & 0x3) << 4);
	        out += "==";
	        break;
	    }
	    c2 = str.charCodeAt(i++);
	    if(i == len)
	    {
	        out += base64EncodeChars.charAt(c1 >> 2);
	        out += base64EncodeChars.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
	        out += base64EncodeChars.charAt((c2 & 0xF) << 2);
	        out += "=";
	        break;
	    }
	    c3 = str.charCodeAt(i++);
	    out += base64EncodeChars.charAt(c1 >> 2);
	    out += base64EncodeChars.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
	    out += base64EncodeChars.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >>6));
	    out += base64EncodeChars.charAt(c3 & 0x3F);
	    }
	    return out;
	},
	
	/**
	 * http://simonwillison.net/2006/Jan/20/escape/
	 */
	escapeRegexp: function(text) {
	  if (!arguments.callee.sRE) {
	    var specials = [
	      '/', '.', '*', '+', '?', '|',
	      '(', ')', '[', ']', '{', '}', '\\'
	    ];
	    arguments.callee.sRE = new RegExp(
	      '(\\' + specials.join('|\\') + ')', 'g'
	    );
	  }
	  return text.replace(arguments.callee.sRE, '\\$1');
	},
	
	stripTags: function(s) {
		return s.replace(/<\/?[^>]+>/gi, '').replace(/[<>]/g, '');
	},
	
	stripScripts: function(s) {
  	return s.replace(new RegExp('<script[^>]*>([\\S\\s]*?)<\/script>', 'img'), '');
  },
  
  scan: function(s, pattern, func) {
  	if (!pattern.global) throw 'string.scan: pattern is not global';
  	
  	var a;
	while (a = pattern.exec(s)) {
		func(a);
	}
  },
  
  /**
   * In javascript, ''.split('') returns [] but ''.split(',') returns ['']
   * This method first strips the string and splits it. If the stripped string is empty, [] is returned
   */
  split: function(s, by) {
  	s = this.strip(s || '');
  	
  	return s ? s.split(by) : [];
  },
  
  truncate: function(s, maxLen, ellipsis) {
  	if (s.length <= maxLen) return s;
  	if ((typeof ellipsis) != 'string') ellipsis = '...';
  	
  	return s.substr(0, maxLen - ellipsis.length) + ellipsis;
  }
};

//shortcut
Utils.s = Utils.string;

Utils.url = {
	/**
	 * Tells whether a is a subdomain of b
	 */
	isSubDomainOf: function(a, b) {
		
	}
};
/**
 * utils.json
 */
Utils.json = {
    toJSON: function(a) { return JSON.stringify(a); },
    parseJSON: function(a) { return JSON.parse(a); }
};

/**
 * utils.xpcom
 */
Utils.xpcom = function() {
	var _CI = Ci;
	var _CC = Cc;
	
	return {
		CC: function(cName) {
		    return _CC[cName];
		},
		
		CI: function(ifaceName) {
		    return _CI[ifaceName];
		},
		
		CCSV: function(cName, ifaceName) {
		    return _CC[cName].getService(_CI[ifaceName]);        
		},
		
		CCIN: function(cName, ifaceName) {
		    return _CC[cName].createInstance(_CI[ifaceName]);
		},
		
		QI: function(obj, iface) {
		    return obj.QueryInterface(iface);
		}
	};
}();

/**
 * utils.prefs
 * see http://developer.mozilla.org/en/docs/Code_snippets:Preferences#Complex_types for infomation on complex types
 */
 
/**
 * @method getPref
 * @method setPref
 */
Utils.prefs = {
	/**
	 * 
	 * @param {String} prefName
	 * @param {Object} complexType optional e.g nsISupportsString if omitted, a simple type value will be returned
	 */
	getPref: function(prefName, complexType) {
		const prefs = Utils.xpcom.CCSV("@mozilla.org/preferences-service;1", "nsIPrefBranch");
		const nsIPrefBranch = Utils.xpcom.CI("nsIPrefBranch");
		
		var type = prefs.getPrefType(prefName);
		if (type == nsIPrefBranch.PREF_INVALID) //not exist
      return undefined;
    else if (complexType) {
//			if (type == nsIPrefBranch.PREF_STRING)
				return prefs.getComplexValue(prefName, complexType);
		} else if (type == nsIPrefBranch.PREF_STRING)
          return prefs.getCharPref(prefName);
      else if (type == nsIPrefBranch.PREF_INT)
          return prefs.getIntPref(prefName);
      else if (type == nsIPrefBranch.PREF_BOOL)
          return prefs.getBoolPref(prefName);
  },
	
	//supports non-ascii (unicode) string
	getUnicodePref: function(prefName) {
		var r = this.getPref(prefName, Components.interfaces.nsISupportsString);
		return r && r.data;
	},
	
	setUnicodePref: function(prefName, str) {
		var s = Components.classes["@mozilla.org/supports-string;1"]
		      .createInstance(Components.interfaces.nsISupportsString);
		s.data = str;
		return this.setPref(prefName, s, Components.interfaces.nsISupportsString);	
	},
	
  setPref: function(prefName, value, complexType) {
		const prefs = Utils.xpcom.CCSV("@mozilla.org/preferences-service;1", "nsIPrefBranch");
		const nsIPrefBranch = Utils.xpcom.CI("nsIPrefBranch");
			
			if (complexType) {
				prefs.setComplexValue(prefName, complexType, value);
			} else {
				//simple types
        var type = prefs.getPrefType(prefName);
        var prefType;
        
        if (type == nsIPrefBranch.PREF_STRING)
            prefType = 'Char';
        else if (type == nsIPrefBranch.PREF_INT)
            prefType = 'Int';
        else if (type == nsIPrefBranch.PREF_BOOL)
            prefType = 'Bool';
        else {
        	if (typeof value == 'boolean')
        		prefType = 'Bool';
        	else if (typeof value == 'string')
        		prefType = 'Char';
        	else if (typeof value == 'number')
        		prefType = 'Int';
        	else {
        		throw "invalid pref type";
        	}
        }
        
        prefs['set' + prefType + 'Pref'](prefName, value);
			}
          
      return this.getPref(prefName, complexType);
  },
  
  deletePref: function(prefName) {
  	var prefs = Utils.xpcom.CCSV("@mozilla.org/preferences-service;1", "nsIPrefBranch");
  	return prefs.deleteBranch(prefName);
  },
	
	hasUserValue: function(prefName) {
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
    getService(Components.interfaces.nsIPrefService);

		return prefs.prefHasUserValue(prefName);
	}
};


/**
 * utils.func
 */
Utils.func = {
	/**
	 * Returns a throttled version of a function.
	 * The throttled function cannot run more frequently than once every interval time.
	 * This is useful when a computationally expensive function is triggered too frequently.
	 * e.g. IE triggers resize event continuously. Here we limit a time consuming relayout function to be invoked no more than twice per second.
	 * window.onresize = throttle(relayout, 500); 
	 * 
	 * @param {Function} f
	 * @param {Integer} minInterval Defaults to 500(ms)
	 */
	throttle: function(f, minInterval) {
		minInterval = minInterval || 500;
		var busy = false, timerId = null, pending = false;
		return function() {
			var args = Utils.toArray(arguments);
			
			if (busy) {
				//wait and run
				pending = true;
			} else {
				//run and appear busy until minInterval time later
				busy = true;
				
				timerId = setTimeout(function() {
					clearTimeout(timerId);
					
					if (pending) f.apply(null, args);
					
					busy = timerId = pending = false;
				}, minInterval);
				
				f.apply(null, args);
			}
		};
	}
};

Utils.each({
  argumentNames: function() {
    var names = this.toString().match(/^[\s\(]*function\s*\((.*?)\)/)[1].split(",").map(function(s) {return Utils.string.strip(s);});
    return names.length == 1 && !names[0] ? [] : names;
  },
  
  bind: function() {
    if (arguments.length < 2 && arguments[0] === undefined) return this;
    var __method = this, args = [].slice.call(arguments), object = args.shift();
    return function() {
      return __method.apply(object, args.concat([].slice.call(arguments)));
    }
  },
  
  bindAsEventListener: function() {
    var __method = this, args = [].slice.call(arguments), object = args.shift();
    return function(event) {
      return __method.apply(object, [event || window.event].concat(args));
    }
  },
  
  curry: function() {
    if (!arguments.length) return this;
    var __method = this, args = [].slice.call(arguments);
    return function() {
      return __method.apply(this, args.concat([].slice.call(arguments)));
    }
  },

  delay: function() { 
    var __method = this, args = [].slice.call(arguments), timeout = args.shift() * 1000; 
    return window.setTimeout(function() {
      return __method.apply(__method, args);
    }, timeout);
  },
  
  wrap: function(wrapper) {
    var __method = this;
    return function() {
      return wrapper.apply(this, [Utils.func.bind(__method, this)].concat([].slice.call(arguments))); 
    }
  },
  
  methodize: function() {
    if (this._methodized) return this._methodized;
    var __method = this;
    return this._methodized = function() {
      return __method.apply(null, [this].concat([].slice.call(arguments)));
    };
  }
},

//xxx -> utils.func.xxx(fn, ...)
function(v, k) {
	Utils.func[k] = function() {
		var args = [].slice.call(arguments);
		var fn = args.shift();
		return v.apply(fn, args);
	};
});
 
 
/**
 * utils.Class
 * taken from Prototype
 */
Utils.Class = {
  create: function(parent, properties) {
    if (arguments.length == 1 && typeof parent != 'function')
      properties = parent, parent = null;
    
    function klass() {
      this.initialize.apply(this, arguments);
    }
  
    klass.superclass = parent;
    klass.subclasses = [];
  
    if (parent) {
      var subclass = function() { };
      subclass.prototype = parent.prototype;
      klass.prototype = new subclass;
      parent.subclasses.push(klass);
    }

    if (properties) Utils.Class.extend(klass, properties);
    if (!klass.prototype.initialize)
      klass.prototype.initialize = function() {};
      
    klass.prototype.constructor = klass;

    return klass;
  },
  
  extend: function(destination, source) {
    var ancestor = destination.superclass && destination.superclass.prototype;

    for (var property in source) {
      var value = source[property];
      if (ancestor && typeof value == 'function' &&
          Utils.func.argumentNames(value)[0] == "$super") {
        var method = value, value = Utils.extend(
		Utils.func.wrap((function(m) { 
          return function() { return ancestor[m].apply(this, arguments); };
        })(property), method), {
          valueOf:  function() { return method; },
          toString: function() { return method.toString(); }  
        });
      }
      destination.prototype[property] = value;
    }
    
    return destination;
  }
};

/**
 * utils.dom
 */
Utils.dom = {
	/**
	 * http://mg.to/2006/02/27/easy-dom-creation-for-jquery-and-prototype#comment-176
	 * bulid(
     * 'table', { 'class':"MyTable" }, [
     * 'tbody', {}, [
     *    'tr', {}, [
     *       'td', { 'class':"MyTableCol1" }, [ "howdy" ],
     *       'td', { 'class':"MyTableCol2" }, [
     *          "Link:", 'a', { 'class':"MyLink", 'href':"http://www.example.com" }, ["example.com"] ] ] ] ]
     * )
	 */
	 //FIXME
	build: function(document) {
	   var ret = [], a = [].slice.call(arguments, 1), e, o, i=0, j;
	      a = a[0] instanceof Array ? a[0] : a;
	      for (; i<a.length; i++) {
	      if (a[i+1] instanceof Object) {
	         e = ret[ret.length] = document.createElement(a[i]);
	         for (j in a[++i]) { e.setAttribute(j, a[i][j]); }
	         if (a[i+1] instanceof Array) {
	            o = arguments.callee(document, a[++i]);
	            for (j=0; j<o.length; j++) { e.appendChild(o[j]); }
	         }
	      } else { ret[ret.length] = document.createTextNode(a[i]); }
	   }
	   return ret;
	},
	
	build2: function() {
	   var ret = [], a = arguments, e, o, i=0, j;
	      a = a[0] instanceof Array ? a[0] : a;
	      for (; i<a.length; i++) {
	      if (a[i+1] instanceof Object) {
	         e = ret[ret.length] = document.createElement(a[i]);
	         for (j in a[++i]) { e.setAttribute(j, a[i][j]); }
	         if (a[i+1] instanceof Array) {
	            o = arguments.callee(a[++i]);
	            for (j=0; j<o.length; j++) { e.appendChild(o[j]); }
	         }
	      } else { ret[ret.length] = document.createTextNode(a[i]); }
	   }
	   return ret;
	},
	
	buildOne: function(tag, attrs, children) {
		return this.build2(tag, attrs, children)[0];
	},
	
	/**
	 * Whether a is a child node of b
	 */
	isChildOf: function(a, b) {
		var p = a;
		while (p && (p = p.parentNode))
			if (p == b) return true;
		return false;
	},
	
	HTML_addCSS: function(doc, cssText, id) {
//		var style = doc.defaultView.s = this.build( doc,
//			'style', {type: 'css/text'}, [cssText]
//		);
		
		var style = doc.createElement('style');
		style.setAttribute('type', 'text/css');
		if (id) style.setAttribute('id', id);
		style.appendChild(doc.createTextNode(cssText));
		
		(doc.getElementsByTagName('head')[0] || doc.body)
		.appendChild(style);
		//doc.getElementsByTagName('head')[0].appendChild(doc.createTextNode(cssText));
	},
	HTML_runScript: function(doc, szScript) {
		if(!doc.getElementById('diigotb-ad-script')){
			var script = doc.createElement('script');
			script.setAttribute('type', 'text/javascript');
			script.setAttribute('id', 'diigotb-ad-script');
			script.innerHTML=szScript;
			
			(doc.getElementsByTagName('head')[0] || doc.body)
			.appendChild(script);
		}else{
			var unsafeWin = doc.defaultView.wrappedJSObject || doc.defaultView;
			unsafeWin.insertDiigoContent(); 
		}
	},
	
	HTML_addScript: function(doc, src) {
		var script = doc.createElement('script');
		script.setAttribute('type', 'text/javascript');
		script.setAttribute('src', src);
		
		(doc.getElementsByTagName('head')[0] || doc.body)
		.appendChild(script);
	},
	
	HTML_addDiv: function(doc, property) {
		var div = doc.createElement('div');
		for(n in property){
			div.setAttribute(n, property[n]);
		}
		
		doc.body.appendChild(div);
	},
	
	once: function(ele, eventType, fn) {
		function f() {
			ele.removeEventListener(eventType, f, false);
			fn();
		};
		
		ele.addEventListener(eventType, f, false);
	},
	
	/**
	 * Gets xml source for an element
	 * XML Serialization
	 * 
	 * taken from firebug 1.2 lib.js
	 */
	getElementXML: function(element) {
		var z = this;
	    function toXML(elt)
	    {
	        if (elt.nodeType == 1)
	        {
	            xml.push('<', elt.localName.toLowerCase());
	
	            for (var i = 0; i < elt.attributes.length; ++i)
	            {
	                var attr = elt.attributes[i];
	
	                // Hide attributes set by Firebug
	                if (attr.localName.indexOf("firebug-") == 0)
	                    continue;
	
	                xml.push(' ', attr.localName, '=', z.escapeHTMLAttribute(attr.nodeValue));
	            }
	
	            if (elt.firstChild)
	            {
	                xml.push('>');
	
	                for (var child = elt.firstChild; child; child = child.nextSibling)
	                    toXML(child);
	
	                xml.push('</', elt.localName.toLowerCase(), '>');
	            }
	            else
	                xml.push('/>');
	        }
	        else if (elt.nodeType == 3)
	            xml.push(elt.nodeValue);
	        else if (elt.nodeType == 4)
	            xml.push('<![CDATA[', elt.nodeValue, ']]>');
	        else if (elt.nodeType == 8)
	            xml.push('<!--', elt.nodeValue, '-->');
	    }
	
	    var xml = [];
	    toXML(element);
	    return xml.join("");
	},
	
	/**
	 * Taken from firebug 1.2 lib.js
	 */
	escapeHTMLAttribute: function (value)
	{
	    function replaceChars(ch)
	    {
	        switch (ch)
	        {
	            case "&":
	                return "&amp;";
	            case "'":
	                return apos;
	            case '"':
	                return quot;
	        }
	        return "?";
	    };
	    var apos = "&#39;", quot = "&quot;", around = '"';
	    if( value.indexOf('"') == -1 ) {
	        quot = '"';
	        apos = "'";
	    } else if( value.indexOf("'") == -1 ) {
	        quot = '"';
	        around = "'";
	    }
	    return around + (String(value).replace(/[&'"]/g, replaceChars)) + around;
	},
	
	/**
	 * Taken from firebug 1.2 lib.js
	 */
	escapeHTML: function (value)
	{
	    function replaceChars(ch)
	    {
	        switch (ch)
	        {
	            case "<":
	                return "&lt;";
	            case ">":
	                return "&gt;";
	            case "&":
	                return "&amp;";
	            case "'":
	                return "&#39;";
	            case '"':
	                return "&quot;";
	        }
	        return "?";
	    };
	    return String(value).replace(/[<>&"']/g, replaceChars);
	}
};

Utils.io = {
	readFile: function(file/*nsIFile*/) {
		var data = "";
		var fstream = Cc["@mozilla.org/network/file-input-stream;1"]
		                        .createInstance(Ci.nsIFileInputStream);
		var sstream = Cc["@mozilla.org/scriptableinputstream;1"]
		                        .createInstance(Ci.nsIScriptableInputStream);
		fstream.init(file, -1, 0, 0);
		sstream.init(fstream); 
		
		var str = sstream.read(4096);
		while (str.length > 0) {
		  data += str;
		  str = sstream.read(4096);
		}
		
		sstream.close();
		fstream.close();
		return data;
	},
	
	getLocalFileInterface: function(filePath) {
		try {
			var localFileInterface = Cc["@mozilla.org/file/local;1"].createInstance(local_common.myLocalFile);
			localFileInterface.initWithPath(filePath);
			return localFileInterface;
		} catch (ex) {
			return null;
		}
	},

  getSpecialFolder: function(aFolderType) {
    // http://lxr.mozilla.org/mozilla/source/xpcom/io/nsAppDirectoryServiceDefs.h
    // http://lxr.mozilla.org/mozilla/source/xpcom/io/nsDirectoryServiceDefs.h
    // Returns object not string path
		var directoryService = Cc['@mozilla.org/file/directory_service;1']
                             .getService(Ci.nsIProperties);

		var thisDirectory = directoryService.get(aFolderType, Ci.nsIFile);
		
		return thisDirectory;
  },

  getSpecialFolderPath: function(aFolder) {
		return this.getSpecialFolder(aFolder).path;
  }
};

Utils.cookie = {
	getCookie: function(domain, name) {
		var cmClass = "@mozilla.org/cookiemanager;1";
		var cms = Cc[cmClass].getService();
		var cm = cms.QueryInterface(Ci.nsICookieManager);
		var enumerator = cm.enumerator;
		var cIface = Ci.nsICookie;
		var domainRegex = new RegExp(domain.replace('\.', '\\.')+'$');
		while(enumerator.hasMoreElements()) {
			var nextCookie = enumerator.getNext().QueryInterface(cIface);
			if(nextCookie.host.match(domainRegex)) {
				if(nextCookie.name == name) {
					return nextCookie;
				}
			}
		}
		return null;
	},
	
	getAllCookies: function(domain) {
		var cookies = [];
		var cmClass = "@mozilla.org/cookiemanager;1";
		var cms = Cc[cmClass].getService();
		var cm = cms.QueryInterface(Ci.nsICookieManager);
		var enumerator = cm.enumerator;
		var cIface = Ci.nsICookie;
		var domainRegex = new RegExp(domain.replace('\.', '\\.')+'$');
		while(enumerator.hasMoreElements()) {
			var nextCookie = enumerator.getNext().QueryInterface(cIface);
			if(nextCookie.host.match(domainRegex)) {
				cookies.push(nextCookie);
			}
		}
		return cookies;
	}
};

Utils.canvas={
	rgbColor:{
		'capture-black':'#000',
		'capture-white':'#fff',
		'capture-gray':'#808080',
		'capture-red':'#ff0000',
		'capture-orange':'#ff9900',
		'capture-yellow':'#ffff00',
		'capture-blue':'#0000ff',
		'capture-cyan':'#00ffff',
		'capture-light-gray':'#c0c0c0',
		'capture-purple':'#9900ff',
		'capture-green':'#00ff00',
		'capture-pink':'#ff00ff'
	},
	cColor:'',
	setStyle:function(ctx,color,lineWeight,cap){
		if(color){
			this.cColor=color;
			ctx.strokeStyle=this.rgbColor[color];
			ctx.fillStyle=this.rgbColor[color];
		}
		if(lineWeight)
			ctx.lineWidth =lineWeight;
		if(cap)
			ctx.lineCap = cap;
	},
	drawRect:function(ctx,l,t,r,b){
		r-=l;
		b-=t;
		ctx.strokeRect(l,t,r,b); 
		ctx.save();
	},
	
    ellipse:function(ctx,aX, aY, aWidth, aHeight){
        var hB = (aWidth / 2) * .5522848,
            vB = (aHeight / 2) * .5522848,
            eX = aX + aWidth,
            eY = aY + aHeight,
            mX = aX + aWidth / 2,
            mY = aY + aHeight / 2;
        ctx.moveTo(aX, mY);
        ctx.bezierCurveTo(aX, mY - vB, mX - hB, aY, mX, aY);
        ctx.bezierCurveTo(mX + hB, aY, eX, mY - vB, eX, mY);
        ctx.bezierCurveTo(eX, mY + vB, mX + hB, eY, mX, eY);
        ctx.bezierCurveTo(mX - hB, eY, aX, mY + vB, aX, mY);
        ctx.closePath();
    },

	drawRound:function(ctx,l,t,r,b){
		ctx.beginPath();
		this.ellipse(ctx,l, t, r-l, b-t);
		ctx.stroke();

	},
	
	degToRad: function(degrees)
	{
	   return degrees/180*Math.PI;
	},
	
	drawArrow:function(ctx,l,t,r,b){
		var height=b-t;
		var width=r-l;
		var alpha=Math.atan(height/width);
		var headerLength=10;
		var angleDegree=15;
		var positive=width>0?-1:1;
		a1 = r + (headerLength * Math.cos(alpha + this.degToRad(angleDegree)))*positive;
		b1 = b + (headerLength * Math.sin(alpha + this.degToRad(angleDegree)))*positive;

		//final point is end of the second barb
		c1 = r + (headerLength * Math.cos(alpha - this.degToRad(angleDegree)))*positive;
		d1 = b + (headerLength * Math.sin(alpha - this.degToRad(angleDegree)))*positive;
		
		headerLength=25;
		angleDegree=30;
		a2 = r + (headerLength * Math.cos(alpha + this.degToRad(angleDegree)))*positive;
		b2 = b + (headerLength * Math.sin(alpha + this.degToRad(angleDegree)))*positive;

		//final point is end of the second barb
		c2 = r + (headerLength * Math.cos(alpha - this.degToRad(angleDegree)))*positive;
		d2 = b + (headerLength * Math.sin(alpha - this.degToRad(angleDegree)))*positive;
		
		ctx.beginPath();
		ctx.moveTo(l, t);
		ctx.lineTo(a1, b1);
		ctx.lineTo(a2, b2);
		ctx.lineTo(r, b);
		ctx.lineTo(c2, d2);
		ctx.lineTo(c1, d1);
		ctx.fill();

	},
	drawText:function(ctx,l,t,text,color){
		this.setStyle(ctx,color,1,null);
		ctx.font = "18px Helvetica,Arial,sans-serif";
		ctx.textBaseline = "bottom";
		
		textlist=text.split('\n');
		for(var i=0;i<textlist.length;i++){
			tl=textlist[i];
			if(tl&&tl.length>0){
				ctx.fillText(tl, l, t+22+22*i);
			}
		}
	},
	
	beginDraw:function(ctx,style,l,t,r,b,text,color){
		diigoUtil.dump('[canvas beginDraw]', color);
		this.setStyle(ctx,color,4,"round");
		if(style==1){//rect
			this.drawRect(ctx,l,t,r,b);
		}else if(style==2){
			this.drawRound(ctx,l,t,r,b);
		}else if(style==3){
			this.drawText(ctx,l,t,text,color);
		}else if(style==4){
			this.drawArrow(ctx,l,t,r,b);
		}
		return {
				type:style,
				left:l,
				top:t,
				right:r,
				bottom:b,
				color:this.cColor,//ctx.fillStyle,
				text:text
			};
	},
	reDrawOper:function(ctx,ctx2,offsetY,list,l,t,w,h){
		ctx2.clearRect(l,t-offsetY,w,h); 
		ctx.clearRect(0,0,w,h); 
		for(var i=0;i<list.length;i++){
			node=list[i];
			if(node){
				this.beginDraw(ctx,node.type,node.left,node.top,node.right,node.bottom,node.text,node.color);
			}
		}
	},
	drawBgImage:function(ctx,url,w,h){
		var img = new Image();  
		img.src = url;  
		img.onload = function(){  
		   
		     // create pattern  
			ctx.drawImage(img,w,h);
		   
		}  
	},
	
	saveTempFile:function(canvas,ct,left,top,w,h){
		canvas.style.width = w + "px";  
		canvas.style.height = h + "px";  
		canvas.width = w;  
		canvas.height = h;  
		
		var ctx=canvas.getContext("2d");
		ctx.clearRect(0, 0, w, h);
		ctx.save();
		ctx.scale(1, 1);

		ctx.drawWindow(ct, left, top, w, h, "rgb(255,255,255)");
		ctx.restore();
		var dataUrl = canvas.toDataURL("image/png");
		//alert(dataUrl);
	          		
		var nsIoService = Components.classes["@mozilla.org/network/io-service;1"].
							getService(Components.interfaces.nsIIOService);
		var channel = nsIoService.newChannelFromURI(nsIoService.newURI(dataUrl, null, null));
		var channsteam = channel.open();
		
		var binaryInputStream = Components.classes["@mozilla.org/binaryinputstream;1"].
		createInstance(Components.interfaces.nsIBinaryInputStream);
		binaryInputStream.setInputStream(channsteam);
		
		var numBytes = binaryInputStream.available();
		var bytes = binaryInputStream.readBytes(numBytes);
		
		return {v:bytes,n:numBytes,url:dataUrl};
	}
};

/**
 * http://developer.mozilla.org/en/docs/Code_snippets:Tabbed_browser
 */
Utils.firefox = {
	APP_ID_FIREFOX: '{ec8030f7-c20a-464f-9b0e-13a3a9e97384}',
	APP_ID_FLOCK: '{a463f10c-3994-11da-9945-000d60ca027b}',
	
	/**
	 * Firefox 3 or later
	 * http://developer.mozilla.org/en/docs/Using_nsIXULAppInfo
	 */

	ff4OrAbove: function() {
		try{
		var appInfo = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
		var vc = Cc['@mozilla.org/xpcom/version-comparator;1'].getService(Ci.nsIVersionComparator);
		
		var appID = appInfo.ID;
		var version = appInfo.version;
		if (appID == this.APP_ID_FIREFOX) {
			return vc.compare(version, '4.0') >= 0;
		}
		}catch(e){
			return false;
		}
	},
	
	ff18OrAbove: function() {
		try{
		var appInfo = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
		var vc = Cc['@mozilla.org/xpcom/version-comparator;1'].getService(Ci.nsIVersionComparator);
		
		var appID = appInfo.ID;
		var version = appInfo.version;
		if (appID == this.APP_ID_FIREFOX) {
			return vc.compare(version, '18.0') >= 0;
		}
		}catch(e){
			return false;
		}
	},
	
	ff31beta: function() {
		var appInfo = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
		var vc = Cc['@mozilla.org/xpcom/version-comparator;1'].getService(Ci.nsIVersionComparator);
		
		var appID = appInfo.ID;
		var version = appInfo.version;
		if (appID == this.APP_ID_FIREFOX) {
			return vc.compare(version, '3.1b2') >= 0;
		//} else if (appID == this.APP_ID_FLOCK) {
		//	return vc.compare(version, '2.0a') >= 0;
		} else {
			//compare the platform id
			return vc.compare(appInfo.platformVersion, '1.9.1b2') >= 0;
		}
	},	
	isApp: function(what) {
		var appInfo = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
		return appInfo.ID == {firefox: this.APP_ID_FIREFOX, flock: this.APP_ID_FLOCK}[what];
	},
	
//	isAppFlock: function() {this.isApp('flock');},
	
	//the OS platform
	isOS: function(what) {
		var p = window.navigator.platform;
		if (what == 'win' || what == 'windows') {
			return /^win/i.test(p);
		} else if (what == 'mac') {
			return /^mac/i.test(p);
		} else if (what == 'linux') {
			return /^linux/i.test(p);
		}
	},
	
	makeUri: function(spec) {
  	var ios = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
  	return ios.newURI(spec, null, null);
  },
	
	//http://developer.mozilla.org/en/docs/Code_snippets:Post_data_to_window
	post: function(url, referrer, dataString, useCurrentTab, reuseTab) {
		// POST method requests must wrap the encoded text in a MIME
		// stream
		var stringStream = Cc["@mozilla.org/io/string-input-stream;1"].
		                   createInstance(Ci.nsIStringInputStream);
		if ("data" in stringStream) // Gecko 1.9 or newer
		  stringStream.data = dataString;
		else // 1.8 or older
		  stringStream.setData(dataString, dataString.length);
		
		var postData = Cc["@mozilla.org/network/mime-input-stream;1"].
		               createInstance(Ci.nsIMIMEInputStream);
		postData.addHeader("Content-Type", "application/x-www-form-urlencoded");
		postData.addContentLength = true;
		postData.setData(stringStream);
		
		if (useCurrentTab)
			loadURI(url, referrer || null, postData);
		else if(reuseTab) {
			var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
	                     .getService(Ci.nsIWindowMediator);
		  var browserEnumerator = wm.getEnumerator("navigator:browser");
		
		  // Check each browser instance for our URL
		  var found = false;
		  while (!found && browserEnumerator.hasMoreElements()) {
		    var browserInstance = browserEnumerator.getNext().getBrowser();
		
		    // Check each tab of this browser instance
		    var numTabs = browserInstance.tabContainer.childNodes.length;
		    for(var index=0; index<numTabs; index++) {
		      var currentBrowser = browserInstance.getBrowserAtIndex(index);
		      if (url == currentBrowser.currentURI.spec) {
		
		        // The URL is already opened. Select this tab.
		        browserInstance.selectedTab = browserInstance.tabContainer.childNodes[index];
		
		        // Focus *this* browser
		        browserInstance.focus();
		        found = true;
						loadURI(url, null, postData, false);
		        break;
		      }
		    }
		  }
		
		  // Our URL isn't open. Open it now.
		  if (!found) {
		    var recentWindow = wm.getMostRecentWindow("navigator:browser");
		    if (recentWindow) {
		      // Use an existing browser window
		      recentWindow.delayedOpenTab(url, referrer || null, null, postData, null);
		    }
		    else {
		      // No browser windows are open, so open a new one.
		      //doesn't work
					//window.open('about:blank').delayedOpenTab(url, referrer || null, null, postData, null);
		    }
		  }
		} else {
			delayedOpenTab(url, referrer || null, null, postData, null);
		}
	},
	
	openTab: function(url) {
		return this.openAndReuseOneTabPerURL(url, true);
	},
	
	closeTab: function(index) {
		var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
			           .getService(Ci.nsIWindowMediator);

		var browserEnumerator = wm.getEnumerator("navigator:browser");
		
		var browserInstance = browserEnumerator.getNext().getBrowser(); 
		
		browserInstance.removeCurrentTab();

	},
	
	loadTwitterinTab: function(url,callback) {
		var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
			           .getService(Ci.nsIWindowMediator);

		var browserEnumerator = wm.getEnumerator("navigator:browser");
		
		var browserInstance = browserEnumerator.getNext().getBrowser(); 
			  
			    // Create tab
		var newTab = browserInstance.addTab(url);
		newTab.setAttribute("myextension-myattribute", "xyz");
					  
			    // Focus tab
		browserInstance.selectedTab = newTab;
			    
			    // Focus *this* browser in case another one is currently focused
		browserInstance.focus();
		if(callback){
			callback(browserInstance.getBrowserForTab(newTab));
		}
	},
	
	getMainWindow: function() {
		return window.QueryInterface(Ci.nsIInterfaceRequestor)
                   .getInterface(Ci.nsIWebNavigation)
                   .QueryInterface(Ci.nsIDocShellTreeItem)
                   .rootTreeItem
                   .QueryInterface(Ci.nsIInterfaceRequestor)
                   .getInterface(Ci.nsIDOMWindow);
	},
	
	/**
	 * http://developer.mozilla.org/en/docs/Code_snippets:Windows
	 */
	getMostRecentWindow: function() {
		var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Ci.nsIWindowMediator);
		var newWindow = wm.getMostRecentWindow("navigator:browser");
		return newWindow;
	},
	
	/**
	 * 
	 * @param {Object} url
	 * @param {Boolean} reloadExistingTab when an existing tab if found, whether to reload it 
	 */
	openAndReuseOneTabPerURL: function(url, reloadExistingTab) {
	  var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
	                     .getService(Ci.nsIWindowMediator);
	  var browserEnumerator = wm.getEnumerator("navigator:browser");
	
	  // Check each browser instance for our URL
	  var found = false;
	  while (!found && browserEnumerator.hasMoreElements()) {
	    var browserInstance = browserEnumerator.getNext().getBrowser();
	
	    // Check each tab of this browser instance
	    var numTabs = browserInstance.tabContainer.childNodes.length;
	    for(var index=0; index<numTabs; index++) {
	      var currentBrowser = browserInstance.getBrowserAtIndex(index);
	      if (url == currentBrowser.currentURI.spec) {
	
	        // The URL is already opened. Select this tab.
	        browserInstance.selectedTab = browserInstance.tabContainer.childNodes[index];
	
	        // Focus *this* browser
	        browserInstance.focus();
					if (reloadExistingTab) {
						browserInstance.reload();
					}
	        found = true;
	        break;
	      }
	    }
	  }
	
	  // Our URL isn't open. Open it now.
	  if (!found) {
	    var recentWindow = wm.getMostRecentWindow("navigator:browser");
	    if (recentWindow) {
	      // Use an existing browser window
	      recentWindow.delayedOpenTab(url, null, null, null, null);
	    }
	    else {
	      // No browser windows are open, so open a new one.
	      window.open(url);
	    }
	  }
	},
	
	openAndReuseOneTabPerAttribute: function(attrName, url) {
	  var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
	           .getService(Ci.nsIWindowMediator);
	  for (var found = false, index = 0, browserInstance = wm.getEnumerator('navigator:browser').getNext().getBrowser();
	       index < browserInstance.mTabContainer.childNodes.length && !found;
	       index++) {
	
	    // Get the next tab
	    var currentTab = browserInstance.mTabContainer.childNodes[index];
	  
	    // Does this tab contain our custom attribute?
	    if (currentTab.hasAttribute(attrName)) {
	
	      // Yes--select and focus it.
	      browserInstance.selectedTab = currentTab;
	
	      // Focus *this* browser in case another one is currently focused
	      browserInstance.focus();
	      found = true;
	    }
	  }
	
	  if (!found) {
	    // Our tab isn't open. Open it now.
	    var browserEnumerator = wm.getEnumerator("navigator:browser");
	    var browserInstance = browserEnumerator.getNext().getBrowser(); 
	  
	    // Create tab
	    var newTab = browserInstance.addTab(url);
	    newTab.setAttribute("myextension-myattribute", "xyz");
	  
	    // Focus tab
	    browserInstance.selectedTab = newTab;
	    
	    // Focus *this* browser in case another one is currently focused
	    browserInstance.focus();
	  }
	},
	
	/**
	 * FIXME
	 */
	invokeCommand: function(cmd) {
		var controller = document.commandDispatcher.getControllerForCommand(cmd);
		if (controller && controller.isCommandEnabled(cmd)) {
		  controller.doCommand(cmd);
		}
	},
	
	/**
	 * Copies a string to clipboard
	 * http://developer.mozilla.org/en/docs/Using_the_Clipboard
	 */
	copyString: function(s) {
		var gClipboardHelper = Cc["@mozilla.org/widget/clipboardhelper;1"]
			.getService(Ci.nsIClipboardHelper); 
		gClipboardHelper.copyString(s);
	},
	
	/**
	 * from google toolbar
	 * the dummy hack: http://www.google.com/codesearch?hl=en&q=+lang:javascript+dummy+%3D+manager.datasource+show:oIPOiGvIf-8:luz-i9OoIHE:u8NpeZPknPM&sa=N&cd=8&ct=rc&cs_p=http://home.eng.iastate.edu/~wagner84/CyberNet/eFox.zip&cs_f=eFox/App/Firefox/GlobalProfile/extensions/%7B3112ca9c-de6d-4884-a869-9855de68056c%7D/components/gtbComponent.js#first
	 */
	uninstallExtension: function(extId) {
		var manager = Cc["@mozilla.org/extensions/manager;1"]
				.getService(Ci.nsIExtensionManager), dummy = manager.datasource; // HACK: force datasource initialization
		if ("uninstallExtension" in manager) {
			manager.uninstallExtension(extId)
		} else if ("uninstallItem" in manager) {
			manager.uninstallItem(extId)
		} else {
			return false
		}
		dummy = null; // REHACK: don't leak any resources
		return true
	},
	
	promptService: function() {
		return Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
	}
};

})();
