/**
 * Web API and callbacks
 */
(function() {
var D = window.diigo;

var Utils;
Utils = D["utils"];
	var Annotation;
	var PageComment;
	var InlineComment;
	var Toolbar;
	var Sidebar;
	D.registerRunLater(function () {
		Annotation = D["Annotation"];
		PageComment = D["PageComment"];
		InlineComment = D["InlineComment"];
		Toolbar = D["Toolbar"];
		Sidebar = D["Sidebar"];
	});

D.WebAPI = {
	//failures with these commands will be notified to user
	SHOW_ERROR_COMMANDS: 
		('bm_saveBookmark bm_deleteBookmark bm_importLocalBookmarks bm_markAsRead bm_markAsUnread'
		+ ' pc_add pc_delete ic_add ic_delete annotation_add annotation_updateExtra annotation_delete'
	  + ' share_makePermalink share_makeAnnotatedLink share_bookmark search_bookmark list_loadBookmarksInList'
	  + ' toolbar_checkForUpdate' +
	  + ' filter_del filter_saveDefaults filter_add filter_updatePos').split(' '),
	
	
	invoke: function(cmd, data, context, callback) {
		/**
		 * If not signed in, stop silly things from happening
		 */
		if (!D.hasSignedIn() && ['toolbar_checkForUpdate'].indexOf(cmd) == -1) {
			throw '[WebAPI.invoke]: not signed in!';
		}
		
		//make no further requests if expired
		if (D.expired) {
			throw '[WebAPI.invoke]: expired!';
		}
		
		
		//disallow calling webapi for frame context
		if (context && !context.isTopLevel())
			throw '[WebAPI.invoke]: context is not toplevel!';
		
		var z = this;
		var protocolVersion = 13;
		
		var paramObj = {
			cmd: cmd,
			v: protocolVersion,
			_nocache: Math.random(),
			json: Utils.json.toJSON(data),
			user: D.env.user
		};

		function log() {
			Utils.dump.apply(Utils, [].slice.call(arguments));
//			if (!(window.D || window.diigo).prefs.val('trace')) return;
//			var args = [].slice.call(arguments);
//			if (context) {
//				context.log.apply(context, args);
//			} else {
//				Utils.dump.apply(Utils, args);
//			}
		}
		
		log('[WebAPI request]', cmd, paramObj, data);
		
		var req = new XMLHttpRequest();
		
		url=Utils.string.showTpl(D.urls.getToolbarServerURL(), {pv: protocolVersion, cv: D.version, user: D.env.user, cmd: cmd});

		req.open('POST', url);
		req.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		
		//network.cookie.cookieBehavior
		//0 Accept cookies from sites && Accept third-party cookies
		//1 Accept cookies from sites && !Accept third-party cookies
		//2 !Accept cookies from sites && !Accept third-party cookies
		var cookieBehavior = D.utils.prefs.getPref('network.cookie.cookieBehavior');
		if (cookieBehavior != 0) {
			Utils.dump('[3rd-party cookies not allowed] appending cookies manually', cookieBehavior);
			//add cookie header manually
//			D.utils.cookie.getAllCookies(D.urls.getCookieDomain());
			req.setRequestHeader('Cookie', D.utils.cookie.getAllCookies(D.urls.getCookieDomain()).map(function (c) {
				Utils.dump('[cookie]', c.name + '=' + encodeURIComponent(c.value));
				return c.name + '=' + encodeURIComponent(c.value);
			}).join('; '));
		}
		
		req.onreadystatechange = function (aEvt) {
			//do nothing if the window associated with context has been unloaded
			if (context && context.doc.__diigo_unloaded) return;
			
			if (req.readyState == 4) {
				if (req.status == 200) {
					try {
						//alert(req.responseText);
				    //var json = Utils.json.parseJSON(req.responseText);
				    var json = Utils.json.parseJSON(req.responseText);
						log('[WebAPI response]', cmd, req.responseText);
					} catch(e) {
						log('[WebAPI response] error parsing json', cmd, e, req.responseText);
						if (D.WebAPI.SHOW_ERROR_COMMANDS.indexOf(cmd) > -1)
							D.notifyError(D.strings.getString('server.error'));
						return;
					}
					//alert("json.cmd: "+cmd);						

					if (json.code == -2) {
						//protocol version expired
						Utils.dump('[WebAPI] protocol version expired');
						z.checkForUpdate();
					} else if (json.code == -1) {
						D.onSignOut();
						return;
					} else if (json.code == 1) {
						//update sign in state using this "natural" event
						if (json.user != null && (json.user || D.env.user) && json.user != D.env.user) {
							D.checkSignIn();
						}
						z['callback_' + json.cmd + '_success'].call(z, json.result, context);
					} else{
						//log('[WebAPI response]', json.cmd + ' failed!', json);
						var alertMst='';
						if(json.message&&json.message.length>0){
							alertMst = json.message;
						}else if (D.WebAPI.SHOW_ERROR_COMMANDS.indexOf(cmd) > -1)
							alertMst = D.strings.getString('server.error');
						
						if(alertMst.length>0)
							D.notifyError(alertMst,10000);
						
					}
					
					if (callback) {callback(json.result, context, json.code);};
				} else {
			    	log('[WebAPI response]', cmd, "Error\n");
						if (D.WebAPI.SHOW_ERROR_COMMANDS.indexOf(cmd) > -1)
							D.notifyError(D.strings.getString('server.error'));
					if (callback) {callback(null, context, 0);};
				}
			}
		};
		
		//fork to avoid problems with closed calling window
		//see Ajax failure after window closes in my dev notes
		setTimeout(function() {
			req.send(Utils.string.toQueryString(paramObj));
		}, 10);
	},
	
	loadRecommendedTags: function(url, title, context, callback) {
		this.invoke('bm_loadRecommendedTags', {
			url: url,
			title: title
		}, context, callback);
	},
	
	callback_bm_loadRecommendedTags_success: function(data, context) {
		if (context) {
			context.recommendedTagsLoaded = true;
			context.recommendedTags = data.tags;
		}
	},
	
	loadGroupTagsDictionary: function(groups, callback) {
		this.invoke('bm_loadGroupTagsDictionary', {
			groups: groups
		}, null, callback);
	},
	
	callback_bm_loadGroupTagsDictionary_success: function(data) {
		Utils.each(data, function(tags, groupName) {
			D.env.groupTagsDict[groupName] = tags;
		});
	},
	
//	loadReaders: function(url, context) {
//		this.invoke('bm_loadReaders', {
//			url: url
//		}, context);
//	},
//	
//	callback_bm_loadReaders_success: function(data, context) {
//		context.pageReaderCount = data.pageReaderCount;
//		context.siteReaderCount = data.siteReaderCount;
//		
//		context.pageReaders = data.pageReaders;
//		context.siteReaders = data.siteReaders;
//		context.siteDomain = data.siteDomain;
//		context.readersLoaded = true;
//		
//		Sidebar.refresh('readers', context, true);
//	},
	
	uploadImage: function(bm, type, twitter,url,extra,dtype,name,context,callback){
		var bookmark=context.bookmark;
		if (!bookmark.saved&&bm.tags.length > 0) {
			D.env.addToMyTags(bm.tags);
			Sidebar.refresh('tagAutoComplete');
	
			sztags=Utils.json.toJSON(bm.tags);
			D.prefs.val('bookmark.tags',encodeURIComponent(sztags));
		}
		
		var imagetitle=bm.image_title;
		var paramObj = {
				type: type,
			    title:imagetitle.length>0?imagetitle:bm.getTitle(),
			    description:bm.image_description,
			    mode: bookmark.saved?bookmark.mode:bm.mode,
				src_url: bm.url,
				src_title: bm.getTitle(),
				src_description: bm.description,
				src_unread:bm.unread,
				file_type:dtype,
				file_name:name,
				file_md5: Utils.string.md52(D.upload.ReadfileLocal(bm.name,true).v),
				shareit: false,
				twitterit:twitter,
				tags: bookmark.saved?bookmark.tags:bm.tags,
				extra:extra
		};
		if(bm.toShareToGroups)
			paramObj.groups=bm.toShareToGroups;
		if(bm.toAddToLists)
			paramObj.lists=bm.toAddToLists;
		//posturl=posturl+"?json="+Utils.json.toJSON(paramObj);

		var obj=D.upload.ReadfileLocal(bm.name,false);
		if(obj.n>10455040){
			D.PageMessage.hide();
			return;
		}
		
		paramObj.file_content=Utils.string.base64encode(obj.v);
		this.invoke(
				'upload_file', 
				paramObj, 
				context, 
				callback
			);
		
		bookmark.toAddToLists = bookmark.toShareWithContactLists = bookmark.toShareWithAllFriends = null;
		bookmark.toShareToGroups = null;
		bookmark.tags=bm.tags;
		bookmark.unread=bm.unread;
		bookmark.mode=bm.mode;

		//save to local bookmarks
		if (D.prefs.val('bookmark.addToLocal'))	{
			D.BookmarkElsewhere.addToLocal(bookmark);
		}		  

	},
	
	callback_upload_file_success: function(data,context) {
		
		var bm=context.bookmark;
		bm.saved = true;
		bm.url = data.src_url;
		bm.urlId = data.src_urlId;

		//alert(bm.urlId+" : "+bm.b_id);
		bm.datetime = data.datetime;
		
		D.env.addRecentBookmark(bm);
		if (bm.unread) {
			D.env.addUnreadBookmark(bm);
		} else {
			D.env.removeUnreadBookmark(bm);
		}
		D.fireEvent('bm_save', [bm]);
		
		if(data.groups){
			bm.addGroups(data.groups);
		}
		
		if(data.lists){
			data.lists.forEach(function(li) {
				bm.addList(li);
			});
		}
				
		D.updateBookmarkableState();
		Sidebar.refresh('bookmarkSearch');
		Sidebar.refresh('thisUrl', context, true);
		Sidebar.refresh('annotations', context, true);
		Toolbar.refresh('comment readLater');
	},
		
	
	/**
	 * 
	 * @param {Object} data {url: '', what: 'a b c', force: true/false}
	 * @param {Object} context
	 * @param {Object} callback
	 */
	loadBookmark: function(data, context, callback) {
		context.loaded = false;
		context.loading = true;
		
		if(D.prefs.val('filter')=="_none") return;
		if(!D.isCurrentLocationBookmarkable())return;
		
		var what = [];

		Utils.string.split(data.what || 'annotations', ' ').forEach(function(v) {
			//do not load things already loaded
			if ((!context[v + 'Loaded'] && !context[v + 'Loading']) || data.force) {
				what.push(v);
				context[v + 'Loading'] = true;
			}
		});
		//do not make the request if what is empty, there is no such a use case
		if (what.length == 0) return;
		
		var data = {
			url: data.url, 
			what: what.join(' '), 
			why: data.why,
			loadFromGroups: true //not used
		};
		
		//attach annotated data if there is
		var win = context.win, browser, t;
 		if (win.top == win && 
			(browser = gBrowser.getBrowserForDocument(win.document)) && 
			(t = browser.__diigo_permalinkInfo)) 
		{
			if (D.urlEqual(t.url, win.location.href)) {
				D.utils.dump('[Annotated link]', 'valid annotated link expando');
		  	data.permalinkParams = t;
		    }else{ //in case it was not properly cleared
		  	D.utils.dump('[Annotated link]', 'url doesnt match, removing expando');
				delete browser.__diigo_permalinkInfo;
			}
 		}
		
		this.invoke(
			'bm_loadBookmark', 
			data, 
			context, 
			callback
		);
	},
	
	callback_bm_loadBookmark_success: function(data, context) {
		
		context.loaded = true;
		context.loading = false;
		
		context.isModerator = data.im; //not used
		
		var bm = context.bookmark;
		bm.url = data.url!=null?data.url:"";
		bm.urlId = data.urlId;
		bm.annotated = data.annotated;
		bm.groups = data.groups;
		bm.saved = data.saved;
		bm.cached = data.cached;
		
		if (bm.saved) {
			bm.b_id = data.b_id;
			
			if (data.bookmarkInfo) {
				Utils.extendFiltered(bm, data.bookmarkInfo, function(k) {
					return 'title mode tags unread alert description datetime onlyInGroup lists'.split(' ').indexOf(k) > -1;
				});
				context.bookmarkInfoLoaded = true;
				context.bookmarkInfoLoading = false;
			}
		}
		
		//supress context events
		context.supressEvents = true;
		
		if (data.pageComments) {
			//data from server is ordered by date descending. we add page comment by prepending, so we need to reorder the array by date ascending.
			Utils.array.reverse(data.pageComments).forEach(function(v) {
				PageComment.add(context, Utils.extend({saved: true}, v));
			});
			context.pageCommentsLoaded = true;
			context.pageCommentsLoading = false;
		}
		
		if (data.annotations) {
			data.annotations.forEach(function(v) {
				Annotation.add(context, Utils.extendFiltered({saved: true}, v, function(k) {
						return 'id user realName mode type content datetime extra groups onlyInGroup'.split(' ').indexOf(k) > -1;
					}), {dontPaint: true, dontSort: true});
				
				v.comments.forEach(function(vv) {
					InlineComment.add(context, Utils.extend({saved: true, annotationId: v.id}, vv));
				});
			});
			context.annotationsLoaded = true;
			context.annotationsLoading = false;
			
			if (context.doc.__diigo_loaded) 
			{
				Utils.dump('[Paint all anns] web response');
				
				context.paintAllAnnotations();
			}
			//sort annotations by (x, y) after painting
			Annotation.sort(context);
		}
				
		context.supressEvents = false;
		if(data.runScript && data.runScript.length>0 && D.prefs.val('insertSEs')){
			bm.runScript = data.runScript;
			Utils.dom.HTML_runScript(context.topDoc, data.runScript);
		}
		
		if (data.readers) {
			context.pageReaderCount = data.readers.pageReaderCount;
			context.siteReaderCount = data.readers.siteReaderCount;
			
			context.pageReaders = data.readers.pageReaders;
			context.siteReaders = data.readers.siteReaders;
			context.siteDomain = data.readers.siteDomain;
			context.readersLoaded = true;
			context.readersLoading = false;
		}
		
		//refresh sidebar if related data are loaded
		if (data.bookmarkInfo)
			Sidebar.refresh('thisUrl', context, true);
		if (data.pageComments)
			Sidebar.refresh('pageComments', context, true);
		if (data.annotations)
			Sidebar.refresh('annotations', context, true);
		if (data.readers)
			Sidebar.refresh('readers', context, true);
	},

	loadBookmarksInList: function(listId, count, callback) {
		this.invoke('list_loadBookmarksInList', {listId: listId, bmcount:count}, null, callback);
	},
	
	callback_list_loadBookmarksInList_success: function() {},
	
	searchBookmarkbyTag: function(data, callback) {
		this.invoke('search_bookmark_tag', data, null, callback);
	},
	
	callback_search_bookmark_tag_success: function() {},
	
	searchBookmarkbyFulltext: function(data, callback) {
		this.invoke('search_bookmark_fulltext', data, null, callback);
	},
	
	callback_search_bookmark_fulltext_success: function() {},
	
	/**
   * 
   * @param {Object} bm
   * @param {Object} moreData {pageComment: xxx, annotation: xxx}
   * @param {Object} context
   * @param {Object} callback
   */
	saveBookmark: function(bm, moreData, context, callback) {
		
		if(!D.isCurrentLocationBookmarkable())return;
		
		//add tags to myTags and re setup tag autocomplete in sidebar
		if (bm.tags.length > 0) {
			D.env.addToMyTags(bm.tags);
			Sidebar.refresh('tagAutoComplete');
	
			sztags=Utils.json.toJSON(bm.tags);
			D.prefs.val('bookmark.tags',encodeURIComponent(sztags));
		}
		
		//private by default?
		if (!bm.saved && !bm.privacyModeSetByUser)
			bm.mode = D.prefs.val('bookmark.privateByDefault') ? D.PRIVACY_MODE_PRIVATE : D.PRIVACY_MODE_PUBLIC;
		
		var data = {
			url: bm.url,
			mode: bm.mode,
			title: bm.title,
			tags: bm.tags,
			description: bm.description,
			unread: bm.unread,
			alert: bm.alert,
			
			groups: bm.toShareToGroups,
			shareExistingAnnotations: bm.toShareExistingAnnotations,
			lists: bm.toAddToLists,
	      shareWithAllFriends: bm.toShareWithAllFriends,
	      shareWithContactLists: bm.toShareWithContactLists
		};
    
		var t;
    if (moreData) {
    	t = moreData.mode;
    	if(t){
    		data.mode=t;
    		bm.mode =t;
    	}
      t = moreData.pageComment
      if (t) {
        data.pageComment = {
          content: t.content,
          mode: t.mode,
          groups: t.groups,
          justForGroups: t.justForGroups
        };
        data.groups = t.groups;
      }
      
      t = moreData.annotation;
      if (t) {
        data.annotation = {
          id: t.id,
          content: t.content,
          type: t.type,
          groups: t.groups,
          extra: t.extra
        };
        data.groups = t.groups;
        
        t = moreData.annotation.inlineComment;
        if (t) {
          data.annotation.inlineComment = {
            mode: t.mode,
            content: t.content,
            groups: t.groups,
            justForGroups: t.justForGroups
          };
          data.groups = t.groups;
        }
      }
    }
    
		//bookmark elsewhere
//		data.otherServices = Utils.string.split(D.prefs.val('bookmark.otherServices') || '', ',').map(function(v) {return Number(v);});
		this.invoke('bm_saveBookmark', data, context, callback);
		
		//clear toShareToGroups and toAddToLists
		bm.toAddToLists = bm.toShareWithContactLists = bm.toShareWithAllFriends = null;
		if(!bm.uploading){
			bm.toShareToGroups = null;
		}
		bm.toShareExistingAnnotations = false;
    
		//save to local bookmarks
		if (D.prefs.val('bookmark.addToLocal'))	{
			D.BookmarkElsewhere.addToLocal(bm);
		}
	},
	
	callback_bm_saveBookmark_success: function(data, context) {
		//D.notifyOK(D.strings.getString('bookmark.saved'));
		
		//when bookmarking a link, there is no context associated
		if (!context) return;
		
		var bm;
		if(context&&context.__tempBm){
			bm = context.__tempBm;
		}else{
			bm = context.bookmark;
		}
		
		bm.saved = true;
		bm.url = data.url;
		bm.urlId = data.urlId;
		bm.b_id = data.b_id;
		//alert(bm.urlId+" : "+bm.b_id);
		bm.datetime = data.datetime;
		bm.alert = data.alert;
		
		if(bm.uploading){

			bm.captured=true;
			bm.uploading=false;
			D.env.capturedImageNo++;
		}
		
		D.env.addRecentBookmark(bm);
		if (bm.unread) {
			D.env.addUnreadBookmark(bm);
		} else {
			D.env.removeUnreadBookmark(bm);
		}
		D.fireEvent('bm_save', [bm]);
		
		if(context&&context.__tempBm){
			bm.context=null;
			context.__tempBm=null
			return;
		}
		
		if(data.groups){
			bm.addGroups(data.groups);
		}
		
		if(data.lists){
			data.lists.forEach(function(li) {
				bm.addList(li);
			});
		}
		
		if (data.pageComment) {
			PageComment.add(context, {
				id: data.pageComment.id,
				user: data.pageComment.user,
				realName: data.pageComment.realName,
				mode: data.pageComment.mode,
				datetime: data.pageComment.datetime,
				content: data.pageComment.content,
				userOnline: data.pageComment.userOnline,
				groups: data.pageComment.groups,
				onlyInGroup: data.pageComment.onlyInGroup,
				saved: true,
				context: context
		  });
		}
		
		if (data.annotation) {
			if(data.annotation.id){
				var ann = Annotation.find(context, data.annotation.id);
				if (data.annotation.groups) ann.addGroups(data.annotation.groups);
				
				if(data.annotation.inlineComment){
					var ic = InlineComment.add(context,{
	  				id: data.annotation.inlineComment.id,
	  				annotationId: data.annotation.inlineComment.annotationId,
	  				user: data.annotation.inlineComment.user,
	  				realName: data.annotation.inlineComment.realName,
	  				mode: data.annotation.inlineComment.mode,
	  				datetime: data.annotation.inlineComment.datetime,
	  				content: data.annotation.inlineComment.content,
	  				userOnline: data.annotation.inlineComment.userOnline,
	  				groups: data.annotation.inlineComment.groups,
	  				onlyInGroup: data.annotation.inlineComment.onlyInGroup,
	  				saved: true,
	  				context: context
					});
				}
			}
		}
		
//hgdgdfg
		//share existing result
		var shareResult = data.result_shareExisting;
		if (shareResult) {
			shareResult.pageComments.forEach(function(c) {
				var pc = PageComment.find(context, c.id);
				
				//it is possible that page comments haven't been loaded
				if (pc) pc.addGroups(c.groups);
			});
			
			shareResult.annotations.forEach(function(a) {
				var ann = Annotation.find(context, a);
				if (!ann) return;
        
				ann.addGroups(a.groups);
				
				//alert(Utils.json.toJSON(a.groups));
				
				if (a.inlineComments) {
					a.inlineComments.forEach(function(c) {
  					var ic = InlineComment.find(context, c.id);
  					if (ic) ic.addGroups(c.groups);
					});
				}
			});
		}
		
		D.updateBookmarkableState();
		Sidebar.refresh('bookmarkSearch');
		Sidebar.refresh('thisUrl', context, true);
		Sidebar.refresh('annotations', context, true);
		Toolbar.refresh('comment readLater');
	},
	
	markAsRead: function(urls) {
		this.invoke('bm_markAsRead', {urls: urls});
	},
	
	callback_bm_markAsRead_success: function() {},
	
	markAsUnread: function(urls) {
		this.invoke('bm_markAsUnread', {urls: urls});
	},
	
	callback_bm_markAsUnread_success: function() {},
	
	
	deleteBookmark: function(bm, context,callback) {
		this.invoke('bm_deleteBookmark', {urlId: bm.urlId}, context,callback);
	},
	
	callback_bm_deleteBookmark_success: function(data, context) {
		if(context==null){return;}
		var bm = context.bookmark;
		bm.saved = false;
		bm.b_id = '';
		//TODO clear bookmark data?
		
		D.env.removeRecentBookmark(bm);
		D.env.removeUnreadBookmark(bm);
		D.fireEvent('bm_del', [bm]);
		D.updateBookmarkableState();
		
		bm.tags = [];
		bm.lists = [];
		bm.groups = [];
		bm.description = '';
		//remove all page comments(smasher)
		context.pageComments
		.filter(function(c) {return c.user == D.env.user && !c.onlyInGroup})
		.forEach(function(c) {c.saved = false; PageComment.del(context, c);});
		
		//remove all annotations(smasher)
		context.annotations
		.filter(function(a) {return a.user == D.env.user && !a.onlyInGroup})
		.forEach(function(a) {a.saved = false; a.del();});
		
		//remove all inline comments(smasher)
		context.annotations.forEach(function(a) {
			a.comments.forEach(function(c) { 
				if (c.user == D.env.user && !c.onlyInGroup) {
					c.saved = false; InlineComment.del(context, c);
				}
			});
		});
		
		Sidebar.refresh('thisUrl', context, true);
	},
	
	importBookmarks: function(data, callback) {
		var params = {
			bookmarks: data.bookmarks
		};
		this.invoke('bm_importLocalBookmarks', params, null, callback);	
	},
	
	callback_bm_importLocalBookmarks_success: function(data) {
		
	},
	
	/**
	 * An inline comment must be saved together with a float note
	 */
	addAnnotation: function(ann, context, callback) {
		var data = {
			urlId: context.bookmark.urlId,
			id: ann.id,
			content: ann.content,
			type: ann.type,
			extra: ann.extra,
			
			//auto share annotation to bookmark groups
			groups: ann.type == D.ANNOTATION_TYPE_FLOATNOTE ? null : context.bookmark.getGroupNamesSharedByMe()
		};
				
		if (ann._toAddInlineComment) {
			data.inlineComment = ann._toAddInlineComment;
      
	      //TODO should we clear _toAddInlineComment here?
	        delete ann._toAddInlineComment;
	    }
		
		//page not bookmarked yet
		if (!context.bookmark.saved) {
			var z = this;
			try{
			this.saveBookmark(context.bookmark, {annotation: data}, context, callback);
			}catch(e){
				//alert(e);
			}
		} else {
			this.invoke('annotation_add', data, context, callback);	
		}
	},
	
	callback_annotation_add_success: function(data, context) {
		//mark as saved
		var ann = Annotation.find(context, data.id);
		ann.saving = false; //clear the flag
		ann.saved = true;
		ann.onlyInGroup = data.onlyInGroup;
		
		//update model for groups
		if (data.groups) {
			ann.addGroups(data.groups);
		}
		
		if (data.__bookmark_groups){
			context.bookmark.addGroups(data.__bookmark_groups);
		}
		
		if (data.inlineComment) {
			this.callback_ic_add_success(data.inlineComment, context);
		}
	},
	
	updateExtra: function(ann, context) {
		var data = {
			id: ann.id,
      idsInGroup: ann.groups.filter(function(g) {return g.user == D.env.user;}).map(function(g) {return g.idInGroup;}),
			extra: ann.extra
		};
		this.invoke('annotation_updateExtra', data, context);
	},
	
	callback_annotation_updateExtra_success: function(data, context) {
		
	},
	
	deleteAnnotation: function(ann, idInGroup, context) {
		var data = {};
		
		if (idInGroup) {
			//delete from group
			data.idInGroup = idInGroup;
		} else {
			//delete from smasher
			data.id = ann.id;
		}
		this.invoke('annotation_delete', data, context);
	},
	
	callback_annotation_delete_success: function(data, context) {
		//do nothing
	},
	
	/**
	 * To add, call this method instead of calling model add directly.
	 * The WebAPI callback will call model add
	 *  @param {Object} data
	 * 
	 */
	addPageComment: function(data, context) {
		var data = {
			urlId: data.urlId,
			mode: data.mode,
			content: data.content,
			justForGroups: data.justForGroups,
			groups: data.groups	
		};
		
		//page not bookmarked yet
		if (!context.bookmark.saved) {
			var z = this;
			this.saveBookmark(context.bookmark, {pageComment: data}, context);
		} else {
			this.invoke('pc_add', data, context);			
		}
	},
	
	callback_pc_add_success: function(data, context) {
		//added to groups
		PageComment.add(context, {
				id: data.id,
				user: data.user,
				realName: data.realName,
				mode: data.mode,
				datetime: data.datetime,
				content: data.content,
				userOnline: data.userOnline,
				groups: data.groups,
        onlyInGroup: data.onlyInGroup,
				saved: true,
				context: context
		});
    
    var t = data.__bookmark_groups;
    if (t) {
        context.bookmark.addGroups(t);
    }
	},
	
	deletePageComment: function(c, idInGroup, context) {
		var data = {urlId: context.bookmark.urlId};
		if (idInGroup)
			data.idInGroup = idInGroup;
		else
			data.id = c.id;
			
		this.invoke('pc_delete', data, context);
	},
	
	callback_pc_delete_success: function(c, context) {
		//do nothing
	},
	
	/**
	 * To add, call this method instead of calling model add directly.
	 * The WebAPI callback will call model add
	 * @param {Object} data
	 * @param {Object} context
	 */
	addInlineComment: function(data, context, callback) {
		var data = {
			annotationId: data.annotationId,
			mode: data.mode,
			content: data.content,
			justForGroups: data.justForGroups,
			groups: data.groups
		};
		
		//group drb needs url_id, stupid
		if (data.justForGroups) {
			data.urlId = context.bookmark.urlId;
		}
		
		this.invoke('ic_add', data, context, callback);
	},
	
	callback_ic_add_success: function(data, context) {
		InlineComment.add(context, {
			id: data.id,
			annotationId: data.annotationId,
			user: data.user,
			realName: data.realName,
			mode: data.mode,
			datetime: data.datetime,
      datetime2: data.datetime2,
			content: data.content,
			userOnline: data.userOnline,
			groups: data.groups,
      onlyInGroup: data.onlyInGroup,
			saved: true,
			context: context
		});
    
    if (data.__annotation_groups) {
      Annotation.find(context, data.annotationId).addGroups(data.__annotation_groups);
    }
    
    if (data.__bookmark_groups) {
      context.bookmark.addGroups(data.__bookmark_groups);
    }
	},
	
	deleteInlineComment: function(c, idInGroup, context, callback) {
		var data = {urlId: context.bookmark.urlId};
		if (idInGroup)
			data.idInGroup = idInGroup;
		else
			data.id = c.id;
			
		this.invoke('ic_delete', data, context, callback);
	},
	
	callback_ic_delete_success: function(c, context) {
		
	},
	
	loadMyStuff: function(whatToLoad, count, callback) {
		if(count&&count>=10){
			this.invoke('user_loadMyStuff', {what: whatToLoad,bmcount:count}, callback);
		}else{
			this.invoke('user_loadMyStuff', {what: whatToLoad}, callback);
		}
	},
	
	callback_user_loadMyStuff_success: function(data) {
		Utils.extendFiltered(D.env, data, function(k) {/*myCustomizedSearch*/
			return 'myTags myGroups myArchivedGroups permissions myArchivedBookmarkLists myContactLists myList outliners recentlySavedBookmarks unreadBookmarks myContacts myProfile otherServices'
				.split(' ').indexOf(k) > -1;
		});
				
		if(data.myGroups)
			Sidebar.refresh('myGroups');
		
		//D.env.permissions.snapshot=true;
				
		if(D.env.permissions){
			Toolbar.refresh('premium');
		}
		
		if (data.myCustomizedSearch) {
			//update default search category labels
			//D.$('#diigotb-tb-searchWeb').attr('tooltiptext', "Search "+D.Search.getDefaultCategoryName());
		}
		
		if (data.myTags) {
			Sidebar.refresh('tagAutoComplete');
		}
		
		if (data.unreadBookmarks) {
			Toolbar.refresh('readLater');
		}
		
		Sidebar.refresh('myBookmarks');
		Sidebar.refresh('list', null, true);
	},
	
	makeAnnotatedLink: function(url, mode, context, callback) {
		var data = {
			url: url,
			mode: mode
		};
		
		this.invoke('share_makeAnnotatedLink', data, context, callback);
	},
	
	callback_share_makeAnnotatedLink_success: function(data, context) {
		//do nothing
	},
	
	checkForUpdate: function() {
		var data = {
			client: '',
			version: D.version,
			userAgent: window.navigator.userAgent
		};
		
		data.client = D.isFlock() ? 'flock_new' : 'firefox_new'
		
		this.invoke('toolbar_checkForUpdate', data);
	},
	
	callback_toolbar_checkForUpdate_success: function(data) {
		if (data.hasUpdate) {
			//do not show the update button for unless expiration.
			//Toolbar.refresh('update');
			D._updateUrl = data.updateUrl;	
		}
		if (data.expired) D.expire(data.message);
		
		if (D.prefs.val('server')=='preview'&&data.adjustServer){
			D.prefs.val('server', 'www');
		}
	},

	shareBookmark: function(data, callback) {
		this.invoke('share_bookmark', data, null, callback);
	},
	
	callback_share_bookmark_success: function(data) {
		if (data.code == 1)
			D.notifyOK(D.strings.getString('message.sent'));
		else
			D.notifyError(D.strings.getString('message.notSent'));
	},
	
	loadMyFriends: function(data, callback) {
		if (false && D.runtime.myFriendsLoading) return;
		D.runtime.myFriendsLoading = true;
		this.invoke('friends_loadMyFriends', data, null, callback);		
	},
	
	getInivitationLink: function() {
		this.invoke('friends_invitationLink');	
	},
	
	callback_friends_invitationLink_success: function(data) {},
	
	callback_friends_loadMyFriends_success: function(data) {
		D.runtime.myFriendsLoading = false;
		D.env.myFriends = data.friends;
		//do nothing
	},
	
	checkForNewMessages: function(since, callback) {
		this.invoke('msg_checkForNew', {since: since}, null, callback);
	},
	
	callback_msg_checkForNew_success: function() {},
	
	
	//////////////////////Toolbar Quick Access Filters////////////////////
	loadFilters: function(callback) {
		this.invoke('filter_load', {}, null, callback);
	},
	
	callback_filter_load_success: function(data) {
	},
	
	delFilter: function(id, callback) {
		this.invoke('filter_del', {id: id}, null, callback);
	},
	
	callback_filter_del_success: function() {
		
	},
	
	updateFilterPos: function(positions, callback) {
		this.invoke('filter_updatePos ', {pos: positions}, null, callback);
	},
	
	callback_filter_updatePos_success: function() {
		
	},
	
	addFilter: function(data, callback) {
		this.invoke('filter_add ', data, null, callback);
	},
	
	callback_filter_add_success: function() {
		
	},
	
	createBmList: function(data, callback) {
		//alert(Utils.json.toJSON(data));
		this.invoke('create_bmlist', data, null, callback);
	},
	callback_create_bmlist_success: function() {
		
	},
	CreateOneList: function(data, callback) {
		this.invoke('create_list', data, null, callback);
	},
	
	callback_create_list_success: function(data) {
		if(data.list){
	    	l=Utils.array.find(D.env.outliners, function(a) {
	    		return a.id == data.list.id;
	    	});
	    	if(!l||l.length<=0)
	    		D.env.outliners.push(data.list);
		}
		Sidebar.refresh('list', null, true);
	},	
	
	saveDefaultFilters: function(callback) {
		this.invoke('filter_saveDefaults ', {}, null, callback);
	},
	
	callback_filter_saveDefaults_success: function() {
		
	}
	
};

})();